'use strict';

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// a638475a61cc567ad7e79fb1610a59e0cc9eb653

/* eslint-disable standard/no-callback-literal */

var extend = require('lodash/extend');
var each = require('lodash/each');
var once = require('lodash/once');

var _require = require('../../utils'),
    debug = _require.debug,
    ReactiveVar = _require.ReactiveVar,
    Tracker = _require.Tracker;

var _require2 = require('./accounts_common'),
    AccountsCommon = _require2.AccountsCommon;

/**
 * @summary Constructor for the `Accounts` object on the client.
 * @locus Client
 * @class AccountsClient
 * @extends AccountsCommon
 * @instancename accountsClient
 * @param {Object} options an object with fields:
 * @param {Object} options.connection Optional DDP connection to reuse.
 * @param {String} options.ddpUrl Optional URL for creating a new DDP connection.
 */


var AccountsClient = function (_AccountsCommon) {
  (0, _inherits3.default)(AccountsClient, _AccountsCommon);

  function AccountsClient(app) {
    (0, _classCallCheck3.default)(this, AccountsClient);

    var _this = (0, _possibleConstructorReturn3.default)(this, (AccountsClient.__proto__ || (0, _getPrototypeOf2.default)(AccountsClient)).call(this));

    _this.app = app;

    _this._loggingIn = new ReactiveVar(false);
    _this._loggingOut = new ReactiveVar(false);

    _this._pageLoadLoginCallbacks = [];
    _this._pageLoadLoginAttemptInfo = null;

    _this._autoLoginEnabled = true;
    _this._pageLoadLoginEnabled = true;

    // This is for .registerClientLoginFunction & .callLoginFunction.
    _this._loginFuncs = {};

    _this._userId = null;
    _this._userIdDeps = new Tracker.Dependency();
    return _this;
  }

  //
  /// CURRENT USER
  //

  // @override


  (0, _createClass3.default)(AccountsClient, [{
    key: 'userId',
    value: function userId() {
      var self = this;
      if (self._userIdDeps) {
        self._userIdDeps.depend();
      }
      return self._userId;
    }
  }, {
    key: 'setUserId',
    value: function setUserId(userId) {
      var self = this;
      if (self._userId === userId) {
        return;
      }
      self._userId = userId;
      if (self._userIdDeps) {
        self._userIdDeps.changed();
      }
    }

    // This is mostly just called within this file, but Meteor.loginWithPassword
    // also uses it to make loggingIn() be true during the beginPasswordExchange
    // method call too.

  }, {
    key: '_setLoggingIn',
    value: function _setLoggingIn(x) {
      this._loggingIn.set(x);
    }

    /**
     * @summary True if a login method (such as `Meteor.loginWithPassword`, `Meteor.loginWithFacebook`, or `Accounts.createUser`) is currently in progress. A reactive data source.
     * @locus Client
     */

  }, {
    key: 'loggingIn',
    value: function loggingIn() {
      return this._loggingIn.get();
    }

    /**
     * @summary True if a logout method (such as `Meteor.logout`) is currently in progress. A reactive data source.
     * @locus Client
     */

  }, {
    key: 'loggingOut',
    value: function loggingOut() {
      return this._loggingOut.get();
    }

    /**
     * @summary Register a new login function on the client. Intended for OAuth package authors. You can call the login function by using
     `Accounts.callLoginFunction` or `Accounts.callLoginFunction`.
     * @locus Client
     * @param {String} funcName The name of your login function. Used by `Accounts.callLoginFunction` and `Accounts.applyLoginFunction`.
     Should be the OAuth provider name accordingly.
     * @param {Function} func The actual function you want to call. Just write it in the manner of `loginWithFoo`.
     */

  }, {
    key: 'registerClientLoginFunction',
    value: function registerClientLoginFunction(funcName, func) {
      if (this._loginFuncs[funcName]) {
        throw new Error(funcName + ' has been defined already');
      }
      this._loginFuncs[funcName] = func;
    }

    /**
     * @summary Call a login function defined using `Accounts.registerClientLoginFunction`. Excluding the first argument, all remaining
     arguments are passed to the login function accordingly. Use `applyLoginFunction` if you want to pass in an arguments array that contains
     all arguments for the login function.
     * @locus Client
     * @param {String} funcName The name of the login function you wanted to call.
     */

  }, {
    key: 'callLoginFunction',
    value: function callLoginFunction(funcName) {
      if (!this._loginFuncs[funcName]) {
        throw new Error(funcName + ' was not defined');
      }

      for (var _len = arguments.length, funcArgs = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
        funcArgs[_key - 1] = arguments[_key];
      }

      return this._loginFuncs[funcName].apply(this, funcArgs);
    }

    /**
     * @summary Same as ``callLoginFunction` but accept an `arguments` which contains all arguments for the login
     function.
     * @locus Client
     * @param {String} funcName The name of the login function you wanted to call.
     * @param {Array} funcArgs The `arguments` for the login function.
     */

  }, {
    key: 'applyLoginFunction',
    value: function applyLoginFunction(funcName, funcArgs) {
      if (!this._loginFuncs[funcName]) {
        throw new Error(funcName + ' was not defined');
      }
      return this._loginFuncs[funcName].apply(this, funcArgs);
    }

    /**
     * @summary Log the user out.
     * @locus Client
     * @param {Function} [callback] Optional callback. Called with no arguments on success, or with a single `Error` argument on failure.
     */

  }, {
    key: 'logout',
    value: function () {
      var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(callback) {
        var self, userId, token;
        return _regenerator2.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                self = this;
                userId = self._storedUserId();
                token = self._storedLoginToken();

                self._loggingOut.set(true);
                _context2.next = 6;
                return this.app.applyMethod('logout', [userId, token], function () {
                  var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(error, result) {
                    return _regenerator2.default.wrap(function _callee$(_context) {
                      while (1) {
                        switch (_context.prev = _context.next) {
                          case 0:
                            self._loggingOut.set(false);

                            if (!error) {
                              _context.next = 5;
                              break;
                            }

                            callback && callback(error);
                            _context.next = 8;
                            break;

                          case 5:
                            _context.next = 7;
                            return self.makeClientLoggedOut();

                          case 7:
                            callback && callback();

                          case 8:
                          case 'end':
                            return _context.stop();
                        }
                      }
                    }, _callee, this);
                  }));

                  return function (_x2, _x3) {
                    return _ref2.apply(this, arguments);
                  };
                }());

              case 6:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function logout(_x) {
        return _ref.apply(this, arguments);
      }

      return logout;
    }()

    /**
     * @summary Log out other clients logged in as the current user, but does not log out the client that calls this function.
     * @locus Client
     * @param {Function} [callback] Optional callback. Called with no arguments on success, or with a single `Error` argument on failure.
     */

  }, {
    key: 'logoutOtherClients',
    value: function () {
      var _ref3 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee4(callback) {
        var self, userId, currentToken, app;
        return _regenerator2.default.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                self = this;
                userId = self._storedUserId();
                currentToken = self._storedLoginToken();
                app = this.app;
                _context4.next = 6;
                return app.applyMethod('getNewToken', [userId, currentToken], function () {
                  var _ref4 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3(err, result) {
                    var userId, currentToken;
                    return _regenerator2.default.wrap(function _callee3$(_context3) {
                      while (1) {
                        switch (_context3.prev = _context3.next) {
                          case 0:
                            if (err) {
                              _context3.next = 6;
                              break;
                            }

                            self._storeLoginToken(self.userId(), result.token, result.tokenExpires);

                            userId = self._storedUserId();
                            currentToken = self._storedLoginToken();
                            _context3.next = 6;
                            return app.applyMethod('removeOtherTokens', [userId, currentToken], function (err) {
                              callback && callback(err);
                            });

                          case 6:
                          case 'end':
                            return _context3.stop();
                        }
                      }
                    }, _callee3, this);
                  }));

                  return function (_x5, _x6) {
                    return _ref4.apply(this, arguments);
                  };
                }());

              case 6:
              case 'end':
                return _context4.stop();
            }
          }
        }, _callee4, this);
      }));

      function logoutOtherClients(_x4) {
        return _ref3.apply(this, arguments);
      }

      return logoutOtherClients;
    }()
  }]);
  return AccountsClient;
}(AccountsCommon);

var Ap = AccountsClient.prototype;

//
/// LOGIN METHODS
//

// Call a login method on the server.
//
// A login method is a method which on success calls `this.setUserId(id)` and
// `Accounts._setLoginToken` on the server and returns an object with fields
// 'id' (containing the user id), 'token' (containing a resume token), and
// optionally `tokenExpires`.
//
// This function takes care of:
//   - Updating the Meteor.loggingIn() reactive data source
//   - Calling the method in 'wait' mode
//   - On success, saving the resume token to localStorage
//   - On success, calling Accounts.connection.setUserId()
//   - Setting up an onReconnect handler which logs in with
//     the resume token
//
// Options:
// - methodName: The method to call (default 'login')
// - methodArguments: The arguments for the method
// - validateResult: If provided, will be called with the result of the
//                 method. If it throws, the client will not be logged in (and
//                 its error will be passed to the callback).
// - userCallback: Will be called with no arguments once the user is fully
//                 logged in, or with the error on error.
//
Ap.callLoginMethod = function () {
  var _ref5 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee8(options) {
    var self, app, loginCallbacks, loggedInAndDataReadyCallback;
    return _regenerator2.default.wrap(function _callee8$(_context8) {
      while (1) {
        switch (_context8.prev = _context8.next) {
          case 0:
            self = this;


            options = extend({
              methodName: 'login',
              methodArguments: [{}],
              _suppressLoggingIn: false
            }, options);

            // Set defaults for callback arguments to no-op functions; make sure we
            // override falsey values too.
            each(['validateResult', 'userCallback'], function (f) {
              if (!options[f]) {
                options[f] = function () {};
              }
            });

            // Prepare callbacks: user provided and onLogin/onLoginFailure hooks.
            app = this.app;
            loginCallbacks = once(function () {
              var _ref7 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee7(_ref6) {
                var error = _ref6.error,
                    loginDetails = _ref6.loginDetails;
                return _regenerator2.default.wrap(function _callee7$(_context7) {
                  while (1) {
                    switch (_context7.prev = _context7.next) {
                      case 0:
                        if (error) {
                          _context7.next = 7;
                          break;
                        }

                        _context7.next = 3;
                        return self._onLoginHook.eachAsync(function () {
                          var _ref8 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee5(callback) {
                            return _regenerator2.default.wrap(function _callee5$(_context5) {
                              while (1) {
                                switch (_context5.prev = _context5.next) {
                                  case 0:
                                    _context5.next = 2;
                                    return callback(loginDetails);

                                  case 2:
                                    return _context5.abrupt('return', true);

                                  case 3:
                                  case 'end':
                                    return _context5.stop();
                                }
                              }
                            }, _callee5, this);
                          }));

                          return function (_x9) {
                            return _ref8.apply(this, arguments);
                          };
                        }());

                      case 3:
                        _context7.next = 5;
                        return app.resetStore();

                      case 5:
                        _context7.next = 9;
                        break;

                      case 7:
                        _context7.next = 9;
                        return self._onLoginFailureHook.eachAsync(function () {
                          var _ref9 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee6(callback) {
                            return _regenerator2.default.wrap(function _callee6$(_context6) {
                              while (1) {
                                switch (_context6.prev = _context6.next) {
                                  case 0:
                                    _context6.next = 2;
                                    return callback({ error: error });

                                  case 2:
                                    return _context6.abrupt('return', true);

                                  case 3:
                                  case 'end':
                                    return _context6.stop();
                                }
                              }
                            }, _callee6, this);
                          }));

                          return function (_x10) {
                            return _ref9.apply(this, arguments);
                          };
                        }());

                      case 9:
                        options.userCallback(error, loginDetails);

                      case 10:
                      case 'end':
                        return _context7.stop();
                    }
                  }
                }, _callee7, this);
              }));

              return function (_x8) {
                return _ref7.apply(this, arguments);
              };
            }());

            // This callback is called once the local cache of the current-user
            // subscription (and all subscriptions, in fact) are guaranteed to be up to
            // date.

            loggedInAndDataReadyCallback = function loggedInAndDataReadyCallback(error, result) {
              // Note that we need to call this even if _suppressLoggingIn is true,
              // because it could be matching a _setLoggingIn(true) from a
              // half-completed pre-reconnect login method.
              self._setLoggingIn(false);
              if (error || !result) {
                error = error || new Error('No result from call to ' + options.methodName);
                loginCallbacks({ error: error });
                return;
              }
              try {
                options.validateResult(result);
              } catch (e) {
                loginCallbacks({ error: e });
                return;
              }

              // Make the client logged in. (The user data should already be loaded!)
              self.makeClientLoggedIn(result.id, result.token, result.tokenExpires);
              loginCallbacks({
                loginDetails: {
                  type: result.type
                }
              });
            };

            if (!options._suppressLoggingIn) {
              self._setLoggingIn(true);
            }
            _context8.next = 9;
            return this.app.applyMethod(options.methodName, options.methodArguments, loggedInAndDataReadyCallback);

          case 9:
          case 'end':
            return _context8.stop();
        }
      }
    }, _callee8, this);
  }));

  return function (_x7) {
    return _ref5.apply(this, arguments);
  };
}();

Ap.makeClientLoggedOut = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee10() {
  return _regenerator2.default.wrap(function _callee10$(_context10) {
    while (1) {
      switch (_context10.prev = _context10.next) {
        case 0:
          this._unstoreLoginToken();

          if (!this._userId) {
            _context10.next = 8;
            break;
          }

          _context10.next = 4;
          return this._onLogoutHook.eachAsync(function () {
            var _ref11 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee9(callback) {
              return _regenerator2.default.wrap(function _callee9$(_context9) {
                while (1) {
                  switch (_context9.prev = _context9.next) {
                    case 0:
                      _context9.next = 2;
                      return callback();

                    case 2:
                      return _context9.abrupt('return', true);

                    case 3:
                    case 'end':
                      return _context9.stop();
                  }
                }
              }, _callee9, this);
            }));

            return function (_x11) {
              return _ref11.apply(this, arguments);
            };
          }());

        case 4:
          _context10.next = 6;
          return this.app.resetCache();

        case 6:
          _context10.next = 8;
          return this.app.resetStore();

        case 8:

          this.setUserId(null);

        case 9:
        case 'end':
          return _context10.stop();
      }
    }
  }, _callee10, this);
}));

Ap.makeClientLoggedIn = function (userId, token, tokenExpires) {
  this._storeLoginToken(userId, token, tokenExpires);
  this.setUserId(userId);
};

// Some login services such as the redirect login flow or the resume
// login handler can log the user in at page load time.  The
// Meteor.loginWithX functions have a callback argument, but the
// callback function instance won't be in memory any longer if the
// page was reloaded.  The `onPageLoadLogin` function allows a
// callback to be registered for the case where the login was
// initiated in a previous VM, and we now have the result of the login
// attempt in a new VM.

// Register a callback to be called if we have information about a
// login attempt at page load time.  Call the callback immediately if
// we already have the page load login attempt info, otherwise stash
// the callback to be called if and when we do get the attempt info.
//
Ap.onPageLoadLogin = function (f) {
  if (this._pageLoadLoginAttemptInfo) {
    f(this._pageLoadLoginAttemptInfo);
  } else {
    this._pageLoadLoginCallbacks.push(f);
  }
};

// Receive the information about the login attempt at page load time.
// Call registered callbacks, and also record the info in case
// someone's callback hasn't been registered yet.
//
Ap._pageLoadLogin = function (attemptInfo) {
  if (this._pageLoadLoginAttemptInfo) {
    debug('Ignoring unexpected duplicate page load login attempt info');
    return;
  }

  each(this._pageLoadLoginCallbacks, function (callback) {
    callback(attemptInfo);
  });

  this._pageLoadLoginCallbacks = [];
  this._pageLoadLoginAttemptInfo = attemptInfo;
};

module.exports = {
  AccountsClient: AccountsClient
};