'use strict';

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// meteor/packages/accounts-base/accounts_common.js
// a638475a61cc567ad7e79fb1610a59e0cc9eb653

var each = require('lodash/each');
var keys = require('lodash/keys');
var includes = require('lodash/includes');

var _require = require('../../utils'),
    Hook = _require.Hook,
    makeErrorType = _require.makeErrorType;

/**
 * @summary Super-constructor for AccountsClient and AccountsServer.
 * @locus Anywhere
 * @class AccountsCommon
 * @instancename accountsClientOrServer
 * @param options {Object} an object with fields:
 * - connection {Object} Optional DDP connection to reuse.
 * - ddpUrl {String} Optional URL for creating a new DDP connection.
 */


var AccountsCommon = function () {
  function AccountsCommon(options) {
    (0, _classCallCheck3.default)(this, AccountsCommon);

    // Currently this is read directly by packages like accounts-password
    // and accounts-ui-unstyled.
    this._options = {};

    // Callback exceptions are printed with Meteor._debug and ignored.
    this._onLoginHook = new Hook({
      debugPrintExceptions: 'onLogin callback'
    });

    this._onLoginFailureHook = new Hook({
      debugPrintExceptions: 'onLoginFailure callback'
    });

    this._onLogoutHook = new Hook({
      debugPrintExceptions: 'onLogout callback'
    });
  }

  // Set up config for the accounts system. Call this on both the client
  // and the server.
  //
  // Note that this method gets overridden on AccountsServer.prototype, but
  // the overriding method calls the overridden method.
  //
  // XXX we should add some enforcement that this is called on both the
  // client and the server. Otherwise, a user can
  // 'forbidClientAccountCreation' only on the client and while it looks
  // like their app is secure, the server will still accept createUser
  // calls. https://github.com/meteor/meteor/issues/828
  //
  // @param options {Object} an object with fields:
  // - sendVerificationEmail {Boolean}
  //     Send email address verification emails to new users created from
  //     client signups.
  // - forbidClientAccountCreation {Boolean}
  //     Do not allow clients to create accounts directly.
  // - restrictCreationByEmailDomain {Function or String}
  //     Require created users to have an email matching the function or
  //     having the string as domain.
  // - loginExpirationInDays {Number}
  //     Number of days since login until a user is logged out (login token
  //     expires).
  // - passwordResetTokenExpirationInDays {Number}
  //     Number of days since password reset token creation until the
  //     token cannt be used any longer (password reset token expires).
  // - ambiguousErrorMessages {Boolean}
  //     Return ambiguous error messages from login failures to prevent
  //     user enumeration.
  // - bcryptRounds {Number}
  //     Allows override of number of bcrypt rounds (aka work factor) used
  //     to store passwords.

  /**
   * @summary Set global accounts options.
   * @locus Anywhere
   * @param {Object} options
   * @param {Boolean} options.sendVerificationEmail New users with an email address will receive an address verification email.
   * @param {Boolean} options.forbidClientAccountCreation Calls to [`createUser`](#accounts_createuser) from the client will be rejected. In addition, if you are using [accounts-ui](#accountsui), the "Create account" link will not be available.
   * @param {String | Function} options.restrictCreationByEmailDomain If set to a string, only allows new users if the domain part of their email address matches the string. If set to a function, only allows new users if the function returns true.  The function is passed the full email address of the proposed new user.  Works with password-based sign-in and external services that expose email addresses (Google, Facebook, GitHub). All existing users still can log in after enabling this option. Example: `Accounts.config({ restrictCreationByEmailDomain: 'school.edu' })`.
   * @param {Number} options.loginExpirationInDays The number of days from when a user logs in until their token expires and they are logged out. Defaults to 90. Set to `null` to disable login expiration.
   * @param {String} options.oauthSecretKey When using the `oauth-encryption` package, the 16 byte key using to encrypt sensitive account credentials in the database, encoded in base64.  This option may only be specifed on the server.  See packages/oauth-encryption/README.md for details.
   * @param {Number} options.passwordResetTokenExpirationInDays The number of days from when a link to reset password is sent until token expires and user can't reset password with the link anymore. Defaults to 3.
   * @param {Number} options.passwordEnrollTokenExpirationInDays The number of days from when a link to set inital password is sent until token expires and user can't set password with the link anymore. Defaults to 30.
   * @param {Boolean} options.ambiguousErrorMessages Return ambiguous error messages from login failures to prevent user enumeration. Defaults to false.
   */


  (0, _createClass3.default)(AccountsCommon, [{
    key: 'config',
    value: function config(options) {
      var self = this;

      // validate option keys
      var VALID_KEYS = ['sendVerificationEmail', 'forbidClientAccountCreation', 'passwordEnrollTokenExpirationInDays', 'restrictCreationByEmailDomain', 'loginExpirationInDays', 'passwordResetTokenExpirationInDays', 'ambiguousErrorMessages', 'bcryptRounds'];
      each(keys(options), function (key) {
        if (!includes(VALID_KEYS, key)) {
          throw new Error('Accounts.config: Invalid key: ' + key);
        }
      });

      // set values in Accounts._options
      each(VALID_KEYS, function (key) {
        if (key in options) {
          if (key in self._options) {
            throw new Error("Can't set `" + key + '` more than once');
          }
          self._options[key] = options[key];
        }
      });
    }

    /**
     * @summary Register a callback to be called after a login attempt succeeds.
     * @locus Anywhere
     * @param {Function} func The callback to be called when login is successful.
     *                        The callback receives a single object that
     *                        holds login details. This object contains the login
     *                        result type (password, resume, etc.) on both the
     *                        client and server. `onLogin` callbacks registered
     *                        on the server also receive extra data, such
     *                        as user details, connection information, etc.
     */

  }, {
    key: 'onLogin',
    value: function onLogin(func) {
      return this._onLoginHook.register(func);
    }

    /**
     * @summary Register a callback to be called after a login attempt fails.
     * @locus Anywhere
     * @param {Function} func The callback to be called after the login has failed.
     */

  }, {
    key: 'onLoginFailure',
    value: function onLoginFailure(func) {
      return this._onLoginFailureHook.register(func);
    }

    /**
     * @summary Register a callback to be called after a logout attempt succeeds.
     * @locus Anywhere
     * @param {Function} func The callback to be called when logout is successful.
     */

  }, {
    key: 'onLogout',
    value: function onLogout(func) {
      return this._onLogoutHook.register(func);
    }
  }, {
    key: '_getTokenLifetimeMs',
    value: function _getTokenLifetimeMs() {
      // When loginExpirationInDays is set to null, we'll use a really high
      // number of days (LOGIN_UNEXPIRABLE_TOKEN_DAYS) to simulate an
      // unexpiring token.
      var loginExpirationInDays = this._options.loginExpirationInDays === null ? LOGIN_UNEXPIRING_TOKEN_DAYS : this._options.loginExpirationInDays;
      return (loginExpirationInDays || DEFAULT_LOGIN_EXPIRATION_DAYS) * 24 * 60 * 60 * 1000;
    }
  }, {
    key: '_getPasswordResetTokenLifetimeMs',
    value: function _getPasswordResetTokenLifetimeMs() {
      return (this._options.passwordResetTokenExpirationInDays || DEFAULT_PASSWORD_RESET_TOKEN_EXPIRATION_DAYS) * 24 * 60 * 60 * 1000;
    }
  }, {
    key: '_getPasswordEnrollTokenLifetimeMs',
    value: function _getPasswordEnrollTokenLifetimeMs() {
      return (this._options.passwordEnrollTokenExpirationInDays || DEFAULT_PASSWORD_ENROLL_TOKEN_EXPIRATION_DAYS) * 24 * 60 * 60 * 1000;
    }
  }, {
    key: '_tokenExpiration',
    value: function _tokenExpiration(when) {
      // We pass when through the Date constructor for backwards compatibility
      // `when` used to be a number.
      return new Date(new Date(when).getTime() + this._getTokenLifetimeMs());
    }
  }, {
    key: '_tokenExpiresSoon',
    value: function _tokenExpiresSoon(when) {
      var minLifetimeMs = 0.1 * this._getTokenLifetimeMs();
      var minLifetimeCapMs = MIN_TOKEN_LIFETIME_CAP_SECS * 1000;
      if (minLifetimeMs > minLifetimeCapMs) {
        minLifetimeMs = minLifetimeCapMs;
      }
      return new Date() > new Date(when) - minLifetimeMs;
    }
  }]);
  return AccountsCommon;
}();

var Ap = AccountsCommon.prototype;

// how long (in days) until a login token expires
var DEFAULT_LOGIN_EXPIRATION_DAYS = 90;
// Expose for testing.
Ap.DEFAULT_LOGIN_EXPIRATION_DAYS = DEFAULT_LOGIN_EXPIRATION_DAYS;

// how long (in days) until reset password token expires
var DEFAULT_PASSWORD_RESET_TOKEN_EXPIRATION_DAYS = 3;
// how long (in days) until enrol password token expires
var DEFAULT_PASSWORD_ENROLL_TOKEN_EXPIRATION_DAYS = 30;
// Clients don't try to auto-login with a token that is going to expire within
// .1 * DEFAULT_LOGIN_EXPIRATION_DAYS, capped at MIN_TOKEN_LIFETIME_CAP_SECS.
// Tries to avoid abrupt disconnects from expiring tokens.
var MIN_TOKEN_LIFETIME_CAP_SECS = 3600; // one hour
// how often (in milliseconds) we check for expired tokens
var EXPIRE_TOKENS_INTERVAL_MS = 600 * 1000; // 10 minutes

// A large number of expiration days (approximately 100 years worth) that is
// used when creating unexpiring tokens.
var LOGIN_UNEXPIRING_TOKEN_DAYS = 365 * 100;
// Expose for testing.
Ap.LOGIN_UNEXPIRING_TOKEN_DAYS = LOGIN_UNEXPIRING_TOKEN_DAYS;

// Thrown when the user cancels the login process (eg, closes an oauth
// popup, declines retina scan, etc)
var lceName = 'Accounts.LoginCancelledError';
Ap.LoginCancelledError = makeErrorType(lceName, function (description) {
  this.message = description;
});
Ap.LoginCancelledError.prototype.name = lceName;

// This is used to transmit specific subclass errors over the wire. We should
// come up with a more generic way to do this (eg, with some sort of symbolic
// error code rather than a number).
Ap.LoginCancelledError.numericError = 0x8acdc2f;

module.exports = {
  AccountsCommon: AccountsCommon,
  EXPIRE_TOKENS_INTERVAL_MS: EXPIRE_TOKENS_INTERVAL_MS
};