'use strict';

var _require = require('../../../app/next-plus'),
    addAppModule = _require.addAppModule;

var _require2 = require('../../utils'),
    isServer = _require2.isServer,
    isClient = _require2.isClient;

var _require3 = require('./accounts_client'),
    AccountsClient = _require3.AccountsClient;

require('./localstorage_token.js');

addAppModule('defaultAccountsBaseModule', {
  setup: function setup(ctx) {
    if (isServer) {
      this.debug('Browser only module. accounts-base module not loaded.');
      return;
    }

    this.loggingIn = loggingIn.bind(this);
    this.loggingOut = loggingOut.bind(this);
    this.logout = logout.bind(this);
    this.logoutOtherClients = logoutOtherClients.bind(this);
    this.loginWithToken = loginWithToken.bind(this);
    this.getServiceConfig = getServiceConfig.bind(this);

    if (window.app) {
      window.app.accounts._pageLoadLoginEnabled = false;
      clearInterval(window.app.accounts._pollIntervalTimer);
    }

    if (this.accounts && this.accounts._initialized) {
      this.accounts._pageLoadLoginEnabled = false;
      clearInterval(this.accounts._pollIntervalTimer);
      return;
    }

    var app = this;
    app.accounts = new AccountsClient(app);
    var accounts = app.accounts;

    // !ssr: client page navigating. ex. click link redirect and render
    // ssr: server init and render, send to client
    if (ctx.ssr && ctx.currentUser) {
      accounts._pageLoadLoginEnabled = false;
      accounts._userId = ctx.currentUser._id;
    } else if (ctx.ssr && !ctx.currentUser) {
      // nothing
    } else if (!ctx.ssr && ctx.currentUser) {
      accounts._pageLoadLoginEnabled = false;
      accounts._userId = ctx.currentUser._id;
    } else if (!ctx.ssr && !ctx.currentUser) {
      accounts._pageLoadLoginEnabled = false;
    }
  },
  afterStartup: function afterStartup(ctx) {
    // initial: server init and client init
    // isClient && !ctx.initial: browser environment and rendering
    if (isClient && !ctx.initial) {
      this.accounts._initLocalStorage();
    }
  }
});

var loggingIn = function loggingIn() {
  var accounts = this.accounts;

  return accounts.loggingIn();
};
var loggingOut = function loggingOut() {
  var accounts = this.accounts;

  return accounts.loggingOut();
};
var logout = function logout(callback) {
  var accounts = this.accounts;

  return accounts.logout(callback);
};
var logoutOtherClients = function logoutOtherClients(callback) {
  var accounts = this.accounts;

  return accounts.logoutOtherClients(callback);
};
var loginWithToken = function loginWithToken(token, callback) {
  var accounts = this.accounts;

  return accounts.loginWithToken(token, callback);
};

var getServiceConfig = function getServiceConfig(serviceName) {
  var services = this.getSetting('services') || {};
  return services[serviceName] || null;
};