'use strict';

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var cookie = require('cookie');

var _require = require('../../utils'),
    debug = _require.debug,
    LocalStorage = _require.LocalStorage,
    USER_ID_KEY = _require.USER_ID_KEY,
    LOGIN_TOKEN_KEY = _require.LOGIN_TOKEN_KEY,
    LOGIN_TOKEN_EXPIRES_KEY = _require.LOGIN_TOKEN_EXPIRES_KEY,
    COOKIES_LOGIN_TOKEN_KEY = _require.COOKIES_LOGIN_TOKEN_KEY;

var _require2 = require('./accounts_client'),
    AccountsClient = _require2.AccountsClient;

var Ap = AccountsClient.prototype;

// This file deals with storing a login token and user id in the
// browser's localStorage facility. It polls local storage every few
// seconds to synchronize login state between multiple tabs in the same
// browser.

Ap.loginWithToken = function () {
  var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(token, callback) {
    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return this.callLoginMethod({
              methodArguments: [{
                resume: token
              }],
              userCallback: callback
            });

          case 2:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, this);
  }));

  return function (_x, _x2) {
    return _ref.apply(this, arguments);
  };
}();

// Semi-internal API. Call this function to re-enable auto login after
// if it was disabled at startup.
Ap._enableAutoLogin = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2() {
  return _regenerator2.default.wrap(function _callee2$(_context2) {
    while (1) {
      switch (_context2.prev = _context2.next) {
        case 0:
          this._autoLoginEnabled = true;
          _context2.next = 3;
          return this._pollStoredLoginToken();

        case 3:
        case 'end':
          return _context2.stop();
      }
    }
  }, _callee2, this);
}));

//
/// STORING
//

Ap._storeLoginToken = function (userId, token, tokenExpires) {
  LocalStorage.setItem(USER_ID_KEY, userId);
  LocalStorage.setItem(LOGIN_TOKEN_KEY, token);
  if (!tokenExpires) {
    tokenExpires = this._tokenExpiration(new Date());
  }
  LocalStorage.setItem(LOGIN_TOKEN_EXPIRES_KEY, tokenExpires);

  this._lastLoginTokenWhenPolled = token;

  document.cookie = cookie.serialize(COOKIES_LOGIN_TOKEN_KEY, token, {
    expires: new Date(this._storedLoginTokenExpires()),
    path: '/'
  });
};

Ap._unstoreLoginToken = function () {
  LocalStorage.removeItem(USER_ID_KEY);
  LocalStorage.removeItem(LOGIN_TOKEN_KEY);
  LocalStorage.removeItem(LOGIN_TOKEN_EXPIRES_KEY);

  this._lastLoginTokenWhenPolled = null;

  document.cookie = cookie.serialize(COOKIES_LOGIN_TOKEN_KEY, '', {
    maxAge: -1,
    path: '/'
  });
};

// This is private, but it is exported for now because it is used by a
// test in accounts-password.
//
Ap._storedLoginToken = function () {
  return LocalStorage.getItem(LOGIN_TOKEN_KEY);
};

Ap._storedLoginTokenExpires = function () {
  return LocalStorage.getItem(LOGIN_TOKEN_EXPIRES_KEY);
};

Ap._storedUserId = function () {
  return LocalStorage.getItem(USER_ID_KEY);
};

Ap._unstoreLoginTokenIfExpiresSoon = function () {
  var tokenExpires = this._storedLoginTokenExpires();
  if (tokenExpires && this._tokenExpiresSoon(new Date(tokenExpires))) {
    this._unstoreLoginToken();
  }
};

//
/// AUTO-LOGIN
//

Ap._initLocalStorage = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee5() {
  var self, token;
  return _regenerator2.default.wrap(function _callee5$(_context5) {
    while (1) {
      switch (_context5.prev = _context5.next) {
        case 0:
          self = this;


          self._unstoreLoginTokenIfExpiresSoon();
          token = self._storedLoginToken();

          if (!(self._autoLoginEnabled && self._pageLoadLoginEnabled)) {
            _context5.next = 7;
            break;
          }

          if (!token) {
            _context5.next = 7;
            break;
          }

          _context5.next = 7;
          return self.loginWithToken(token, function () {
            var _ref4 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3(err) {
              return _regenerator2.default.wrap(function _callee3$(_context3) {
                while (1) {
                  switch (_context3.prev = _context3.next) {
                    case 0:
                      if (!err) {
                        _context3.next = 4;
                        break;
                      }

                      debug('Error logging in with token: ' + err);
                      _context3.next = 4;
                      return self.makeClientLoggedOut();

                    case 4:

                      self._pageLoadLogin({
                        type: 'resume',
                        allowed: !err,
                        error: err,
                        methodName: 'login',
                        // XXX This is duplicate code with loginWithToken, but
                        // loginWithToken can also be called at other times besides
                        // page load.
                        methodArguments: [{ resume: token }]
                      });

                    case 5:
                    case 'end':
                      return _context3.stop();
                  }
                }
              }, _callee3, this);
            }));

            return function (_x3) {
              return _ref4.apply(this, arguments);
            };
          }());

        case 7:

          // Poll local storage every 3 seconds to login if someone logged in in
          // another tab
          self._lastLoginTokenWhenPolled = token;

          if (self._pollIntervalTimer) {
            // Unlikely that _initLocalStorage will be called more than once for
            // the same AccountsClient instance, but just in case...
            clearInterval(self._pollIntervalTimer);
          }

          self._pollIntervalTimer = setInterval((0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee4() {
            return _regenerator2.default.wrap(function _callee4$(_context4) {
              while (1) {
                switch (_context4.prev = _context4.next) {
                  case 0:
                    _context4.next = 2;
                    return self._pollStoredLoginToken();

                  case 2:
                  case 'end':
                    return _context4.stop();
                }
              }
            }, _callee4, this);
          })), 3000);

        case 10:
        case 'end':
          return _context5.stop();
      }
    }
  }, _callee5, this);
}));

Ap._pollStoredLoginToken = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee7() {
  var self, currentLoginToken, cookies, cookieLoginToken;
  return _regenerator2.default.wrap(function _callee7$(_context7) {
    while (1) {
      switch (_context7.prev = _context7.next) {
        case 0:
          self = this;

          if (self._autoLoginEnabled) {
            _context7.next = 3;
            break;
          }

          return _context7.abrupt('return');

        case 3:
          currentLoginToken = self._storedLoginToken();
          cookies = cookie.parse(document.cookie);
          cookieLoginToken = cookies[COOKIES_LOGIN_TOKEN_KEY];

          // != instead of !== just to make sure undefined and null are treated the same
          // eslint-disable-next-line

          if (!(self._lastLoginTokenWhenPolled != currentLoginToken)) {
            _context7.next = 14;
            break;
          }

          if (!(currentLoginToken && currentLoginToken === cookieLoginToken)) {
            _context7.next = 12;
            break;
          }

          _context7.next = 10;
          return self.loginWithToken(currentLoginToken, function () {
            var _ref7 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee6(err) {
              return _regenerator2.default.wrap(function _callee6$(_context6) {
                while (1) {
                  switch (_context6.prev = _context6.next) {
                    case 0:
                      if (!err) {
                        _context6.next = 3;
                        break;
                      }

                      _context6.next = 3;
                      return self.makeClientLoggedOut();

                    case 3:
                    case 'end':
                      return _context6.stop();
                  }
                }
              }, _callee6, this);
            }));

            return function (_x4) {
              return _ref7.apply(this, arguments);
            };
          }());

        case 10:
          _context7.next = 14;
          break;

        case 12:
          _context7.next = 14;
          return self.logout();

        case 14:

          self._lastLoginTokenWhenPolled = currentLoginToken;

        case 15:
        case 'end':
          return _context7.stop();
      }
    }
  }, _callee7, this);
}));