'use strict';

var extend = require('lodash/extend');

var _require = require('../../../app/next-plus'),
    addAppModule = _require.addAppModule;

var _require2 = require('../../utils'),
    isServer = _require2.isServer;

addAppModule('defaultAccountsGoogleModule', {
  setup: function setup(ctx) {
    if (isServer) {
      this.debug('Browser only module. accounts-google module not loaded.');
      return;
    }

    if (!this.getServiceConfig('google')) {
      this.debug('Service config \'google\' not found. accounts-google module not loaded.');
      return;
    }

    this.loginWithGoogle = loginWithGoogle.bind(this);

    if (this.accounts._initialized) return;

    var accounts = this.accounts;
    accounts.oauth.registerService('google');

    accounts.registerClientLoginFunction('google', function (options, callback) {
      if (!callback && typeof options === 'function') {
        callback = options;
        options = null;
      }

      if (typeof accounts._options.restrictCreationByEmailDomain === 'string') {
        options = extend({}, options || {});
        options.loginUrlParameters = extend({}, options.loginUrlParameters || {});
        options.loginUrlParameters.hd = accounts._options.restrictCreationByEmailDomain;
      }

      var credentialRequestCompleteCallback = accounts.oauth.credentialRequestCompleteHandler(callback);
      accounts.google.requestCredential(options, credentialRequestCompleteCallback);
    });
  }
});

var loginWithGoogle = function loginWithGoogle() {
  if (!this.getServiceConfig('google')) {
    throw new Error('Service \'google\' not loaded.');
  }
  return this.accounts.applyLoginFunction('google', arguments);
};