'use strict';

var pick = require('lodash/pick');

var _require = require('../../../app/next-plus'),
    addAppModule = _require.addAppModule;

var _require2 = require('../../utils'),
    AppError = _require2.AppError,
    isClient = _require2.isClient,
    isServer = _require2.isServer;

var convertError = function convertError(accounts, err) {
  if (err && err instanceof AppError && err.error === accounts.LoginCancelledError.numericError) {
    return new accounts.LoginCancelledError(err.reason);
  } else {
    return err;
  }
};

addAppModule('defaultAccountsOAuthModule', {
  setup: function setup(ctx) {
    if (isServer) {
      this.debug('Browser only module. accounts-oauth module not loaded.');
      return;
    }

    if (this.accounts._initialized) return;

    var app = this;
    var oauth = app.oauth;
    var accounts = app.accounts;

    require('./common')(accounts);

    accounts.oauth.tryLoginAfterPopupClosed = function (credentialToken, callback) {
      var credentialSecret = oauth._retrieveCredentialSecret(credentialToken) || null;
      if (credentialSecret === null && app.getSetting('ignoreAccountNullCredentialSecret')) {
        return;
      }
      accounts.callLoginMethod({
        methodArguments: [{
          oauth: {
            credentialToken: credentialToken,
            credentialSecret: credentialSecret
          }
        }],
        userCallback: callback && function (err) {
          callback(convertError(accounts, err));
        }
      });
    };

    accounts.oauth.credentialRequestCompleteHandler = function (callback) {
      return function (credentialTokenOrError) {
        if (credentialTokenOrError && credentialTokenOrError instanceof Error) {
          callback && callback(credentialTokenOrError);
        } else {
          accounts.oauth.tryLoginAfterPopupClosed(credentialTokenOrError, callback);
        }
      };
    };
  },
  afterStartup: function afterStartup(ctx) {
    if (isClient) {
      if (this.accounts._initialized) return;

      // get data after redirect
      var accounts = this.accounts;

      var data = this.oauth.getDataAfterRedirect();
      if (!data) {
        return;
      }

      var methodName = 'login';
      var methodArguments = [{ oauth: pick(data, 'credentialToken', 'credentialSecret') }];

      accounts.callLoginMethod({
        methodArguments: methodArguments,
        userCallback: function userCallback(err) {
          err = convertError(accounts, err);
          accounts._pageLoadLogin({
            type: data.loginService,
            allowed: !err,
            error: err,
            methodName: methodName,
            methodArguments: methodArguments
          });
        }
      });
    }
  }
});