'use strict';

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// meteor/packages/accounts-password/password_client.js
// 2afc4bfd535ae197663354f391b9d574a0701ecd

var clone = require('lodash/clone');

var _require = require('../../../app/next-plus'),
    addAppModule = _require.addAppModule;

var _require2 = require('../../utils'),
    SHA256 = _require2.SHA256,
    check = _require2.check,
    isServer = _require2.isServer,
    AppError = _require2.AppError;

// Used in the various functions below to handle errors consistently


function reportError(error, callback) {
  if (callback) {
    callback(error);
  } else {
    throw error;
  }
}

addAppModule('defaultAccountsPasswordModule', {
  setup: function setup() {
    if (isServer) {
      this.debug('Browser only module. accounts-password module not loaded.');
      return;
    }

    this.loginWithPassword = loginWithPassword.bind(this);

    if (this.accounts._initialized) return;

    var passwordConfig = this.getServiceConfig('password');
    if (passwordConfig && passwordConfig.disabled === true) {
      this.debug('Service \'password\' disabled. accounts-password module not loaded.');
      return;
    }

    var app = this;
    var accounts = app.accounts;

    accounts._hashPassword = function (password) {
      return {
        digest: SHA256(password),
        algorithm: 'sha-256'
      };
    };

    // Attempt to log in as a new user.

    /**
     * @summary Create a new user.
     * @locus Anywhere
     * @param {Object} options
     * @param {String} options.username A unique name for this user.
     * @param {String} options.email The user's email address.
     * @param {String} options.password The user's password. This is __not__ sent in plain text over the wire.
     * @param {Object} options.profile The user's profile, typically including the `name` field.
     * @param {Function} [callback] Client only, optional callback. Called with no arguments on success, or with a single `Error` argument on failure.
     * @importFromPackage accounts-base
     */
    accounts.createUser = function () {
      var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(options, callback) {
        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                options = clone(options); // we'll be modifying options

                if (!(typeof options.password !== 'string')) {
                  _context.next = 3;
                  break;
                }

                throw new Error('options.password must be a string');

              case 3:
                if (options.password) {
                  _context.next = 5;
                  break;
                }

                return _context.abrupt('return', reportError(new AppError(400, 'Password may not be empty'), callback));

              case 5:

                // Replace password with the hashed password.
                options.password = accounts._hashPassword(options.password);

                _context.next = 8;
                return accounts.callLoginMethod({
                  methodName: 'createUser',
                  methodArguments: [options],
                  userCallback: callback
                });

              case 8:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      return function (_x, _x2) {
        return _ref.apply(this, arguments);
      };
    }();

    // Change password. Must be logged in.
    //
    // @param oldPassword {String|null} By default servers no longer allow
    //   changing password without the old password, but they could so we
    //   support passing no password to the server and letting it decide.
    // @param newPassword {String}
    // @param callback {Function(error|undefined)}

    /**
     * @summary Change the current user's password. Must be logged in.
     * @locus Client
     * @param {String} oldPassword The user's current password. This is __not__ sent in plain text over the wire.
     * @param {String} newPassword A new password for the user. This is __not__ sent in plain text over the wire.
     * @param {Function} [callback] Optional callback. Called with no arguments on success, or with a single `Error` argument on failure.
     * @importFromPackage accounts-base
     */
    accounts.changePassword = function () {
      var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(userId, currentToken, oldPassword, newPassword, callback) {
        return _regenerator2.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                check(userId, String);

                if (userId) {
                  _context2.next = 3;
                  break;
                }

                return _context2.abrupt('return', reportError(new AppError(400, 'userId may not be empty'), callback));

              case 3:

                check(currentToken, String);

                if (currentToken) {
                  _context2.next = 6;
                  break;
                }

                return _context2.abrupt('return', reportError(new AppError(400, 'currentToken may not be empty'), callback));

              case 6:

                check(newPassword, String);

                if (newPassword) {
                  _context2.next = 9;
                  break;
                }

                return _context2.abrupt('return', reportError(new AppError(400, 'Password may not be empty'), callback));

              case 9:
                _context2.next = 11;
                return app.applyMethod('changePassword', [userId, currentToken, oldPassword ? accounts._hashPassword(oldPassword) : null, accounts._hashPassword(newPassword)], function (error, result) {
                  if (error || !result) {
                    // A normal error, not an error telling us to upgrade to bcrypt
                    reportError(error || new Error('No result from changePassword.'), callback);
                  } else {
                    callback && callback();
                  }
                });

              case 11:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      return function (_x3, _x4, _x5, _x6, _x7) {
        return _ref2.apply(this, arguments);
      };
    }();

    // Sends an email to a user with a link that can be used to reset
    // their password
    //
    // @param options {Object}
    //   - email: (email)
    // @param callback (optional) {Function(error|undefined)}

    /**
     * @summary Request a forgot password email.
     * @locus Client
     * @param {Object} options
     * @param {String} options.email The email address to send a password reset link.
     * @param {Function} [callback] Optional callback. Called with no arguments on success, or with a single `Error` argument on failure.
     * @importFromPackage accounts-base
     */
    accounts.forgotPassword = function () {
      var _ref3 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3(options, callback) {
        return _regenerator2.default.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                if (options.email) {
                  _context3.next = 2;
                  break;
                }

                return _context3.abrupt('return', reportError(new AppError(400, 'Must pass options.email'), callback));

              case 2:
                if (!callback) {
                  _context3.next = 7;
                  break;
                }

                _context3.next = 5;
                return app.callMethod('forgotPassword', options, callback);

              case 5:
                _context3.next = 9;
                break;

              case 7:
                _context3.next = 9;
                return app.callMethod('forgotPassword', options);

              case 9:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      return function (_x8, _x9) {
        return _ref3.apply(this, arguments);
      };
    }();

    // Resets a password based on a token originally created by
    // Accounts.forgotPassword, and then logs in the matching user.
    //
    // @param token {String}
    // @param newPassword {String}
    // @param callback (optional) {Function(error|undefined)}

    /**
     * @summary Reset the password for a user using a token received in email. Logs the user in afterwards.
     * @locus Client
     * @param {String} token The token retrieved from the reset password URL.
     * @param {String} newPassword A new password for the user. This is __not__ sent in plain text over the wire.
     * @param {Function} [callback] Optional callback. Called with no arguments on success, or with a single `Error` argument on failure.
     * @importFromPackage accounts-base
     */
    accounts.resetPassword = function () {
      var _ref4 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee4(token, newPassword, callback) {
        return _regenerator2.default.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                check(token, String);
                check(newPassword, String);

                if (newPassword) {
                  _context4.next = 4;
                  break;
                }

                return _context4.abrupt('return', reportError(new AppError(400, 'Password may not be empty'), callback));

              case 4:
                _context4.next = 6;
                return accounts.callLoginMethod({
                  methodName: 'resetPassword',
                  methodArguments: [token, accounts._hashPassword(newPassword)],
                  userCallback: callback
                });

              case 6:
              case 'end':
                return _context4.stop();
            }
          }
        }, _callee4, this);
      }));

      return function (_x10, _x11, _x12) {
        return _ref4.apply(this, arguments);
      };
    }();

    // Verifies a user's email address based on a token originally
    // created by Accounts.sendVerificationEmail
    //
    // @param token {String}
    // @param callback (optional) {Function(error|undefined)}

    /**
     * @summary Marks the user's email address as verified. Logs the user in afterwards.
     * @locus Client
     * @param {String} token The token retrieved from the verification URL.
     * @param {Function} [callback] Optional callback. Called with no arguments on success, or with a single `Error` argument on failure.
     * @importFromPackage accounts-base
     */
    accounts.verifyEmail = function () {
      var _ref5 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee5(token, callback) {
        return _regenerator2.default.wrap(function _callee5$(_context5) {
          while (1) {
            switch (_context5.prev = _context5.next) {
              case 0:
                if (token) {
                  _context5.next = 2;
                  break;
                }

                return _context5.abrupt('return', reportError(new AppError(400, 'Need to pass token'), callback));

              case 2:
                _context5.next = 4;
                return accounts.callLoginMethod({
                  methodName: 'verifyEmail',
                  methodArguments: [token],
                  userCallback: callback
                });

              case 4:
              case 'end':
                return _context5.stop();
            }
          }
        }, _callee5, this);
      }));

      return function (_x13, _x14) {
        return _ref5.apply(this, arguments);
      };
    }();
  }
});

var loginWithPassword = function () {
  var _ref6 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee6(selector, password, callback) {
    var passwordConfig, accounts;
    return _regenerator2.default.wrap(function _callee6$(_context6) {
      while (1) {
        switch (_context6.prev = _context6.next) {
          case 0:
            passwordConfig = this.getServiceConfig('password');

            if (!(passwordConfig && passwordConfig.disabled === true)) {
              _context6.next = 3;
              break;
            }

            throw new Error('Service \'password\' not loaded.');

          case 3:
            accounts = this.accounts;


            if (typeof selector === 'string') {
              if (selector.indexOf('@') === -1) selector = { username: selector };else selector = { email: selector };
            }

            _context6.next = 7;
            return accounts.callLoginMethod({
              methodArguments: [{
                user: selector,
                password: accounts._hashPassword(password)
              }],
              userCallback: function userCallback(error, result) {
                if (error) {
                  reportError(error, callback);
                } else {
                  callback && callback();
                }
              }
            });

          case 7:
          case 'end':
            return _context6.stop();
        }
      }
    }, _callee6, this);
  }));

  return function loginWithPassword(_x15, _x16, _x17) {
    return _ref6.apply(this, arguments);
  };
}();