'use strict';

module.exports = function (server) {
  function greet(welcomeMsg) {
    return function (user, url) {
      var greeting = user.profile && user.profile.name ? 'Hello ' + user.profile.name + ',' : 'Hello,';
      return greeting + '\n\n  ' + welcomeMsg + ', simply click the link below.\n\n  ' + url + '\n\n  Thanks.\n  ';
    };
  }

  /**
   * @summary Options to customize emails sent from the Accounts system.
   * @locus Server
   * @importFromPackage accounts-base
   */
  server.accounts.getEmailTemplates = function () {
    return {
      from: 'Accounts Example <no-reply@example.com>',
      siteName: server.absoluteUrl().replace(/^https?:\/\//, '').replace(/\/$/, ''),

      resetPassword: {
        subject: function subject(user) {
          return 'How to reset your password on ' + server.accounts.getEmailTemplates().siteName;
        },
        text: greet('To reset your password')
      },
      verifyEmail: {
        subject: function subject(user) {
          return 'How to verify email address on ' + server.accounts.getEmailTemplates().siteName;
        },
        text: greet('To verify your account email')
      },
      enrollAccount: {
        subject: function subject(user) {
          return 'An account has been created for you on ' + server.accounts.emailTemplates().siteName;
        },
        text: greet('To start using the service')
      }
    };
  };
};