'use strict';

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// meteor/packages/accounts-password/password_server.js
// 1687537f6ea03312183059fed45f12f350de2f17

var map = require('lodash/map');
var flatten = require('lodash/flatten');
var has = require('lodash/has');
var keys = require('lodash/keys');
var extend = require('lodash/extend');
var find = require('lodash/find');
var includes = require('lodash/includes');
var clone = require('lodash/clone');
var isUndefined = require('lodash/isUndefined');

var _require = require('../../../server/next-plus'),
    addServerModule = _require.addServerModule;

var _require2 = require('../../utils'),
    check = _require2.check,
    Random = _require2.Random,
    SHA256 = _require2.SHA256,
    Match = _require2.Match,
    AppError = _require2.AppError,
    Timers = _require2.Timers,
    escapeRegExp = _require2.escapeRegExp,
    ensure = _require2.ensure,
    wrapAsync = _require2.wrapAsync;

var _require3 = require('../../utils/email'),
    Email = _require3.Email;

/// BCRYPT

var bcrypt = require('bcrypt');
var bcryptHash = wrapAsync(bcrypt.hash);
var bcryptCompare = wrapAsync(bcrypt.compare);

addServerModule('defaultAccountsPasswordModule', {
  startup: function () {
    var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee27() {
      var passwordConfig, server, accounts, getPasswordString, hashPassword, getRoundsFromBcryptHash, checkPassword, handleError, selectorForFastCaseInsensitiveLookup, generateCasePermutationsForString, checkForCaseInsensitiveDuplicates, NonEmptyString, userQueryValidator, passwordValidator, _createUser;

      return _regenerator2.default.wrap(function _callee27$(_context27) {
        while (1) {
          switch (_context27.prev = _context27.next) {
            case 0:
              passwordConfig = this.getServiceConfig('password');

              if (!(passwordConfig && passwordConfig.disabled === true)) {
                _context27.next = 4;
                break;
              }

              this.debug('Service \'password\' disabled. accounts-password module not loaded.');
              return _context27.abrupt('return');

            case 4:
              server = this;
              accounts = server.accounts;

              // email_templates

              require('./email_templates')(this);

              // User records have a 'services.password.bcrypt' field on them to hold
              // their hashed passwords (unless they have a 'services.password.srp'
              // field, in which case they will be upgraded to bcrypt the next time
              // they log in).
              //
              // When the client sends a password to the server, it can either be a
              // string (the plaintext password) or an object with keys 'digest' and
              // 'algorithm' (must be "sha-256" for now). The Meteor client always sends
              // password objects { digest: *, algorithm: "sha-256" }, but DDP clients
              // that don't have access to SHA can just send plaintext passwords as
              // strings.
              //
              // When the server receives a plaintext password as a string, it always
              // hashes it with SHA256 before passing it into bcrypt. When the server
              // receives a password as an object, it asserts that the algorithm is
              // "sha-256" and then passes the digest to bcrypt.

              accounts._bcryptRounds = accounts._options.bcryptRounds || 10;

              // Given a 'password' from the client, extract the string that we should
              // bcrypt. 'password' can be one of:
              //  - String (the plaintext password)
              //  - Object with 'digest' and 'algorithm' keys. 'algorithm' must be "sha-256".
              //

              getPasswordString = function getPasswordString(password) {
                if (typeof password === 'string') {
                  password = SHA256(password);
                } else {
                  // 'password' is an object
                  if (password.algorithm !== 'sha-256') {
                    throw new Error('Invalid password hash algorithm. ' + "Only 'sha-256' is allowed.");
                  }
                  password = password.digest;
                }
                return password;
              };

              // Use bcrypt to hash the password for storage in the database.
              // `password` can be a string (in which case it will be run through
              // SHA256 before bcrypt) or an object with properties `digest` and
              // `algorithm` (in which case we bcrypt `password.digest`).
              //


              hashPassword = function () {
                var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(password) {
                  return _regenerator2.default.wrap(function _callee$(_context) {
                    while (1) {
                      switch (_context.prev = _context.next) {
                        case 0:
                          password = getPasswordString(password);
                          return _context.abrupt('return', bcryptHash(password, accounts._bcryptRounds));

                        case 2:
                        case 'end':
                          return _context.stop();
                      }
                    }
                  }, _callee, this);
                }));

                return function hashPassword(_x) {
                  return _ref2.apply(this, arguments);
                };
              }();

              // Extract the number of rounds used in the specified bcrypt hash.


              getRoundsFromBcryptHash = function getRoundsFromBcryptHash(hash) {
                var rounds = void 0;
                if (hash) {
                  var hashSegments = hash.split('$');
                  if (hashSegments.length > 2) {
                    rounds = parseInt(hashSegments[2], 10);
                  }
                }
                return rounds;
              };

              // Check whether the provided password matches the bcrypt'ed password in
              // the database user record. `password` can be a string (in which case
              // it will be run through SHA256 before bcrypt) or an object with
              // properties `digest` and `algorithm` (in which case we bcrypt
              // `password.digest`).
              //


              accounts._checkPassword = function () {
                var _ref3 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3(user, password) {
                  var _this = this;

                  var result, formattedPassword, hash, hashRounds;
                  return _regenerator2.default.wrap(function _callee3$(_context3) {
                    while (1) {
                      switch (_context3.prev = _context3.next) {
                        case 0:
                          result = {
                            userId: user._id
                          };
                          formattedPassword = getPasswordString(password);
                          hash = user.services.password.bcrypt;
                          hashRounds = getRoundsFromBcryptHash(hash);
                          _context3.next = 6;
                          return bcryptCompare(formattedPassword, hash);

                        case 6:
                          if (_context3.sent) {
                            _context3.next = 10;
                            break;
                          }

                          result.error = handleError('Incorrect password', false);
                          _context3.next = 11;
                          break;

                        case 10:
                          if (hash && accounts._bcryptRounds !== hashRounds) {
                            // The password checks out, but the user's bcrypt hash needs to be updated.
                            Timers.defer((0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2() {
                              return _regenerator2.default.wrap(function _callee2$(_context2) {
                                while (1) {
                                  switch (_context2.prev = _context2.next) {
                                    case 0:
                                      _context2.t0 = accounts.users;
                                      _context2.t1 = { _id: user._id };
                                      _context2.next = 4;
                                      return bcryptHash(formattedPassword, accounts._bcryptRounds);

                                    case 4:
                                      _context2.t2 = _context2.sent;
                                      _context2.t3 = {
                                        'services.password.bcrypt': _context2.t2
                                      };
                                      _context2.t4 = {
                                        $set: _context2.t3
                                      };
                                      _context2.next = 9;
                                      return _context2.t0.update.call(_context2.t0, _context2.t1, _context2.t4);

                                    case 9:
                                    case 'end':
                                      return _context2.stop();
                                  }
                                }
                              }, _callee2, _this);
                            })));
                          }

                        case 11:
                          return _context3.abrupt('return', result);

                        case 12:
                        case 'end':
                          return _context3.stop();
                      }
                    }
                  }, _callee3, this);
                }));

                return function (_x2, _x3) {
                  return _ref3.apply(this, arguments);
                };
              }();
              checkPassword = accounts._checkPassword;

              //
              /// ERROR HANDLER
              //

              handleError = function handleError(msg) {
                var throwError = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;

                var error = new AppError(403, accounts._options.ambiguousErrorMessages ? 'Something went wrong. Please check your credentials.' : msg);
                if (throwError) {
                  throw error;
                }
                return error;
              };

              //
              /// LOGIN
              //

              accounts._findUserByQuery = function () {
                var _ref5 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee4(query) {
                  var user, fieldName, fieldValue, selector, candidateUsers;
                  return _regenerator2.default.wrap(function _callee4$(_context4) {
                    while (1) {
                      switch (_context4.prev = _context4.next) {
                        case 0:
                          user = null;

                          if (!query.id) {
                            _context4.next = 7;
                            break;
                          }

                          _context4.next = 4;
                          return accounts.users.findOne({ _id: query.id });

                        case 4:
                          user = _context4.sent;
                          _context4.next = 29;
                          break;

                        case 7:
                          if (!query.username) {
                            _context4.next = 12;
                            break;
                          }

                          fieldName = 'username';
                          fieldValue = query.username;
                          _context4.next = 18;
                          break;

                        case 12:
                          if (!query.email) {
                            _context4.next = 17;
                            break;
                          }

                          fieldName = 'emails.address';
                          fieldValue = query.email;
                          _context4.next = 18;
                          break;

                        case 17:
                          throw new Error("shouldn't happen (validation missed something)");

                        case 18:
                          selector = {};

                          selector[fieldName] = fieldValue;
                          _context4.next = 22;
                          return accounts.users.findOne(selector);

                        case 22:
                          user = _context4.sent;

                          if (user) {
                            _context4.next = 29;
                            break;
                          }

                          selector = selectorForFastCaseInsensitiveLookup(fieldName, fieldValue);
                          _context4.next = 27;
                          return accounts.users.find(selector).fetch();

                        case 27:
                          candidateUsers = _context4.sent;

                          // No match if multiple candidates are found
                          if (candidateUsers.length === 1) {
                            user = candidateUsers[0];
                          }

                        case 29:
                          return _context4.abrupt('return', user);

                        case 30:
                        case 'end':
                          return _context4.stop();
                      }
                    }
                  }, _callee4, this);
                }));

                return function (_x5) {
                  return _ref5.apply(this, arguments);
                };
              }();

              /**
               * @summary Finds the user with the specified username.
               * First tries to match username case sensitively; if that fails, it
               * tries case insensitively; but if more than one user matches the case
               * insensitive search, it returns null.
               * @locus Server
               * @param {String} username The username to look for
               * @returns {Object} A user if found, else null
               * @importFromPackage accounts-base
               */
              accounts.findUserByUsername = function () {
                var _ref6 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee5(username) {
                  return _regenerator2.default.wrap(function _callee5$(_context5) {
                    while (1) {
                      switch (_context5.prev = _context5.next) {
                        case 0:
                          return _context5.abrupt('return', accounts._findUserByQuery({
                            username: username
                          }));

                        case 1:
                        case 'end':
                          return _context5.stop();
                      }
                    }
                  }, _callee5, this);
                }));

                return function (_x6) {
                  return _ref6.apply(this, arguments);
                };
              }();

              /**
               * @summary Finds the user with the specified email.
               * First tries to match email case sensitively; if that fails, it
               * tries case insensitively; but if more than one user matches the case
               * insensitive search, it returns null.
               * @locus Server
               * @param {String} email The email address to look for
               * @returns {Object} A user if found, else null
               * @importFromPackage accounts-base
               */
              accounts.findUserByEmail = function () {
                var _ref7 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee6(email) {
                  return _regenerator2.default.wrap(function _callee6$(_context6) {
                    while (1) {
                      switch (_context6.prev = _context6.next) {
                        case 0:
                          return _context6.abrupt('return', accounts._findUserByQuery({
                            email: email
                          }));

                        case 1:
                        case 'end':
                          return _context6.stop();
                      }
                    }
                  }, _callee6, this);
                }));

                return function (_x7) {
                  return _ref7.apply(this, arguments);
                };
              }();

              // Generates a MongoDB selector that can be used to perform a fast case
              // insensitive lookup for the given fieldName and string. Since MongoDB does
              // not support case insensitive indexes, and case insensitive regex queries
              // are slow, we construct a set of prefix selectors for all permutations of
              // the first 4 characters ourselves. We first attempt to matching against
              // these, and because 'prefix expression' regex queries do use indexes (see
              // http://docs.mongodb.org/v2.6/reference/operator/query/regex/#index-use),
              // this has been found to greatly improve performance (from 1200ms to 5ms in a
              // test with 1.000.000 users).

              selectorForFastCaseInsensitiveLookup = function selectorForFastCaseInsensitiveLookup(fieldName, string) {
                // Performance seems to improve up to 4 prefix characters
                var prefix = string.substring(0, Math.min(string.length, 4));
                var orClause = map(generateCasePermutationsForString(prefix), function (prefixPermutation) {
                  var selector = {};
                  selector[fieldName] = new RegExp('^' + escapeRegExp(prefixPermutation));
                  return selector;
                });
                var caseInsensitiveClause = {};
                caseInsensitiveClause[fieldName] = new RegExp('^' + escapeRegExp(string) + '$', 'i');
                return { $and: [{ $or: orClause }, caseInsensitiveClause] };
              };

              // Generates permutations of all case variations of a given string.


              generateCasePermutationsForString = function generateCasePermutationsForString(string) {
                var permutations = [''];
                for (var i = 0; i < string.length; i++) {
                  var ch = string.charAt(i);
                  permutations = flatten(map(permutations, function (prefix) {
                    var lowerCaseChar = ch.toLowerCase();
                    var upperCaseChar = ch.toUpperCase();
                    // Don't add unneccesary permutations when ch is not a letter
                    if (lowerCaseChar === upperCaseChar) {
                      return [prefix + ch];
                    } else {
                      return [prefix + lowerCaseChar, prefix + upperCaseChar];
                    }
                  }));
                }
                return permutations;
              };

              checkForCaseInsensitiveDuplicates = function checkForCaseInsensitiveDuplicates(fieldName, displayName, fieldValue, ownUserId) {
                // Some tests need the ability to add users with the same case insensitive
                // value, hence the _skipCaseInsensitiveChecksForTest check
                var skipCheck = has(accounts._skipCaseInsensitiveChecksForTest, fieldValue);

                if (fieldValue && !skipCheck) {
                  var matchedUsers = accounts.users.find(selectorForFastCaseInsensitiveLookup(fieldName, fieldValue)).fetch();

                  if (matchedUsers.length > 0 && (
                  // If we don't have a userId yet, any match we find is a duplicate
                  !ownUserId ||
                  // Otherwise, check to see if there are multiple matches or a match
                  // that is not us
                  matchedUsers.length > 1 || matchedUsers[0]._id !== ownUserId)) {
                    handleError(displayName + ' already exists.');
                  }
                }
              };

              // XXX maybe this belongs in the check package


              NonEmptyString = Match.Where(function (x) {
                check(x, String);
                return x.length > 0;
              });
              userQueryValidator = Match.Where(function (user) {
                check(user, {
                  id: Match.Optional(NonEmptyString),
                  username: Match.Optional(NonEmptyString),
                  email: Match.Optional(NonEmptyString)
                });
                if (keys(user).length !== 1) {
                  throw new Match.Error('User property must have exactly one field');
                }
                return true;
              });
              passwordValidator = Match.OneOf(String, {
                digest: String,
                algorithm: String
              });

              // Handler to login with a password.
              //
              // The Meteor client sets options.password to an object with keys
              // 'digest' (set to SHA256(password)) and 'algorithm' ("sha-256").
              //
              // For other DDP clients which don't have access to SHA, the handler
              // also accepts the plaintext password in options.password as a string.
              //
              // (It might be nice if servers could turn the plaintext password
              // option off. Or maybe it should be opt-in, not opt-out?
              // Accounts.config option?)
              //
              // Note that neither password option is secure without SSL.
              //

              accounts.registerLoginHandler('password', function () {
                var _ref8 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee7(options) {
                  var user;
                  return _regenerator2.default.wrap(function _callee7$(_context7) {
                    while (1) {
                      switch (_context7.prev = _context7.next) {
                        case 0:
                          if (options.password) {
                            _context7.next = 2;
                            break;
                          }

                          return _context7.abrupt('return', undefined);

                        case 2:
                          // don't handle

                          check(options, {
                            user: userQueryValidator,
                            password: passwordValidator
                          });

                          _context7.next = 5;
                          return accounts._findUserByQuery(options.user);

                        case 5:
                          user = _context7.sent;

                          if (!user) {
                            handleError('User not found');
                          }

                          if (!user.services || !user.services.password || !user.services.password.bcrypt) {
                            handleError('User has no password set');
                          }

                          return _context7.abrupt('return', checkPassword(user, options.password));

                        case 9:
                        case 'end':
                          return _context7.stop();
                      }
                    }
                  }, _callee7, this);
                }));

                return function (_x8) {
                  return _ref8.apply(this, arguments);
                };
              }());

              //
              /// CHANGING
              //

              /**
               * @summary Change a user's username. Use this instead of updating the
               * database directly. The operation will fail if there is an existing user
               * with a username only differing in case.
               * @locus Server
               * @param {String} userId The ID of the user to update.
               * @param {String} newUsername A new username for the user.
               * @importFromPackage accounts-base
               */
              accounts.setUsername = function () {
                var _ref9 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee8(userId, newUsername) {
                  var user, oldUsername;
                  return _regenerator2.default.wrap(function _callee8$(_context8) {
                    while (1) {
                      switch (_context8.prev = _context8.next) {
                        case 0:
                          check(userId, NonEmptyString);
                          check(newUsername, NonEmptyString);

                          _context8.next = 4;
                          return accounts.users.findOne(userId);

                        case 4:
                          user = _context8.sent;

                          if (!user) {
                            handleError('User not found');
                          }

                          oldUsername = user.username;

                          // Perform a case insensitive check for duplicates before update

                          checkForCaseInsensitiveDuplicates('username', 'Username', newUsername, user._id);

                          _context8.next = 10;
                          return accounts.users.update({ _id: user._id }, { $set: { username: newUsername } });

                        case 10:
                          _context8.prev = 10;

                          checkForCaseInsensitiveDuplicates('username', 'Username', newUsername, user._id);
                          _context8.next = 19;
                          break;

                        case 14:
                          _context8.prev = 14;
                          _context8.t0 = _context8['catch'](10);
                          _context8.next = 18;
                          return accounts.users.update({ _id: user._id }, { $set: { username: oldUsername } });

                        case 18:
                          throw _context8.t0;

                        case 19:
                        case 'end':
                          return _context8.stop();
                      }
                    }
                  }, _callee8, this, [[10, 14]]);
                }));

                return function (_x9, _x10) {
                  return _ref9.apply(this, arguments);
                };
              }();

              // Let the user change their own password if they know the old
              // password. `oldPassword` and `newPassword` should be objects with keys
              // `digest` and `algorithm` (representing the SHA256 of the password).
              //
              // XXX COMPAT WITH 0.8.1.3
              // Like the login method, if the user hasn't been upgraded from SRP to
              // bcrypt yet, then this method will throw an 'old password format'
              // error. The client should call the SRP upgrade login handler and then
              // retry this method again.
              //
              // UNLIKE the login method, there is no way to avoid getting SRP upgrade
              // errors thrown. The reasoning for this is that clients using this
              // method directly will need to be updated anyway because we no longer
              // support the SRP flow that they would have been doing to use this
              // method previously.
              server.addMethods({
                changePassword: function () {
                  var _ref10 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee9(userId, currentToken, oldPassword, newPassword) {
                    var user, result, hashed, currentHashedToken;
                    return _regenerator2.default.wrap(function _callee9$(_context9) {
                      while (1) {
                        switch (_context9.prev = _context9.next) {
                          case 0:
                            check(userId, String);
                            check(currentToken, String);
                            check(oldPassword, passwordValidator);
                            check(newPassword, passwordValidator);

                            if (userId) {
                              _context9.next = 6;
                              break;
                            }

                            throw new AppError(401, 'userId may not be empty');

                          case 6:
                            if (currentToken) {
                              _context9.next = 8;
                              break;
                            }

                            throw new AppError(401, 'currentToken may not be empty');

                          case 8:
                            _context9.next = 10;
                            return accounts.users.findOne(userId);

                          case 10:
                            user = _context9.sent;

                            if (!user) {
                              handleError('User not found');
                            }

                            if (!user.services || !user.services.password || !user.services.password.bcrypt) {
                              handleError('User has no password set');
                            }

                            _context9.next = 15;
                            return checkPassword(user, oldPassword);

                          case 15:
                            result = _context9.sent;

                            if (!result.error) {
                              _context9.next = 18;
                              break;
                            }

                            throw result.error;

                          case 18:
                            _context9.next = 20;
                            return hashPassword(newPassword);

                          case 20:
                            hashed = _context9.sent;


                            // It would be better if this removed ALL existing tokens and replaced
                            // the token for the current connection with a new one, but that would
                            // be tricky, so we'll settle for just replacing all tokens other than
                            // the one for the current connection.
                            currentHashedToken = accounts._hashLoginToken(currentToken);
                            _context9.next = 24;
                            return accounts.users.update({ _id: userId }, {
                              $set: { 'services.password.bcrypt': hashed },
                              $pull: {
                                'services.resume.loginTokens': {
                                  hashedToken: { $ne: currentHashedToken }
                                }
                              },
                              $unset: { 'services.password.reset': 1 }
                            });

                          case 24:
                            return _context9.abrupt('return', { passwordChanged: true });

                          case 25:
                          case 'end':
                            return _context9.stop();
                        }
                      }
                    }, _callee9, this);
                  }));

                  function changePassword(_x11, _x12, _x13, _x14) {
                    return _ref10.apply(this, arguments);
                  }

                  return changePassword;
                }()
              });

              // Force change the users password.

              /**
               * @summary Forcibly change the password for a user.
               * @locus Server
               * @param {String} userId The id of the user to update.
               * @param {String} newPassword A new password for the user.
               * @param {Object} [options]
               * @param {Object} options.logout Logout all current connections with this userId (default: true)
               * @importFromPackage accounts-base
               */
              accounts.setPassword = function () {
                var _ref11 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee10(userId, newPlaintextPassword, options) {
                  var user, update;
                  return _regenerator2.default.wrap(function _callee10$(_context10) {
                    while (1) {
                      switch (_context10.prev = _context10.next) {
                        case 0:
                          options = extend({ logout: true }, options);

                          _context10.next = 3;
                          return accounts.users.findOne(userId);

                        case 3:
                          user = _context10.sent;

                          if (user) {
                            _context10.next = 6;
                            break;
                          }

                          throw new AppError(403, 'User not found');

                        case 6:
                          _context10.t0 = {
                            'services.password.reset': 1
                          };
                          _context10.next = 9;
                          return hashPassword(newPlaintextPassword);

                        case 9:
                          _context10.t1 = _context10.sent;
                          _context10.t2 = {
                            'services.password.bcrypt': _context10.t1
                          };
                          update = {
                            $unset: _context10.t0,
                            $set: _context10.t2
                          };


                          if (options.logout) {
                            update.$unset['services.resume.loginTokens'] = 1;
                          }

                          _context10.next = 15;
                          return accounts.users.update({ _id: user._id }, update);

                        case 15:
                        case 'end':
                          return _context10.stop();
                      }
                    }
                  }, _callee10, this);
                }));

                return function (_x15, _x16, _x17) {
                  return _ref11.apply(this, arguments);
                };
              }();

              //
              /// RESETTING VIA EMAIL
              //

              // Method called by a user to request a password reset email. This is
              // the start of the reset process.
              server.addMethods({
                forgotPassword: function () {
                  var _ref12 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee11(options) {
                    var user, emails, caseSensitiveEmail;
                    return _regenerator2.default.wrap(function _callee11$(_context11) {
                      while (1) {
                        switch (_context11.prev = _context11.next) {
                          case 0:
                            check(options, { email: String });

                            _context11.next = 3;
                            return accounts.findUserByEmail(options.email);

                          case 3:
                            user = _context11.sent;

                            if (!user) {
                              handleError('User not found');
                            }

                            emails = map(user.emails || [], 'address');
                            caseSensitiveEmail = find(emails, function (email) {
                              return email.toLowerCase() === options.email.toLowerCase();
                            });
                            _context11.next = 9;
                            return accounts.sendResetPasswordEmail(user._id, caseSensitiveEmail);

                          case 9:
                          case 'end':
                            return _context11.stop();
                        }
                      }
                    }, _callee11, this);
                  }));

                  function forgotPassword(_x18) {
                    return _ref12.apply(this, arguments);
                  }

                  return forgotPassword;
                }()
              });

              /**
               * @summary Generates a reset token and saves it into the database.
               * @locus Server
               * @param {String} userId The id of the user to generate the reset token for.
               * @param {String} email Which address of the user to generate the reset token for. This address must be in the user's `emails` list. If `null`, defaults to the first email in the list.
               * @param {String} reason `resetPassword` or `enrollAccount`.
               * @param {Object} [extraTokenData] Optional additional data to be added into the token record.
               * @returns {Object} Object with {email, user, token} values.
               * @importFromPackage accounts-base
               */
              accounts.generateResetToken = function () {
                var _ref13 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee12(userId, email, reason, extraTokenData) {
                  var user, token, tokenRecord;
                  return _regenerator2.default.wrap(function _callee12$(_context12) {
                    while (1) {
                      switch (_context12.prev = _context12.next) {
                        case 0:
                          _context12.next = 2;
                          return accounts.users.findOne(userId);

                        case 2:
                          user = _context12.sent;

                          if (!user) {
                            handleError("Can't find user");
                          }

                          // pick the first email if we weren't passed an email.
                          if (!email && user.emails && user.emails[0]) {
                            email = user.emails[0].address;
                          }

                          // make sure we have a valid email
                          if (!email || !includes(map(user.emails || [], 'address'), email)) {
                            handleError('No such email for user.');
                          }

                          token = Random.secret();
                          tokenRecord = {
                            token: token,
                            email: email,
                            when: new Date()
                          };


                          if (reason === 'resetPassword') {
                            tokenRecord.reason = 'reset';
                          } else if (reason === 'enrollAccount') {
                            tokenRecord.reason = 'enroll';
                          } else if (reason) {
                            // fallback so that this function can be used for unknown reasons as well
                            tokenRecord.reason = reason;
                          }

                          if (extraTokenData) {
                            extend(tokenRecord, extraTokenData);
                          }

                          _context12.next = 12;
                          return accounts.users.update({ _id: user._id }, {
                            $set: {
                              'services.password.reset': tokenRecord
                            }
                          });

                        case 12:

                          // before passing to template, update user object with new token
                          ensure(user, 'services', 'password').reset = tokenRecord;

                          return _context12.abrupt('return', { email: email, user: user, token: token });

                        case 14:
                        case 'end':
                          return _context12.stop();
                      }
                    }
                  }, _callee12, this);
                }));

                return function (_x19, _x20, _x21, _x22) {
                  return _ref13.apply(this, arguments);
                };
              }();

              /**
               * @summary Generates an e-mail verification token and saves it into the database.
               * @locus Server
               * @param {String} userId The id of the user to generate the  e-mail verification token for.
               * @param {String} email Which address of the user to generate the e-mail verification token for. This address must be in the user's `emails` list. If `null`, defaults to the first unverified email in the list.
               * @param {Object} [extraTokenData] Optional additional data to be added into the token record.
               * @returns {Object} Object with {email, user, token} values.
               * @importFromPackage accounts-base
               */
              accounts.generateVerificationToken = function () {
                var _ref14 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee13(userId, email, extraTokenData) {
                  var user, emailRecord, token, tokenRecord;
                  return _regenerator2.default.wrap(function _callee13$(_context13) {
                    while (1) {
                      switch (_context13.prev = _context13.next) {
                        case 0:
                          _context13.next = 2;
                          return accounts.users.findOne(userId);

                        case 2:
                          user = _context13.sent;

                          if (!user) {
                            handleError("Can't find user");
                          }

                          // pick the first unverified email if we weren't passed an email.
                          if (!email) {
                            emailRecord = find(user.emails || [], function (e) {
                              return !e.verified;
                            });

                            email = (emailRecord || {}).address;

                            if (!email) {
                              handleError('That user has no unverified email addresses.');
                            }
                          }

                          // make sure we have a valid email
                          if (!email || !includes(map(user.emails || [], 'address'), email)) {
                            handleError('No such email for user.');
                          }

                          token = Random.secret();
                          tokenRecord = {
                            token: token,
                            // TODO: This should probably be renamed to "email" to match reset token record.
                            address: email,
                            when: new Date()
                          };


                          if (extraTokenData) {
                            extend(tokenRecord, extraTokenData);
                          }

                          _context13.next = 11;
                          return accounts.users.update({ _id: user._id }, {
                            $push: {
                              'services.email.verificationTokens': tokenRecord
                            }
                          });

                        case 11:

                          // before passing to template, update user object with new token
                          ensure(user, 'services', 'email');
                          if (!user.services.email.verificationTokens) {
                            user.services.email.verificationTokens = [];
                          }
                          user.services.email.verificationTokens.push(tokenRecord);

                          return _context13.abrupt('return', { email: email, user: user, token: token });

                        case 15:
                        case 'end':
                          return _context13.stop();
                      }
                    }
                  }, _callee13, this);
                }));

                return function (_x23, _x24, _x25) {
                  return _ref14.apply(this, arguments);
                };
              }();

              /**
               * @summary Creates options for email sending for reset password and enroll account emails.
               * You can use this function when customizing a reset password or enroll account email sending.
               * @locus Server
               * @param {Object} email Which address of the user's to send the email to.
               * @param {Object} user The user object to generate options for.
               * @param {String} url URL to which user is directed to confirm the email.
               * @param {String} reason `resetPassword` or `enrollAccount`.
               * @returns {Object} Options which can be passed to `Email.send`.
               * @importFromPackage accounts-base
               */
              accounts.generateOptionsForEmail = function (email, user, url, reason) {
                var emailTemplates = accounts.getEmailTemplates();
                var options = {
                  to: email,
                  from: emailTemplates[reason].from ? emailTemplates[reason].from(user) : emailTemplates.from,
                  subject: emailTemplates[reason].subject(user)
                };

                if (typeof emailTemplates[reason].text === 'function') {
                  options.text = emailTemplates[reason].text(user, url);
                }

                if (typeof emailTemplates[reason].html === 'function') {
                  options.html = emailTemplates[reason].html(user, url);
                }

                if ((0, _typeof3.default)(emailTemplates.headers) === 'object') {
                  options.headers = emailTemplates.headers;
                }

                return options;
              };

              // send the user an email with a link that when opened allows the user
              // to set a new password, without the old password.

              /**
               * @summary Send an email with a link the user can use to reset their password.
               * @locus Server
               * @param {String} userId The id of the user to send email to.
               * @param {String} [email] Optional. Which address of the user's to send the email to. This address must be in the user's `emails` list. Defaults to the first email in the list.
               * @param {Object} [extraTokenData] Optional additional data to be added into the token record.
               * @returns {Object} Object with {email, user, token, url, options} values.
               * @importFromPackage accounts-base
               */
              accounts.sendResetPasswordEmail = function () {
                var _ref15 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee14(userId, email, extraTokenData) {
                  var _ref16, realEmail, user, token, url, options;

                  return _regenerator2.default.wrap(function _callee14$(_context14) {
                    while (1) {
                      switch (_context14.prev = _context14.next) {
                        case 0:
                          _context14.next = 2;
                          return accounts.generateResetToken(userId, email, 'resetPassword', extraTokenData);

                        case 2:
                          _ref16 = _context14.sent;
                          realEmail = _ref16.email;
                          user = _ref16.user;
                          token = _ref16.token;
                          url = server.absoluteUrl('reset-password/' + token);
                          options = accounts.generateOptionsForEmail(realEmail, user, url, 'resetPassword');

                          Email.send(options);
                          return _context14.abrupt('return', { email: realEmail, user: user, token: token, url: url, options: options });

                        case 10:
                        case 'end':
                          return _context14.stop();
                      }
                    }
                  }, _callee14, this);
                }));

                return function (_x26, _x27, _x28) {
                  return _ref15.apply(this, arguments);
                };
              }();

              // send the user an email informing them that their account was created, with
              // a link that when opened both marks their email as verified and forces them
              // to choose their password. The email must be one of the addresses in the
              // user's emails field, or undefined to pick the first email automatically.
              //
              // This is not called automatically. It must be called manually if you
              // want to use enrollment emails.

              /**
               * @summary Send an email with a link the user can use to set their initial password.
               * @locus Server
               * @param {String} userId The id of the user to send email to.
               * @param {String} [email] Optional. Which address of the user's to send the email to. This address must be in the user's `emails` list. Defaults to the first email in the list.
               * @param {Object} [extraTokenData] Optional additional data to be added into the token record.
               * @returns {Object} Object with {email, user, token, url, options} values.
               * @importFromPackage accounts-base
               */
              accounts.sendEnrollmentEmail = function () {
                var _ref17 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee15(userId, email, extraTokenData) {
                  var _ref18, realEmail, user, token, url, options;

                  return _regenerator2.default.wrap(function _callee15$(_context15) {
                    while (1) {
                      switch (_context15.prev = _context15.next) {
                        case 0:
                          _context15.next = 2;
                          return accounts.generateResetToken(userId, email, 'enrollAccount', extraTokenData);

                        case 2:
                          _ref18 = _context15.sent;
                          realEmail = _ref18.email;
                          user = _ref18.user;
                          token = _ref18.token;
                          url = server.absoluteUrl('enroll-account/' + token);
                          options = accounts.generateOptionsForEmail(realEmail, user, url, 'enrollAccount');

                          Email.send(options);
                          return _context15.abrupt('return', { email: realEmail, user: user, token: token, url: url, options: options });

                        case 10:
                        case 'end':
                          return _context15.stop();
                      }
                    }
                  }, _callee15, this);
                }));

                return function (_x29, _x30, _x31) {
                  return _ref17.apply(this, arguments);
                };
              }();

              // Take token from sendResetPasswordEmail or sendEnrollmentEmail, change
              // the users password, and log them in.
              server.addMethods({
                resetPassword: function () {
                  var _ref19 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee17(token, newPassword, context) {
                    return _regenerator2.default.wrap(function _callee17$(_context17) {
                      while (1) {
                        switch (_context17.prev = _context17.next) {
                          case 0:
                            return _context17.abrupt('return', accounts._loginMethod('resetPassword', [token, newPassword], context, 'password', (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee16() {
                              var user, when, reason, tokenLifetimeMs, currentTimeMs, email, hashed, affectedRecords;
                              return _regenerator2.default.wrap(function _callee16$(_context16) {
                                while (1) {
                                  switch (_context16.prev = _context16.next) {
                                    case 0:
                                      check(token, String);
                                      check(newPassword, passwordValidator);

                                      _context16.next = 4;
                                      return accounts.users.findOne({
                                        'services.password.reset.token': token
                                      });

                                    case 4:
                                      user = _context16.sent;

                                      if (user) {
                                        _context16.next = 7;
                                        break;
                                      }

                                      throw new AppError(403, 'Token expired');

                                    case 7:
                                      when = user.services.password.reset.when;
                                      reason = user.services.password.reset.reason;
                                      tokenLifetimeMs = accounts._getPasswordResetTokenLifetimeMs();

                                      if (reason === 'enroll') {
                                        tokenLifetimeMs = accounts._getPasswordEnrollTokenLifetimeMs();
                                      }
                                      currentTimeMs = Date.now();

                                      if (!(currentTimeMs - when > tokenLifetimeMs)) {
                                        _context16.next = 14;
                                        break;
                                      }

                                      throw new AppError(403, 'Token expired');

                                    case 14:
                                      email = user.services.password.reset.email;

                                      if (includes(map(user.emails || [], 'address'), email)) {
                                        _context16.next = 17;
                                        break;
                                      }

                                      return _context16.abrupt('return', {
                                        userId: user._id,
                                        error: new AppError(403, 'Token has invalid email address')
                                      });

                                    case 17:
                                      _context16.next = 19;
                                      return hashPassword(newPassword);

                                    case 19:
                                      hashed = _context16.sent;
                                      _context16.prev = 20;
                                      _context16.next = 23;
                                      return accounts.users.update({
                                        _id: user._id,
                                        'emails.address': email,
                                        'services.password.reset.token': token
                                      }, {
                                        $set: {
                                          'services.password.bcrypt': hashed,
                                          'emails.$.verified': true
                                        },
                                        $unset: {
                                          'services.password.reset': 1
                                        }
                                      });

                                    case 23:
                                      affectedRecords = _context16.sent;

                                      if (!(affectedRecords !== 1)) {
                                        _context16.next = 26;
                                        break;
                                      }

                                      return _context16.abrupt('return', {
                                        userId: user._id,
                                        error: new AppError(403, 'Invalid email')
                                      });

                                    case 26:
                                      _context16.next = 31;
                                      break;

                                    case 28:
                                      _context16.prev = 28;
                                      _context16.t0 = _context16['catch'](20);
                                      throw _context16.t0;

                                    case 31:
                                      _context16.next = 33;
                                      return accounts._clearAllLoginTokens(user._id);

                                    case 33:
                                      return _context16.abrupt('return', { userId: user._id });

                                    case 34:
                                    case 'end':
                                      return _context16.stop();
                                  }
                                }
                              }, _callee16, this, [[20, 28]]);
                            }))));

                          case 1:
                          case 'end':
                            return _context17.stop();
                        }
                      }
                    }, _callee17, this);
                  }));

                  function resetPassword(_x32, _x33, _x34) {
                    return _ref19.apply(this, arguments);
                  }

                  return resetPassword;
                }()
              });

              //
              /// EMAIL VERIFICATION
              //

              // send the user an email with a link that when opened marks that
              // address as verified

              /**
               * @summary Send an email with a link the user can use verify their email address.
               * @locus Server
               * @param {String} userId The id of the user to send email to.
               * @param {String} [email] Optional. Which address of the user's to send the email to. This address must be in the user's `emails` list. Defaults to the first unverified email in the list.
               * @param {Object} [extraTokenData] Optional additional data to be added into the token record.
               * @returns {Object} Object with {email, user, token, url, options} values.
               * @importFromPackage accounts-base
               */
              accounts.sendVerificationEmail = function () {
                var _ref21 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee18(userId, email, extraTokenData) {
                  var _ref22, realEmail, user, token, url, options;

                  return _regenerator2.default.wrap(function _callee18$(_context18) {
                    while (1) {
                      switch (_context18.prev = _context18.next) {
                        case 0:
                          _context18.next = 2;
                          return accounts.generateVerificationToken(userId, email, extraTokenData);

                        case 2:
                          _ref22 = _context18.sent;
                          realEmail = _ref22.email;
                          user = _ref22.user;
                          token = _ref22.token;
                          url = server.absoluteUrl('verify-email/' + token);
                          options = accounts.generateOptionsForEmail(realEmail, user, url, 'verifyEmail');

                          Email.send(options);
                          return _context18.abrupt('return', { email: realEmail, user: user, token: token, url: url, options: options });

                        case 10:
                        case 'end':
                          return _context18.stop();
                      }
                    }
                  }, _callee18, this);
                }));

                return function (_x35, _x36, _x37) {
                  return _ref21.apply(this, arguments);
                };
              }();

              // Take token from sendVerificationEmail, mark the email as verified,
              // and log them in.
              server.addMethods({
                verifyEmail: function () {
                  var _ref23 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee20(token, context) {
                    return _regenerator2.default.wrap(function _callee20$(_context20) {
                      while (1) {
                        switch (_context20.prev = _context20.next) {
                          case 0:
                            return _context20.abrupt('return', accounts._loginMethod('verifyEmail', [token], context, 'password', (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee19() {
                              var user, tokenRecord, emailsRecord;
                              return _regenerator2.default.wrap(function _callee19$(_context19) {
                                while (1) {
                                  switch (_context19.prev = _context19.next) {
                                    case 0:
                                      check(token, String);

                                      _context19.next = 3;
                                      return accounts.users.findOne({
                                        'services.email.verificationTokens.token': token
                                      });

                                    case 3:
                                      user = _context19.sent;

                                      if (user) {
                                        _context19.next = 6;
                                        break;
                                      }

                                      throw new AppError(403, 'Verify email link expired');

                                    case 6:
                                      tokenRecord = find(user.services.email.verificationTokens, function (t) {
                                        return t.token === token;
                                      });

                                      if (tokenRecord) {
                                        _context19.next = 9;
                                        break;
                                      }

                                      return _context19.abrupt('return', {
                                        userId: user._id,
                                        error: new AppError(403, 'Verify email link expired')
                                      });

                                    case 9:
                                      emailsRecord = find(user.emails, function (e) {
                                        return e.address === tokenRecord.address;
                                      });

                                      if (emailsRecord) {
                                        _context19.next = 12;
                                        break;
                                      }

                                      return _context19.abrupt('return', {
                                        userId: user._id,
                                        error: new AppError(403, 'Verify email link is for unknown address')
                                      });

                                    case 12:
                                      _context19.next = 14;
                                      return accounts.users.update({
                                        _id: user._id,
                                        'emails.address': tokenRecord.address
                                      }, {
                                        $set: { 'emails.$.verified': true },
                                        $pull: {
                                          'services.email.verificationTokens': {
                                            address: tokenRecord.address
                                          }
                                        }
                                      });

                                    case 14:
                                      return _context19.abrupt('return', { userId: user._id });

                                    case 15:
                                    case 'end':
                                      return _context19.stop();
                                  }
                                }
                              }, _callee19, this);
                            }))));

                          case 1:
                          case 'end':
                            return _context20.stop();
                        }
                      }
                    }, _callee20, this);
                  }));

                  function verifyEmail(_x38, _x39) {
                    return _ref23.apply(this, arguments);
                  }

                  return verifyEmail;
                }()
              });

              /**
               * @summary Add an email address for a user. Use this instead of directly
               * updating the database. The operation will fail if there is a different user
               * with an email only differing in case. If the specified user has an existing
               * email only differing in case however, we replace it.
               * @locus Server
               * @param {String} userId The ID of the user to update.
               * @param {String} newEmail A new email address for the user.
               * @param {Boolean} [verified] Optional - whether the new email address should
               * be marked as verified. Defaults to false.
               * @importFromPackage accounts-base
               */
              accounts.addEmail = function () {
                var _ref25 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee21(userId, newEmail, verified) {
                  var user, caseInsensitiveRegExp, didUpdateOwnEmail, i, email;
                  return _regenerator2.default.wrap(function _callee21$(_context21) {
                    while (1) {
                      switch (_context21.prev = _context21.next) {
                        case 0:
                          check(userId, NonEmptyString);
                          check(newEmail, NonEmptyString);
                          check(verified, Match.Optional(Boolean));

                          if (isUndefined(verified)) {
                            verified = false;
                          }

                          _context21.next = 6;
                          return accounts.users.findOne(userId);

                        case 6:
                          user = _context21.sent;

                          if (user) {
                            _context21.next = 9;
                            break;
                          }

                          throw new AppError(403, 'User not found');

                        case 9:

                          // Allow users to change their own email to a version with a different case

                          // We don't have to call checkForCaseInsensitiveDuplicates to do a case
                          // insensitive check across all emails in the database here because: (1) if
                          // there is no case-insensitive duplicate between this user and other users,
                          // then we are OK and (2) if this would create a conflict with other users
                          // then there would already be a case-insensitive duplicate and we can't fix
                          // that in this code anyway.
                          caseInsensitiveRegExp = new RegExp('^' + escapeRegExp(newEmail) + '$', 'i');
                          didUpdateOwnEmail = false;
                          i = 0;

                        case 12:
                          if (!(i < user.emails.length)) {
                            _context21.next = 22;
                            break;
                          }

                          email = user.emails[i];

                          if (!caseInsensitiveRegExp.test(email.address)) {
                            _context21.next = 19;
                            break;
                          }

                          _context21.next = 17;
                          return accounts.users.update({
                            _id: user._id,
                            'emails.address': email.address
                          }, {
                            $set: {
                              'emails.$.address': newEmail,
                              'emails.$.verified': verified
                            }
                          });

                        case 17:
                          didUpdateOwnEmail = true;
                          return _context21.abrupt('break', 22);

                        case 19:
                          i++;
                          _context21.next = 12;
                          break;

                        case 22:
                          if (!didUpdateOwnEmail) {
                            _context21.next = 24;
                            break;
                          }

                          return _context21.abrupt('return');

                        case 24:

                          // Perform a case insensitive check for duplicates before update
                          checkForCaseInsensitiveDuplicates('emails.address', 'Email', newEmail, user._id);

                          _context21.next = 27;
                          return accounts.users.update({
                            _id: user._id
                          }, {
                            $addToSet: {
                              emails: {
                                address: newEmail,
                                verified: verified
                              }
                            }
                          });

                        case 27:
                          _context21.prev = 27;

                          checkForCaseInsensitiveDuplicates('emails.address', 'Email', newEmail, user._id);
                          _context21.next = 36;
                          break;

                        case 31:
                          _context21.prev = 31;
                          _context21.t0 = _context21['catch'](27);
                          _context21.next = 35;
                          return accounts.users.update({ _id: user._id }, { $pull: { emails: { address: newEmail } } });

                        case 35:
                          throw _context21.t0;

                        case 36:
                        case 'end':
                          return _context21.stop();
                      }
                    }
                  }, _callee21, this, [[27, 31]]);
                }));

                return function (_x40, _x41, _x42) {
                  return _ref25.apply(this, arguments);
                };
              }();

              /**
               * @summary Remove an email address for a user. Use this instead of updating
               * the database directly.
               * @locus Server
               * @param {String} userId The ID of the user to update.
               * @param {String} email The email address to remove.
               * @importFromPackage accounts-base
               */
              accounts.removeEmail = function () {
                var _ref26 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee22(userId, email) {
                  var user;
                  return _regenerator2.default.wrap(function _callee22$(_context22) {
                    while (1) {
                      switch (_context22.prev = _context22.next) {
                        case 0:
                          check(userId, NonEmptyString);
                          check(email, NonEmptyString);

                          _context22.next = 4;
                          return accounts.users.findOne(userId);

                        case 4:
                          user = _context22.sent;

                          if (user) {
                            _context22.next = 7;
                            break;
                          }

                          throw new AppError(403, 'User not found');

                        case 7:
                          _context22.next = 9;
                          return accounts.users.update({ _id: user._id }, { $pull: { emails: { address: email } } });

                        case 9:
                        case 'end':
                          return _context22.stop();
                      }
                    }
                  }, _callee22, this);
                }));

                return function (_x43, _x44) {
                  return _ref26.apply(this, arguments);
                };
              }();

              //
              /// CREATING USERS
              //

              // Shared createUser function called from the createUser method, both
              // if originates in client or server code. Calls user provided hooks,
              // does the actual user insertion.
              //
              // returns the user id

              _createUser = function () {
                var _ref27 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee23(options) {
                  var username, email, user, hashed, userId;
                  return _regenerator2.default.wrap(function _callee23$(_context23) {
                    while (1) {
                      switch (_context23.prev = _context23.next) {
                        case 0:
                          // Unknown keys allowed, because a onCreateUserHook can take arbitrary
                          // options.
                          check(options, Match.ObjectIncluding({
                            username: Match.Optional(String),
                            email: Match.Optional(String),
                            password: Match.Optional(passwordValidator)
                          }));

                          username = options.username;
                          email = options.email;

                          if (!(!username && !email)) {
                            _context23.next = 5;
                            break;
                          }

                          throw new AppError(400, 'Need to set a username or email');

                        case 5:
                          user = { services: {} };

                          if (!options.password) {
                            _context23.next = 11;
                            break;
                          }

                          _context23.next = 9;
                          return hashPassword(options.password);

                        case 9:
                          hashed = _context23.sent;

                          user.services.password = { bcrypt: hashed };

                        case 11:

                          if (username) user.username = username;
                          if (email) user.emails = [{ address: email, verified: false }];

                          // Perform a case insensitive check before insert
                          checkForCaseInsensitiveDuplicates('username', 'Username', username);
                          checkForCaseInsensitiveDuplicates('emails.address', 'Email', email);

                          _context23.next = 17;
                          return accounts.insertUserDoc(options, user);

                        case 17:
                          userId = _context23.sent;
                          _context23.prev = 18;

                          checkForCaseInsensitiveDuplicates('username', 'Username', username, userId);
                          checkForCaseInsensitiveDuplicates('emails.address', 'Email', email, userId);
                          _context23.next = 28;
                          break;

                        case 23:
                          _context23.prev = 23;
                          _context23.t0 = _context23['catch'](18);
                          _context23.next = 27;
                          return accounts.users.remove(userId);

                        case 27:
                          throw _context23.t0;

                        case 28:
                          return _context23.abrupt('return', userId);

                        case 29:
                        case 'end':
                          return _context23.stop();
                      }
                    }
                  }, _callee23, this, [[18, 23]]);
                }));

                return function _createUser(_x45) {
                  return _ref27.apply(this, arguments);
                };
              }();

              // method for create user. Requests come from the client.


              server.addMethods({
                createUser: function () {
                  var _ref28 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee25(options, context) {
                    return _regenerator2.default.wrap(function _callee25$(_context25) {
                      while (1) {
                        switch (_context25.prev = _context25.next) {
                          case 0:
                            return _context25.abrupt('return', accounts._loginMethod('createUser', [options], context, 'password', (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee24() {
                              var userId;
                              return _regenerator2.default.wrap(function _callee24$(_context24) {
                                while (1) {
                                  switch (_context24.prev = _context24.next) {
                                    case 0:
                                      // createUser() above does more checking.
                                      check(options, Object);

                                      if (!accounts._options.forbidClientAccountCreation) {
                                        _context24.next = 3;
                                        break;
                                      }

                                      return _context24.abrupt('return', {
                                        error: new AppError(403, 'Signups forbidden')
                                      });

                                    case 3:
                                      _context24.next = 5;
                                      return _createUser(options);

                                    case 5:
                                      userId = _context24.sent;

                                      if (userId) {
                                        _context24.next = 8;
                                        break;
                                      }

                                      throw new Error('createUser failed to insert new user');

                                    case 8:
                                      if (!(options.email && accounts._options.sendVerificationEmail)) {
                                        _context24.next = 11;
                                        break;
                                      }

                                      _context24.next = 11;
                                      return accounts.sendVerificationEmail(userId, options.email);

                                    case 11:
                                      return _context24.abrupt('return', { userId: userId });

                                    case 12:
                                    case 'end':
                                      return _context24.stop();
                                  }
                                }
                              }, _callee24, this);
                            }))));

                          case 1:
                          case 'end':
                            return _context25.stop();
                        }
                      }
                    }, _callee25, this);
                  }));

                  function createUser(_x46, _x47) {
                    return _ref28.apply(this, arguments);
                  }

                  return createUser;
                }()
              });

              // Create user directly on the server.
              //
              // Unlike the client version, this does not log you in as this user
              // after creation.
              //
              // returns userId or throws an error if it can't create
              //
              // XXX add another argument ("server options") that gets sent to onCreateUser,
              // which is always empty when called from the createUser method? eg, "admin:
              // true", which we want to prevent the client from setting, but which a custom
              // method calling Accounts.createUser could set?
              //
              accounts.createUser = function () {
                var _ref30 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee26(options, callback) {
                  return _regenerator2.default.wrap(function _callee26$(_context26) {
                    while (1) {
                      switch (_context26.prev = _context26.next) {
                        case 0:
                          options = clone(options);

                          // XXX allow an optional callback?

                          if (!callback) {
                            _context26.next = 3;
                            break;
                          }

                          throw new Error('Accounts.createUser with callback not supported on the server yet.');

                        case 3:
                          return _context26.abrupt('return', _createUser(options));

                        case 4:
                        case 'end':
                          return _context26.stop();
                      }
                    }
                  }, _callee26, this);
                }));

                return function (_x48, _x49) {
                  return _ref30.apply(this, arguments);
                };
              }();

              //
              /// PASSWORD-SPECIFIC INDEXES ON USERS
              //
              _context27.next = 43;
              return accounts.users._ensureIndex('services.email.verificationTokens.token', {
                unique: 1,
                sparse: 1
              });

            case 43:
              _context27.next = 45;
              return accounts.users._ensureIndex('services.password.reset.token', {
                unique: 1,
                sparse: 1
              });

            case 45:
            case 'end':
              return _context27.stop();
          }
        }
      }, _callee27, this);
    }));

    function startup() {
      return _ref.apply(this, arguments);
    }

    return startup;
  }()
});