'use strict';

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _require = require('../../../app/next-plus'),
    addAppModule = _require.addAppModule;

var _require2 = require('../../utils'),
    isServer = _require2.isServer,
    Random = _require2.Random;

var ILLEGAL_PARAMETERS = {
  response_type: 1,
  client_id: 1,
  scope: 1,
  redirect_uri: 1,
  state: 1
};

addAppModule('defaultGoogleOAuthModule', {
  setup: function setup(ctx) {
    if (isServer) {
      this.debug('Browser only module. google-oauth module not loaded.');
      return;
    }

    if (!this.getServiceConfig('google')) {
      this.debug('Service config \'google\' not found. google-oauth module not loaded.');
      return;
    }

    var app = this;

    if (app.accounts._initialized) return;

    var oauth = app.oauth;
    app.accounts.google = {
      requestCredential: function requestCredential(options, credentialRequestCompleteCallback) {
        if (!credentialRequestCompleteCallback && typeof options === 'function') {
          credentialRequestCompleteCallback = options;
          options = {};
        } else if (!options) {
          options = {};
        }

        var services = app.getSetting('services') || {};
        var config = services['google'];
        if (!config) {
          var error = new Error('Service google not configured');
          credentialRequestCompleteCallback && credentialRequestCompleteCallback(error);
        }

        var credentialToken = Random.secret();

        var requiredScopes = { email: 1 };
        var scopes = options.requestPermissions || ['profile'];
        scopes.forEach(function (scope) {
          requiredScopes[scope] = 1;
        });
        scopes = (0, _keys2.default)(requiredScopes);

        var loginUrlParameters = {};
        if (config.loginUrlParameters) {
          (0, _assign2.default)(loginUrlParameters, config.loginUrlParameters);
        }
        if (options.loginUrlParameters) {
          (0, _assign2.default)(loginUrlParameters, options.loginUrlParameters);
        }

        (0, _keys2.default)(loginUrlParameters).forEach(function (key) {
          if (ILLEGAL_PARAMETERS.hasOwnProperty(key)) {
            throw new Error('Google.requestCredential: Invalid loginUrlParameter: ' + key);
          }
        });

        if (options.requestOfflineToken != null) {
          loginUrlParameters.access_type = options.requestOfflineToken ? 'offline' : 'online';
        }
        if (options.prompt != null) {
          loginUrlParameters.prompt = options.prompt;
        } else if (options.forceApprovalPrompt) {
          loginUrlParameters.prompt = 'consent';
        }

        if (options.loginHint) {
          loginUrlParameters.login_hint = options.loginHint;
        }

        var loginStyle = oauth._loginStyle('google', config, options);

        (0, _assign2.default)(loginUrlParameters, {
          response_type: 'code',
          client_id: config.clientId,
          scope: scopes.join(' '),
          redirect_uri: oauth._redirectUri('google', config),
          state: oauth._stateParam(loginStyle, credentialToken, options.redirectUrl)
        });
        var loginUrl = 'https://accounts.google.com/o/oauth2/auth?' + (0, _keys2.default)(loginUrlParameters).map(function (param) {
          return encodeURIComponent(param) + '=' + encodeURIComponent(loginUrlParameters[param]);
        }).join('&');

        oauth.launchLogin({
          loginService: 'google',
          loginStyle: loginStyle,
          loginUrl: loginUrl,
          credentialRequestCompleteCallback: credentialRequestCompleteCallback,
          credentialToken: credentialToken,
          popupOptions: { height: 600 }
        });
      }
    };
  }
});