'use strict';

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var includes = require('lodash/includes');
var bind = require('lodash/bind');
var has = require('lodash/has');

var _require = require('../../utils'),
    check = _require.check,
    Base64 = _require.Base64,
    debug = _require.debug,
    LocalStorage = _require.LocalStorage;

var _require2 = require('../../utils/reload'),
    Reload = _require2.Reload;

var clone = require('lodash/clone');
var isEmpty = require('lodash/isEmpty');

var _require3 = require('../../utils/url/client'),
    URL = _require3.URL;

var _require4 = require('./oauth_common'),
    OAuthCommon = _require4.OAuthCommon;

var OAuthClient = function (_OAuthCommon) {
  (0, _inherits3.default)(OAuthClient, _OAuthCommon);

  function OAuthClient(app) {
    (0, _classCallCheck3.default)(this, OAuthClient);

    var _this = (0, _possibleConstructorReturn3.default)(this, (OAuthClient.__proto__ || (0, _getPrototypeOf2.default)(OAuthClient)).call(this, app));

    _this.credentialSecrets = {};
    return _this;
  }

  (0, _createClass3.default)(OAuthClient, [{
    key: '_redirectUri',
    value: function _redirectUri(serviceName, config, params, absoluteUrlOptions) {
      var query = config.loginStyle ? null : 'close';

      if (params) {
        params = clone(params);
        if (isEmpty(params)) {
          params = undefined;
        }
      }

      return URL._constructUrl(this.instance.absoluteUrl('_oauth/' + serviceName, absoluteUrlOptions), query, params);
    }
  }]);
  return OAuthClient;
}(OAuthCommon);

OAuthClient.prototype.showPopup = function (url, callback, dimensions) {
  throw new Error('OAuth.showPopup must be implemented on this arch.');
};

// Determine the login style (popup or redirect) for this login flow.
//
//
OAuthClient.prototype._loginStyle = function (service, config, options) {
  var loginStyle = options && options.loginStyle || config.loginStyle || 'popup';

  if (!includes(['popup', 'redirect'], loginStyle)) {
    throw new Error('Invalid login style: ' + loginStyle);
  }

  // If we don't have session storage (for example, Safari in private
  // mode), the redirect login flow won't work, so fallback to the
  // popup style.
  if (loginStyle === 'redirect') {
    try {
      window.sessionStorage.setItem('App.oauth.test', 'test');
      window.sessionStorage.removeItem('App.oauth.test');
    } catch (e) {
      loginStyle = 'popup';
    }
  }

  return loginStyle;
};

OAuthClient.prototype._stateParam = function (loginStyle, credentialToken, redirectUrl) {
  var state = {
    loginStyle: loginStyle,
    credentialToken: credentialToken
  };

  if (loginStyle === 'redirect') {
    state.redirectUrl = redirectUrl || '' + window.location;
  }

  // Encode base64 as not all login services URI-encode the state
  // parameter when they pass it back to us.
  // Use the 'base64' package here because 'btoa' isn't supported in IE8/9.
  return Base64.encode((0, _stringify2.default)(state));
};

// At the beginning of the redirect login flow, before we redirect to
// the login service, save the credential token for this login attempt
// in the reload migration data.
//
OAuthClient.prototype.saveDataForRedirect = function (loginService, credentialToken) {
  Reload._onMigrate('oauth', function () {
    return [true, { loginService: loginService, credentialToken: credentialToken }];
  });
  Reload._migrate(null, { immediateMigration: true });
};

// At the end of the redirect login flow, when we've redirected back
// to the application, retrieve the credentialToken and (if the login
// was successful) the credentialSecret.
//
// Called at application startup.  Returns null if this is normal
// application startup and we weren't just redirected at the end of
// the login flow.
//
OAuthClient.prototype.getDataAfterRedirect = function () {
  var migrationData = Reload._migrationData('oauth');

  if (!(migrationData && migrationData.credentialToken)) {
    return null;
  }

  var credentialToken = migrationData.credentialToken;
  var key = this._storageTokenPrefix + credentialToken;
  var credentialSecret;
  try {
    credentialSecret = window.sessionStorage.getItem(key);
    window.sessionStorage.removeItem(key);
  } catch (e) {
    debug('error retrieving credentialSecret', e);
  }
  return {
    loginService: migrationData.loginService,
    credentialToken: credentialToken,
    credentialSecret: credentialSecret
  };
};

// Launch an OAuth login flow.  For the popup login style, show the
// popup.  For the redirect login style, save the credential token for
// this login attempt in the reload migration data, and redirect to
// the service for the login.
//
// options:
//  loginService: "facebook", "google", etc.
//  loginStyle: "popup" or "redirect"
//  loginUrl: The URL at the login service provider to start the OAuth flow.
//  credentialRequestCompleteCallback: for the popup flow, call when the popup
//    is closed and we have the credential from the login service.
//  credentialToken: our identifier for this login flow.
//
OAuthClient.prototype.launchLogin = function (options) {
  if (!options.loginService) {
    throw new Error('loginService required');
  }
  if (options.loginStyle === 'popup') {
    this.showPopup(options.loginUrl, bind(options.credentialRequestCompleteCallback, null, options.credentialToken), options.popupOptions);
  } else if (options.loginStyle === 'redirect') {
    this.saveDataForRedirect(options.loginService, options.credentialToken);
    window.location = options.loginUrl;
  } else {
    throw new Error('invalid login style');
  }
};

// Called by the popup when the OAuth flow is completed, right before
// the popup closes.
OAuthClient.prototype._handleCredentialSecret = function (credentialToken, secret) {
  check(credentialToken, String);
  check(secret, String);
  if (!has(this.credentialSecrets, credentialToken)) {
    this.credentialSecrets[credentialToken] = secret;
  } else {
    throw new Error('Duplicate credential token from OAuth login');
  }
};

// Used by accounts-oauth, which needs both a credentialToken and the
// corresponding to credential secret to call the `login` method over DDP.
OAuthClient.prototype._retrieveCredentialSecret = function (credentialToken) {
  // First check the secrets collected by OAuth._handleCredentialSecret,
  // then check localStorage. This matches what we do in
  // end_of_login_response.html.
  var secret = this.credentialSecrets[credentialToken];
  if (!secret) {
    var localStorageKey = this._storageTokenPrefix + credentialToken;
    secret = LocalStorage.getItem(localStorageKey);
    LocalStorage.removeItem(localStorageKey);
  } else {
    delete this.credentialSecrets[credentialToken];
  }
  return secret;
};

module.exports = {
  OAuthClient: OAuthClient
};