'use strict';

var _getOwnPropertyNames = require('babel-runtime/core-js/object/get-own-property-names');

var _getOwnPropertyNames2 = _interopRequireDefault(_getOwnPropertyNames);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// meteor/packages/oauth/pending_credentials.js
// 82d1db5107112dbc31bdcbbad60c6831ea1a7030

var _require = require('../../utils'),
    AppError = _require.AppError,
    check = _require.check,
    Match = _require.Match,
    Timers = _require.Timers;

var _require2 = require('./oauth_server'),
    OAuthServer = _require2.OAuthServer;

var Op = OAuthServer.prototype;

Op.initPendingCredentials = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2() {
  var _this = this;

  var _cleanStaleResults;

  return _regenerator2.default.wrap(function _callee2$(_context2) {
    while (1) {
      switch (_context2.prev = _context2.next) {
        case 0:
          this._pendingCredentials = this.server.createCollection('OAuthPendingCredentials', { dbCollectionName: 'oauth_pending_credentials' });

          _context2.next = 3;
          return this._pendingCredentials._ensureIndex('key', { unique: 1 });

        case 3:
          _context2.next = 5;
          return this._pendingCredentials._ensureIndex('credentialSecret');

        case 5:
          _context2.next = 7;
          return this._pendingCredentials._ensureIndex('createdAt');

        case 7:
          _cleanStaleResults = function () {
            var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee() {
              var timeCutoff;
              return _regenerator2.default.wrap(function _callee$(_context) {
                while (1) {
                  switch (_context.prev = _context.next) {
                    case 0:
                      timeCutoff = new Date();

                      timeCutoff.setMinutes(timeCutoff.getMinutes() - 1);
                      _context.next = 4;
                      return _this._pendingCredentials.remove({ createdAt: { $lt: timeCutoff } });

                    case 4:
                    case 'end':
                      return _context.stop();
                  }
                }
              }, _callee, _this);
            }));

            return function _cleanStaleResults() {
              return _ref2.apply(this, arguments);
            };
          }();

          this._cleanStaleResultsInterval = Timers.setInterval(_cleanStaleResults, 60 * 1000);

        case 9:
        case 'end':
          return _context2.stop();
      }
    }
  }, _callee2, this);
}));

Op._storePendingCredential = function () {
  var _ref3 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3(key, credential, credentialSecret) {
    return _regenerator2.default.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            check(key, String);
            check(credentialSecret, Match.Optional(String));

            if (credential instanceof Error) {
              credential = this.storableError(credential);
            } else {
              credential = this.sealSecret(credential);
            }

            _context3.next = 5;
            return this._pendingCredentials.upsert({
              key: key
            }, {
              key: key,
              credential: credential,
              credentialSecret: credentialSecret || null,
              createdAt: new Date()
            });

          case 5:
          case 'end':
            return _context3.stop();
        }
      }
    }, _callee3, this);
  }));

  return function (_x, _x2, _x3) {
    return _ref3.apply(this, arguments);
  };
}();

Op._retrievePendingCredential = function () {
  var _ref4 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee4(key, credentialSecret) {
    var pendingCredential;
    return _regenerator2.default.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            check(key, String);

            _context4.next = 3;
            return this._pendingCredentials.findOne({
              key: key,
              credentialSecret: credentialSecret || null
            });

          case 3:
            pendingCredential = _context4.sent;

            if (!pendingCredential) {
              _context4.next = 14;
              break;
            }

            _context4.next = 7;
            return this._pendingCredentials.remove({ _id: pendingCredential._id });

          case 7:
            if (!pendingCredential.credential.error) {
              _context4.next = 11;
              break;
            }

            return _context4.abrupt('return', this.recreateError(pendingCredential.credential.error));

          case 11:
            return _context4.abrupt('return', this.openSecret(pendingCredential.credential));

          case 12:
            _context4.next = 15;
            break;

          case 14:
            return _context4.abrupt('return', undefined);

          case 15:
          case 'end':
            return _context4.stop();
        }
      }
    }, _callee4, this);
  }));

  return function (_x4, _x5) {
    return _ref4.apply(this, arguments);
  };
}();

Op.storableError = function (error) {
  var plainObject = {};
  (0, _getOwnPropertyNames2.default)(error).forEach(function (key) {
    plainObject[key] = error[key];
  });

  if (error instanceof AppError) {
    plainObject['appError'] = true;
  }

  return { error: plainObject };
};

Op.recreateError = function (errorDoc) {
  var error;

  if (errorDoc.appError) {
    error = new AppError();
    delete errorDoc.appError;
  } else {
    error = new Error();
  }

  (0, _getOwnPropertyNames2.default)(errorDoc).forEach(function (key) {
    error[key] = errorDoc[key];
  });

  return error;
};