'use strict';

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _require = require('../../server/next-plus'),
    addServerModule = _require.addServerModule;

require('./accounts-base/server');
require('./oauth/server');
require('./oauth2/server');
require('./accounts-oauth/server');
require('./google-oauth/server');
require('./accounts-google/server');
require('./accounts-password/server');

addServerModule('defaultAccountsModule', {
  setup: function setup() {
    var _this = this;

    var server = this;

    require('./common')(server);

    this.configGraphqlOptions(function (options) {
      var context = options.context;
      context.getCurrentUser = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee() {
        var currentUser;
        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _context.t0 = context.currentUser;

                if (_context.t0) {
                  _context.next = 5;
                  break;
                }

                _context.next = 4;
                return server.getUserFromToken(context.loginToken);

              case 4:
                _context.t0 = _context.sent;

              case 5:
                currentUser = _context.t0;

                if (currentUser && !context.currentUser) {
                  context.currentUser = currentUser;
                  context.currentUserId = currentUser._id;
                }
                return _context.abrupt('return', currentUser || null);

              case 8:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, _this);
      }));
      return options;
    });

    this.getUserFromToken = getUserFromToken.bind(this);
  },
  schemaStartup: function schemaStartup() {
    this.addGraphqlSchema('\n      type User {\n        _id: String\n        username: String\n        emails: [JSON]\n        createdAt: DateTime\n        isAdmin: Boolean\n        profile: JSON\n        services: JSON\n        groups: [String]\n        displayName: String\n      }\n    ');
    this.addGraphqlQueryField('currentUser: User');
    this.addGraphqlQueryField('getUserFromToken: User');
    this.addGraphqlResolvers({
      Query: {
        currentUser: function () {
          var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(root, args, context) {
            var currentUser;
            return _regenerator2.default.wrap(function _callee2$(_context2) {
              while (1) {
                switch (_context2.prev = _context2.next) {
                  case 0:
                    _context2.next = 2;
                    return context.getCurrentUser();

                  case 2:
                    currentUser = _context2.sent;

                    if (currentUser && currentUser.services) {
                      (0, _keys2.default)(currentUser.services).forEach(function (key) {
                        currentUser.services[key] = {};
                      });
                    }
                    return _context2.abrupt('return', currentUser || null);

                  case 5:
                  case 'end':
                    return _context2.stop();
                }
              }
            }, _callee2, this);
          }));

          function currentUser(_x, _x2, _x3) {
            return _ref2.apply(this, arguments);
          }

          return currentUser;
        }(),
        getUserFromToken: function () {
          var _ref4 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3(root, args, _ref3) {
            var server = _ref3.server,
                loginToken = _ref3.loginToken;

            var accounts, _server$getCollection, Users, hashedToken, user, tokenInformation, expiresAt, isExpired;

            return _regenerator2.default.wrap(function _callee3$(_context3) {
              while (1) {
                switch (_context3.prev = _context3.next) {
                  case 0:
                    accounts = server.accounts;
                    _server$getCollection = server.getCollections(), Users = _server$getCollection.Users;

                    if (!loginToken) {
                      _context3.next = 15;
                      break;
                    }

                    if (!(typeof loginToken !== 'string')) {
                      _context3.next = 5;
                      break;
                    }

                    throw new Error('Invalid Token');

                  case 5:
                    hashedToken = accounts._hashLoginToken(loginToken);
                    _context3.next = 8;
                    return Users.findOne({
                      'services.resume.loginTokens.hashedToken': hashedToken
                    });

                  case 8:
                    user = _context3.sent;

                    if (!user) {
                      _context3.next = 15;
                      break;
                    }

                    tokenInformation = user.services.resume.loginTokens.find(function (tokenInfo) {
                      return tokenInfo.hashedToken === hashedToken;
                    });
                    expiresAt = accounts._tokenExpiration(tokenInformation.when);
                    isExpired = expiresAt < new Date();

                    if (isExpired) {
                      _context3.next = 15;
                      break;
                    }

                    return _context3.abrupt('return', user);

                  case 15:
                    return _context3.abrupt('return', null);

                  case 16:
                  case 'end':
                    return _context3.stop();
                }
              }
            }, _callee3, this);
          }));

          function getUserFromToken(_x4, _x5, _x6) {
            return _ref4.apply(this, arguments);
          }

          return getUserFromToken;
        }()
      }
    });
  }
});

var getUserFromToken = function () {
  var _ref5 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee4(loginToken) {
    var result;
    return _regenerator2.default.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            _context4.next = 2;
            return this.runQuery('getUserFromToken', {}, { loginToken: loginToken });

          case 2:
            result = _context4.sent;
            return _context4.abrupt('return', result.data.getUserFromToken);

          case 4:
          case 'end':
            return _context4.stop();
        }
      }
    }, _callee4, this);
  }));

  return function getUserFromToken(_x7) {
    return _ref5.apply(this, arguments);
  };
}();