'use strict';

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _extends3 = require('babel-runtime/helpers/extends');

var _extends4 = _interopRequireDefault(_extends3);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _require = require('apollo-client'),
    ApolloClient = _require.ApolloClient;

var _require2 = require('apollo-link-batch-http'),
    BatchHttpLink = _require2.BatchHttpLink;

var _require3 = require('apollo-cache-inmemory'),
    InMemoryCache = _require3.InMemoryCache;

var _require4 = require('apollo-link-context'),
    setContext = _require4.setContext;

var _require5 = require('apollo-link'),
    ApolloLink = _require5.ApolloLink;

var _require6 = require('apollo-utilities'),
    getMainDefinition = _require6.getMainDefinition;

var _require7 = require('../../app/next-plus'),
    addAppModule = _require7.addAppModule;

var _require8 = require('../utils'),
    isClient = _require8.isClient,
    isServer = _require8.isServer,
    COOKIES_LOGIN_TOKEN_KEY = _require8.COOKIES_LOGIN_TOKEN_KEY,
    HEADER_LOGIN_TOKEN_KEY = _require8.HEADER_LOGIN_TOKEN_KEY,
    LOGIN_TOKEN_KEY = _require8.LOGIN_TOKEN_KEY;

var resetStore = function () {
  var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee() {
    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            if (!this.client) {
              _context.next = 2;
              break;
            }

            return _context.abrupt('return', this.client.resetStore());

          case 2:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, this);
  }));

  return function resetStore() {
    return _ref.apply(this, arguments);
  };
}();

var resetCache = function () {
  var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2() {
    return _regenerator2.default.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            if (!this.client) {
              _context2.next = 2;
              break;
            }

            return _context2.abrupt('return', this.client.cache.reset());

          case 2:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, this);
  }));

  return function resetCache() {
    return _ref2.apply(this, arguments);
  };
}();

addAppModule('defaultApolloClientModule', {
  setup: function setup(_ref3) {
    var initialState = _ref3.initialState,
        req = _ref3.req;

    if (!this.client) {
      this.client = createApolloClient({
        app: this,
        initialState: initialState,
        loginToken: isServer && req ? req.cookies[COOKIES_LOGIN_TOKEN_KEY] : '',
        uri: this.getSetting('graphqlUrl') || this.absoluteUrl('graphql'),
        headerLoginTokenKey: HEADER_LOGIN_TOKEN_KEY,
        localStorageLoginTokenKey: LOGIN_TOKEN_KEY
      });
    }

    this.resetStore = resetStore.bind(this);
    this.resetCache = resetCache.bind(this);
  }
});

function createApolloClient(_ref4) {
  var app = _ref4.app,
      initialState = _ref4.initialState,
      uri = _ref4.uri,
      loginToken = _ref4.loginToken,
      headerLoginTokenKey = _ref4.headerLoginTokenKey,
      localStorageLoginTokenKey = _ref4.localStorageLoginTokenKey;

  var httpLink = new BatchHttpLink({
    uri: uri,
    credentials: 'same-origin'
  });

  var authLink = setContext(function (_, _ref5) {
    var headers = _ref5.headers;

    var serverLoginToken = isServer && loginToken;

    var localStorageLoginToken = isClient && window.localStorage && window.localStorage.getItem(localStorageLoginTokenKey);

    loginToken = localStorageLoginToken || serverLoginToken;

    return {
      headers: (0, _extends4.default)({}, headers, (0, _defineProperty3.default)({}, headerLoginTokenKey, loginToken || ''))
    };
  });

  var loggerLink = new ActionsLoggerLink(app);

  return new ApolloClient({
    connectToDevTools: true,
    ssrMode: isServer,
    link: ApolloLink.from([loggerLink, authLink, httpLink]),
    cache: new InMemoryCache().restore(initialState || {}),
    addTypename: true
  });
}

var APOLLO_QUERY_INIT = 'APOLLO_QUERY_INIT';
var APOLLO_QUERY_RESULT = 'APOLLO_QUERY_RESULT';
var APOLLO_MUTATION_INIT = 'APOLLO_MUTATION_INIT';
var APOLLO_MUTATION_RESULT = 'APOLLO_MUTATION_RESULT';
var APOLLO_SUBSCRIPTION_INIT = 'APOLLO_SUBSCRIPTION_INIT';
var APOLLO_SUBSCRIPTION_RESULT = 'APOLLO_SUBSCRIPTION_RESULT';

var payload = function payload(_ref6) {
  var operationName = _ref6.operationName,
      variables = _ref6.variables,
      query = _ref6.query;
  return {
    operationName: operationName,
    variables: variables,
    document: query
  };
};

var initAction = function initAction(type) {
  return function (operation) {
    return (0, _extends4.default)({
      type: type
    }, payload(operation));
  };
};

var resultAction = function resultAction(type) {
  return function (result, operation) {
    return (0, _extends4.default)({
      type: type,
      result: result
    }, payload(operation));
  };
};

var queryInit = initAction(APOLLO_QUERY_INIT);
var queryResult = resultAction(APOLLO_QUERY_RESULT);
var mutationInit = initAction(APOLLO_MUTATION_INIT);
var mutationResult = resultAction(APOLLO_MUTATION_RESULT);
var subscriptionInit = initAction(APOLLO_SUBSCRIPTION_INIT);
var subscriptionResult = resultAction(APOLLO_SUBSCRIPTION_RESULT);

var isQuery = function isQuery(op) {
  return op === 'query';
};
var isMutation = function isMutation(op) {
  return op === 'mutation';
};
var isSubscription = function isSubscription(op) {
  return op === 'subscription';
};

var ActionsLoggerLink = function (_ApolloLink) {
  (0, _inherits3.default)(ActionsLoggerLink, _ApolloLink);

  function ActionsLoggerLink(app) {
    (0, _classCallCheck3.default)(this, ActionsLoggerLink);

    var _this = (0, _possibleConstructorReturn3.default)(this, (ActionsLoggerLink.__proto__ || (0, _getPrototypeOf2.default)(ActionsLoggerLink)).call(this));

    _this.app = app;
    return _this;
  }

  (0, _createClass3.default)(ActionsLoggerLink, [{
    key: 'request',
    value: function request(operation, forward) {
      var _this2 = this;

      var observer = forward(operation);
      if (!isClient || !this.app || !this.app.dev) {
        return observer;
      }

      var definition = getMainDefinition(operation.query);
      if (isQuery(definition.operation)) {
        this.app.debug(queryInit(operation));
      } else if (isMutation(definition.operation)) {
        this.app.debug(mutationInit(operation));
      } else if (isSubscription(definition.operation)) {
        this.app.debug(subscriptionInit(operation));
      }
      return observer.map(function (result) {
        if (isQuery(definition.operation)) {
          _this2.app.debug(queryResult(result, operation));
        } else if (isMutation(definition.operation)) {
          _this2.app.debug(mutationResult(result, operation));
        } else if (isSubscription(definition.operation)) {
          _this2.app.debug(subscriptionResult(result, operation));
        }
        return result;
      });
    }
  }]);
  return ActionsLoggerLink;
}(ApolloLink);