'use strict';

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var uniq = require('lodash/uniq');
var gql = require('graphql-tag');

var registerFragment = function registerFragment(fragmentTextSource) {
  var fragmentText = fragmentTextSource.replace(/#.*\n/g, '\n');
  var fragmentName = fragmentText.match(/fragment (.*) on/)[1];
  var matchedSubFragments = fragmentText.match(/\.{3}([_A-Za-z][_0-9A-Za-z]*)/g) || [];
  var subFragments = uniq(matchedSubFragments.map(function (f) {
    return f.replace('...', '');
  }));

  var fragments = this.fragments;

  fragments[fragmentName] = {
    fragmentText: fragmentText,
    subFragments: subFragments
  };
};

var extendFragment = function extendFragment(fragmentName, newProperties) {
  var fragments = this.fragments;

  var fragment = fragments[fragmentName];
  var fragmentEndPosition = fragment.fragmentText.lastIndexOf('}');
  var newFragmentText = [fragment.fragmentText.slice(0, fragmentEndPosition), newProperties, fragment.fragmentText.slice(fragmentEndPosition)].join('');
  this.registerFragment(newFragmentText);
};

var getFragment = function getFragment(fragmentName) {
  var _this = this;

  var fragments = this.fragments;


  var fragment = fragments[fragmentName];

  if (!fragment) {
    throw new Error('Fragment "' + fragmentName + '" not registered.');
  }

  if (fragment.document) {
    return fragment.document;
  }

  var literals = [fragment.fragmentText].concat((0, _toConsumableArray3.default)(fragment.subFragments.map(function (x) {
    return '\n';
  })));
  var gqlArguments = [literals].concat((0, _toConsumableArray3.default)(fragment.subFragments.map(function (f) {
    return _this.getFragment(f);
  })));
  fragment.document = gql.apply(null, gqlArguments);
  return fragment.document;
};

var registerQuery = function registerQuery(queryName, queryTextSource) {
  if (!queryTextSource) {
    queryTextSource = queryName;
    queryName = undefined;
  }
  var queryText = queryTextSource.replace(/#.*\n/g, '\n');
  queryName = queryName || queryText.match(/query ([^() {]*)/)[1];
  if (!queryName) {
    throw new Error('Query name required');
  }
  var matchedFragments = queryText.match(/\.{3}([_A-Za-z][_0-9A-Za-z]*)/g) || [];
  var fragments = uniq(matchedFragments.map(function (f) {
    return f.replace('...', '');
  }));

  var queries = this.queries;

  queries[queryName] = {
    text: queryText,
    fragments: fragments
  };
};

var getQuery = function getQuery(queryName) {
  var _this2 = this;

  var queries = this.queries;


  var query = queries[queryName];
  if (!query) {
    throw new Error('Query "' + queryName + '" not registered.');
  }

  if (query.document) {
    return query.document;
  }

  var literals = [query.text].concat((0, _toConsumableArray3.default)(query.fragments.map(function (x) {
    return '\n';
  })));
  var gqlArguments = [literals].concat((0, _toConsumableArray3.default)(query.fragments.map(function (f) {
    return _this2.getFragment(f);
  })));
  query.document = gql.apply(null, gqlArguments);
  return query.document;
};

var registerMutation = function registerMutation(mutationName, mutationTextSource) {
  if (!mutationTextSource) {
    mutationTextSource = mutationName;
    mutationName = undefined;
  }
  var mutationText = mutationTextSource.replace(/#.*\n/g, '\n');
  mutationName = mutationName || mutationText.match(/mutation ([^() {]*)/)[1];
  if (!mutationName) {
    throw new Error('Mutation name required');
  }
  var matchedFragments = mutationText.match(/\.{3}([_A-Za-z][_0-9A-Za-z]*)/g) || [];
  var fragments = uniq(matchedFragments.map(function (f) {
    return f.replace('...', '');
  }));

  var mutations = this.mutations;

  mutations[mutationName] = {
    text: mutationText,
    fragments: fragments
  };
};

var getMutation = function getMutation(mutationName) {
  var _this3 = this;

  var mutations = this.mutations;


  var mutation = mutations[mutationName];
  if (!mutation) {
    throw new Error('Mutation "' + mutationName + '" not registered.');
  }

  if (mutation.document) {
    return mutation.document;
  }

  var literals = [mutation.text].concat((0, _toConsumableArray3.default)(mutation.fragments.map(function (x) {
    return '\n';
  })));
  var gqlArguments = [literals].concat((0, _toConsumableArray3.default)(mutation.fragments.map(function (f) {
    return _this3.getFragment(f);
  })));
  mutation.document = gql.apply(null, gqlArguments);
  return mutation.document;
};

var getDocument = function getDocument(name) {
  var queries = this.queries,
      mutations = this.mutations;


  var query = queries[name];
  var mutation = mutations[name];

  if (query) {
    return this.getQuery(name);
  } else if (mutation) {
    return this.getMutation(name);
  } else {
    throw new Error('Graphql operation "' + name + '" not registered.');
  }
};

module.exports = {
  registerFragment: registerFragment,
  extendFragment: extendFragment,
  getFragment: getFragment,
  registerQuery: registerQuery,
  getQuery: getQuery,
  registerMutation: registerMutation,
  getMutation: getMutation,
  getDocument: getDocument
};