'use strict';

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var merge = require('lodash/merge');

var _require = require('graphql'),
    graphql = _require.graphql,
    print = _require.print;

var GraphQLJSON = require('graphql-type-json');
var GraphQLDate = require('graphql-date');

var _require2 = require('graphql-iso-date'),
    GraphQLDateTime = _require2.GraphQLDateTime;

var _require3 = require('../../server/next-plus'),
    addServerModule = _require3.addServerModule;

var _require4 = require('./common'),
    registerFragment = _require4.registerFragment,
    extendFragment = _require4.extendFragment,
    getFragment = _require4.getFragment,
    registerQuery = _require4.registerQuery,
    getQuery = _require4.getQuery,
    registerMutation = _require4.registerMutation,
    getMutation = _require4.getMutation;

addServerModule('defaultGraphqlSchemaModule', {
  setup: function setup() {
    this.graphqlSchemaObj = {
      schemas: [],
      queryFields: [],
      mutationFields: []
    };

    this.graphqlResolvers = {
      JSON: GraphQLJSON,
      Date: GraphQLDate,
      DateTime: GraphQLDateTime
    };

    this.graphqlContext = {
      server: this,
      collections: this.collections
    };

    this.queries = {};
    this.mutations = {};
    this.fragments = {};

    this.registerFragment = registerFragment.bind(this);
    this.extendFragment = extendFragment.bind(this);
    this.getFragment = getFragment.bind(this);
    this.registerQuery = registerQuery.bind(this);
    this.getQuery = getQuery.bind(this);
    this.registerMutation = registerMutation.bind(this);
    this.getMutation = getMutation.bind(this);

    this.addGraphqlSchema = addGraphqlSchema.bind(this);
    this.addGraphqlQueryField = addGraphqlQueryField.bind(this);
    this.addGraphqlMutationField = addGraphqlMutationField.bind(this);
    this.addGraphqlResolvers = addGraphqlResolvers.bind(this);
    this.addToGraphqlContext = addToGraphqlContext.bind(this);
    this.runGraphql = runGraphql.bind(this);
    this.runQuery = runQuery.bind(this);
    this.runMutation = runMutation.bind(this);
  }
});

var addGraphqlSchema = function addGraphqlSchema(schema) {
  var graphqlSchemaObj = this.graphqlSchemaObj;

  graphqlSchemaObj.schemas.push(schema);
};

var addGraphqlQueryField = function addGraphqlQueryField(query) {
  var graphqlSchemaObj = this.graphqlSchemaObj;

  graphqlSchemaObj.queryFields.push(query);
};

var addGraphqlMutationField = function addGraphqlMutationField(mutation) {
  var graphqlSchemaObj = this.graphqlSchemaObj;

  graphqlSchemaObj.mutationFields.push(mutation);
};

var addGraphqlResolvers = function addGraphqlResolvers(resolvers) {
  var graphqlResolvers = this.graphqlResolvers;

  merge(graphqlResolvers, resolvers);
};

var addToGraphqlContext = function addToGraphqlContext(object) {
  var graphqlContext = this.graphqlContext;

  merge(graphqlContext, object);
};

var runGraphql = function () {
  var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(requestString) {
    var variables = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    var context = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : { currentUser: { isAdmin: true } };

    var executableSchema, graphqlContext, _configGraphqlOptionsHook, options, result;

    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            executableSchema = this.executableSchema, graphqlContext = this.graphqlContext, _configGraphqlOptionsHook = this._configGraphqlOptionsHook;
            options = {
              execute: true,
              context: (0, _extends3.default)({}, graphqlContext, context)
            };


            _configGraphqlOptionsHook.each(function (func) {
              options = func(options);
              return true;
            });

            _context.next = 5;
            return graphql(executableSchema, requestString, {}, options.context, variables);

          case 5:
            result = _context.sent;

            if (!result.errors) {
              _context.next = 9;
              break;
            }

            console.log('runGraphql error: ' + result.errors[0].message);
            throw new Error(result.errors[0].message);

          case 9:
            return _context.abrupt('return', result);

          case 10:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, this);
  }));

  return function runGraphql(_x3) {
    return _ref.apply(this, arguments);
  };
}();

var runQuery = function () {
  var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(query, variables, context) {
    var queries, requestString, queryObj, document;
    return _regenerator2.default.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            queries = this.queries;
            requestString = '';


            if (typeof query === 'string') {
              if (/^[_A-Za-z][_0-9A-Za-z]*$/.test(query)) {
                queryObj = queries[query];

                if (!queryObj.queryFullText) {
                  document = this.getQuery(query);

                  queryObj.queryFullText = print(document);
                }
                requestString = queryObj.queryFullText;
              } else {
                requestString = query;
              }
            } else if ((typeof query === 'undefined' ? 'undefined' : (0, _typeof3.default)(query)) === 'object') {
              requestString = print(query);
            }

            return _context2.abrupt('return', this.runGraphql(requestString, variables, context));

          case 4:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, this);
  }));

  return function runQuery(_x4, _x5, _x6) {
    return _ref2.apply(this, arguments);
  };
}();

var runMutation = function () {
  var _ref3 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3(mutation, variables, context) {
    var mutations, requestString, mutationObj, document;
    return _regenerator2.default.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            mutations = this.mutations;
            requestString = '';


            if (typeof mutation === 'string') {
              if (/^[_A-Za-z][_0-9A-Za-z]*$/.test(mutation)) {
                mutationObj = mutations[mutation];

                if (!mutationObj.mutationFullText) {
                  document = this.getMutation(mutation);

                  mutationObj.mutationFullText = print(document);
                }
                requestString = mutationObj.mutationFullText;
              } else {
                requestString = mutation;
              }
            } else if ((typeof mutation === 'undefined' ? 'undefined' : (0, _typeof3.default)(mutation)) === 'object') {
              requestString = print(mutation);
            }

            return _context3.abrupt('return', this.runGraphql(requestString, variables, context));

          case 4:
          case 'end':
            return _context3.stop();
        }
      }
    }, _callee3, this);
  }));

  return function runMutation(_x7, _x8, _x9) {
    return _ref3.apply(this, arguments);
  };
}();