'use strict';

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var has = require('lodash/has');

var _require = require('../utils'),
    EJSON = _require.EJSON;

var _require2 = require('../../app/next-plus'),
    addAppModule = _require2.addAppModule;

var _require3 = require('../utils'),
    AppError = _require3.AppError;

var common = require('./common');

addAppModule('defaultMethodsModule', {
  setup: function setup(ctx) {
    common(this);

    this.applyMethod = applyMethod.bind(this);
    this.callMethod = callMethod.bind(this);
  }
});

var applyMethod = function () {
  var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(name, args, callback) {
    var message, app, returnValue, exception;
    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            args = EJSON.clone(args);

            message = {
              msg: 'method',
              method: name,
              params: args
            };
            app = this;
            _context.prev = 3;
            _context.next = 6;
            return methodInvoker(app, message);

          case 6:
            returnValue = _context.sent;
            _context.next = 13;
            break;

          case 9:
            _context.prev = 9;
            _context.t0 = _context['catch'](3);

            console.log("Error invoking Method '" + name + "':", _context.t0.message);
            exception = _context.t0;

          case 13:
            if (!callback) {
              _context.next = 16;
              break;
            }

            callback(exception, returnValue);
            return _context.abrupt('return', undefined);

          case 16:
            if (!exception) {
              _context.next = 18;
              break;
            }

            throw exception;

          case 18:
            return _context.abrupt('return', returnValue);

          case 19:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, this, [[3, 9]]);
  }));

  return function applyMethod(_x, _x2, _x3) {
    return _ref.apply(this, arguments);
  };
}();

var callMethod = function () {
  var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(name) {
    for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
      args[_key - 1] = arguments[_key];
    }

    var callback;
    return _regenerator2.default.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            if (args.length && typeof args[args.length - 1] === 'function') {
              callback = args.pop();
            }
            return _context2.abrupt('return', this.applyMethod(name, args, callback));

          case 2:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, this);
  }));

  return function callMethod(_x4) {
    return _ref2.apply(this, arguments);
  };
}();

var methodInvoker = function () {
  var _ref3 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3(app, message) {
    var result, msg;
    return _regenerator2.default.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            _context3.next = 2;
            return app.client.mutate({
              mutation: app.getMutation('methodInvoker'),
              variables: {
                message: message
              }
            });

          case 2:
            result = _context3.sent;
            msg = result.data.methodInvoker;

            if (!(msg === null || !msg.msg)) {
              _context3.next = 7;
              break;
            }

            console.log('discarding invalid message', msg);
            return _context3.abrupt('return');

          case 7:
            if (!(msg.msg === 'result')) {
              _context3.next = 11;
              break;
            }

            return _context3.abrupt('return', _result(msg));

          case 11:
            if (msg.msg === 'error') {
              _error(msg);
            } else {
              console.log('discarding unknown message type', msg);
            }

          case 12:
            return _context3.abrupt('return', undefined);

          case 13:
          case 'end':
            return _context3.stop();
        }
      }
    }, _callee3, this);
  }));

  return function methodInvoker(_x5, _x6) {
    return _ref3.apply(this, arguments);
  };
}();

var _error = function _error(msg) {
  console.log('Received error from server: ', msg.reason);
  if (msg.offendingMessage) {
    console.log('For: ', msg.offendingMessage);
  }
};

var _result = function _result(msg) {
  if (has(msg, 'error')) {
    throw new AppError(msg.error.error, msg.error.reason, msg.error.details);
  } else {
    return msg.result;
  }
};