'use strict';

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var has = require('lodash/has');
var each = require('lodash/each');

var _require = require('../../server/next-plus'),
    addServerModule = _require.addServerModule;

var _require2 = require('../utils'),
    EJSON = _require2.EJSON,
    AppError = _require2.AppError,
    debug = _require2.debug;

var common = require('./common');

addServerModule('defaultMethodsModule', {
  setup: function setup() {
    common(this);

    this.methodHandlers = {};

    this.addMethods = addMethods.bind(this);
    this.applyMethod = applyMethod.bind(this);
    this.callMethod = callMethod.bind(this);
  },
  schemaStartup: function schemaStartup() {
    this.addGraphqlMutationField('\n      # methodInvoker\n      methodInvoker (message: JSON): JSON\n    ');

    this.addGraphqlResolvers({
      Mutation: {
        methodInvoker: function () {
          var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(root, _ref, context) {
            var message = _ref.message;
            var result;
            return _regenerator2.default.wrap(function _callee$(_context) {
              while (1) {
                switch (_context.prev = _context.next) {
                  case 0:
                    result = void 0;
                    _context.prev = 1;
                    _context.next = 4;
                    return processMessage(message, context);

                  case 4:
                    result = _context.sent;

                    result = JSON.parse((0, _stringify2.default)(result));
                    _context.next = 10;
                    break;

                  case 8:
                    _context.prev = 8;
                    _context.t0 = _context['catch'](1);

                  case 10:
                    return _context.abrupt('return', result);

                  case 11:
                  case 'end':
                    return _context.stop();
                }
              }
            }, _callee, this, [[1, 8]]);
          }));

          function methodInvoker(_x, _x2, _x3) {
            return _ref2.apply(this, arguments);
          }

          return methodInvoker;
        }()
      }
    });
  }
});

var processMessage = function () {
  var _ref3 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(msg, context) {
    var result;
    return _regenerator2.default.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            if (msg) {
              _context2.next = 2;
              break;
            }

            return _context2.abrupt('return');

          case 2:
            if (!has(protocolHandlers, msg.msg)) {
              _context2.next = 8;
              break;
            }

            _context2.next = 5;
            return protocolHandlers[msg.msg](msg, context);

          case 5:
            result = _context2.sent;
            _context2.next = 9;
            break;

          case 8:
            result = getErrorMessage('Bad request', msg);

          case 9:
            return _context2.abrupt('return', result);

          case 10:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, this);
  }));

  return function processMessage(_x4, _x5) {
    return _ref3.apply(this, arguments);
  };
}();

var protocolHandlers = {
  method: function () {
    var _ref4 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3(msg, context) {
      var handler, payload, result;
      return _regenerator2.default.wrap(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              if (!(typeof msg.method !== 'string' || 'params' in msg && !(msg.params instanceof Array))) {
                _context3.next = 2;
                break;
              }

              return _context3.abrupt('return', getErrorMessage('Malformed method invocation', msg));

            case 2:
              handler = context.server.methodHandlers[msg.method];

              if (handler) {
                _context3.next = 5;
                break;
              }

              return _context3.abrupt('return', {
                msg: 'result',
                error: new AppError(404, 'Method \'' + msg.method + '\' not found')
              });

            case 5:
              payload = {
                msg: 'result'
              };
              _context3.prev = 6;
              _context3.next = 9;
              return handler.apply(context.server, [].concat((0, _toConsumableArray3.default)(msg.params), [context]));

            case 9:
              result = _context3.sent;

              if (result !== undefined) {
                payload.result = result;
              }
              _context3.next = 16;
              break;

            case 13:
              _context3.prev = 13;
              _context3.t0 = _context3['catch'](6);

              payload.error = wrapInternalException(_context3.t0, 'while invoking method \'' + msg.method + '\'');

            case 16:
              return _context3.abrupt('return', payload);

            case 17:
            case 'end':
              return _context3.stop();
          }
        }
      }, _callee3, this, [[6, 13]]);
    }));

    function method(_x6, _x7) {
      return _ref4.apply(this, arguments);
    }

    return method;
  }()
};

var wrapInternalException = function wrapInternalException(exception, context) {
  if (!exception || exception instanceof AppError) {
    return exception;
  }

  if (!exception.expected) {
    debug('Exception ' + context, exception.stack);
    if (exception.sanitizedError) {
      debug('Sanitized and reported to the client as:', exception.sanitizedError.message);
      debug();
    }
  }

  if (exception.sanitizedError) {
    if (exception.sanitizedError instanceof AppError) {
      return exception.sanitizedError;
    }
    debug('Exception ' + context + ' provides a sanitizedError that ' + 'is not a server.Error; ignoring');
  }

  return new AppError(500, 'Internal server error');
};

var getErrorMessage = function getErrorMessage(reason, offendingMessage) {
  var msg = { msg: 'error', reason: reason };
  if (offendingMessage) {
    msg.offendingMessage = offendingMessage;
  }
  return msg;
};

var addMethods = function addMethods(methods) {
  var server = this;

  each(methods, function (func, name) {
    if (typeof func !== 'function') {
      throw new Error("Method '" + name + "' must be a function");
    }
    if (server.methodHandlers[name]) {
      throw new Error("A method named '" + name + "' is already defined");
    }
    server.methodHandlers[name] = func;
  });
};

var applyMethod = function () {
  var _ref5 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee4(name, args, callback) {
    var server, handler, result;
    return _regenerator2.default.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            server = this;
            handler = server.methodHandlers[name];

            if (handler) {
              _context4.next = 4;
              break;
            }

            throw AppError(404, 'Method \'' + name + '\' not found');

          case 4:
            _context4.prev = 4;
            _context4.t0 = EJSON;
            _context4.next = 8;
            return handler(EJSON.clone(args));

          case 8:
            _context4.t1 = _context4.sent;
            result = _context4.t0.clone.call(_context4.t0, _context4.t1);

            if (!callback) {
              _context4.next = 13;
              break;
            }

            callback(undefined, result); // eslint-disable-line
            return _context4.abrupt('return');

          case 13:
            return _context4.abrupt('return', result);

          case 16:
            _context4.prev = 16;
            _context4.t2 = _context4['catch'](4);

            if (!callback) {
              _context4.next = 22;
              break;
            }

            callback(_context4.t2);
            _context4.next = 23;
            break;

          case 22:
            throw _context4.t2;

          case 23:
          case 'end':
            return _context4.stop();
        }
      }
    }, _callee4, this, [[4, 16]]);
  }));

  return function applyMethod(_x8, _x9, _x10) {
    return _ref5.apply(this, arguments);
  };
}();

var callMethod = function () {
  var _ref6 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee5(name) {
    for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
      args[_key - 1] = arguments[_key];
    }

    var callback;
    return _regenerator2.default.wrap(function _callee5$(_context5) {
      while (1) {
        switch (_context5.prev = _context5.next) {
          case 0:
            if (args.length && typeof args[args.length - 1] === 'function') {
              callback = args.pop();
            }
            return _context5.abrupt('return', this.applyMethod(name, args, callback));

          case 2:
          case 'end':
            return _context5.stop();
        }
      }
    }, _callee5, this);
  }));

  return function callMethod(_x11) {
    return _ref6.apply(this, arguments);
  };
}();