'use strict';

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _create = require('babel-runtime/core-js/object/create');

var _create2 = _interopRequireDefault(_create);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// meteor/packages/callback-hook/hook.js
// babe85f268c773c4019057e07c63d2b21192c459

var _require = require('../common'),
    bindAndCatch = _require.bindAndCatch;

// XXX This pattern is under development. Do not add more callsites
// using this package for now. See:
// https://meteor.hackpad.com/Design-proposal-Hooks-YxvgEW06q6f
//
// Encapsulates the pattern of registering callbacks on a hook.
//
// The `each` method of the hook calls its iterator function argument
// with each registered callback.  This allows the hook to
// conditionally decide not to call the callback (if, for example, the
// observed object has been closed or terminated).
//
// By default, callbacks are bound with `Meteor.bindEnvironment`, so they will be
// called with the Meteor environment of the calling code that
// registered the callback. Override by passing { bindEnvironment: false }
// to the constructor.
//
// Registering a callback returns an object with a single `stop`
// method which unregisters the callback.
//
// The code is careful to allow a callback to be safely unregistered
// while the callbacks are being iterated over.
//
// If the hook is configured with the `exceptionHandler` option, the
// handler will be called if a called callback throws an exception.
// By default (if the exception handler doesn't itself throw an
// exception, or if the iterator function doesn't return a falsy value
// to terminate the calling of callbacks), the remaining callbacks
// will still be called.
//
// Alternatively, the `debugPrintExceptions` option can be specified
// as string describing the callback.  On an exception the string and
// the exception will be printed to the console log with
// `Meteor._debug`, and the exception otherwise ignored.
//
// If an exception handler isn't specified, exceptions thrown in the
// callback will propagate up to the iterator function, and will
// terminate calling the remaining callbacks if not caught.

var hasOwn = Object.prototype.hasOwnProperty;

var Hook = function () {
  function Hook(options) {
    (0, _classCallCheck3.default)(this, Hook);

    options = options || {};
    this.nextCallbackId = 0;
    this.callbacks = (0, _create2.default)(null);

    if (options.exceptionHandler) {
      this.exceptionHandler = options.exceptionHandler;
    } else if (options.debugPrintExceptions) {
      if (typeof options.debugPrintExceptions !== 'string') {
        throw new Error('Hook option debugPrintExceptions should be a string');
      }
      this.exceptionHandler = options.debugPrintExceptions;
    }
  }

  (0, _createClass3.default)(Hook, [{
    key: 'register',
    value: function register(callback) {
      var _this = this;

      var exceptionHandler = this.exceptionHandler || function (exception) {
        // Note: this relies on the undocumented fact that if bindEnvironment's
        // onException throws, and you are invoking the callback either in the
        // browser or from within a Fiber in Node, the exception is propagated.
        throw exception;
      };

      callback = bindAndCatch(callback, exceptionHandler);

      var id = this.nextCallbackId++;
      this.callbacks[id] = callback;

      return {
        stop: function stop() {
          delete _this.callbacks[id];
        }
      };
    }

    // For each registered callback, call the passed iterator function
    // with the callback.
    //
    // The iterator function can choose whether or not to call the
    // callback.  (For example, it might not call the callback if the
    // observed object has been closed or terminated).
    //
    // The iteration is stopped if the iterator function returns a falsy
    // value or throws an exception.
    //

  }, {
    key: 'each',
    value: function each(iterator) {
      var ids = (0, _keys2.default)(this.callbacks);
      for (var i = 0; i < ids.length; ++i) {
        var id = ids[i];
        // check to see if the callback was removed during iteration
        if (hasOwn.call(this.callbacks, id)) {
          var callback = this.callbacks[id];
          if (!iterator(callback)) {
            break;
          }
        }
      }
    }
  }, {
    key: 'eachAsync',
    value: function () {
      var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(iterator) {
        var ids, i, id, callback;
        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                ids = (0, _keys2.default)(this.callbacks);
                i = 0;

              case 2:
                if (!(i < ids.length)) {
                  _context.next = 13;
                  break;
                }

                id = ids[i];

                if (!hasOwn.call(this.callbacks, id)) {
                  _context.next = 10;
                  break;
                }

                callback = this.callbacks[id];
                _context.next = 8;
                return iterator(callback);

              case 8:
                if (_context.sent) {
                  _context.next = 10;
                  break;
                }

                return _context.abrupt('break', 13);

              case 10:
                ++i;
                _context.next = 2;
                break;

              case 13:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function eachAsync(_x) {
        return _ref.apply(this, arguments);
      }

      return eachAsync;
    }()
  }]);
  return Hook;
}();

module.exports = {
  Hook: Hook
};