'use strict';

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// meteor/packages/meteor/helpers.js
// 7f062192e1f9e4cb669d378b1140da9a33b0db42

/* eslint-disable no-redeclare */

// XXX find a better home for these? Ideally they would be _.get,
// _.ensure, _.delete..

// _get(a,b,c,d) returns a[b][c][d], or else undefined if a[b] or
// a[b][c] doesn't exist.
//

var _require = require('./debug'),
    debug = _require.debug;

var _get = function _get(obj /*, arguments */) {
  for (var i = 1; i < arguments.length; i++) {
    if (!(arguments[i] in obj)) {
      return undefined;
    }
    obj = obj[arguments[i]];
  }
  return obj;
};

// _ensure(a,b,c,d) ensures that a[b][c][d] exists. If it does not,
// it is created and set to {}. Either way, it is returned.
//
var _ensure = function _ensure(obj /*, arguments */) {
  for (var i = 1; i < arguments.length; i++) {
    var key = arguments[i];
    if (!(key in obj)) {
      obj[key] = {};
    }
    obj = obj[key];
  }

  return obj;
};

// _delete(a, b, c, d) deletes a[b][c][d], then a[b][c] unless it
// isn't empty, then a[b] unless it isn't empty.
//
var _delete = function _delete(obj /*, arguments */) {
  var stack = [obj];
  var leaf = true;
  for (var i = 1; i < arguments.length - 1; i++) {
    var key = arguments[i];
    if (!(key in obj)) {
      leaf = false;
      break;
    }
    obj = obj[key];
    if ((typeof obj === 'undefined' ? 'undefined' : (0, _typeof3.default)(obj)) !== 'object') {
      break;
    }
    stack.push(obj);
  }

  for (var i = stack.length - 1; i >= 0; i--) {
    var key = arguments[i + 1];

    if (leaf) {
      leaf = false;
    } else {
      for (var other in stack[i][key]) {
        return; // not empty -- we're done
      }
    }

    delete stack[i][key];
  }
};

// wrapAsync can wrap any function that takes some number of arguments that
// can't be undefined, followed by some optional arguments, where the callback
// is the last optional argument.
// e.g. fs.readFile(pathname, [callback]),
// fs.open(pathname, flags, [mode], [callback])
// For maximum effectiveness and least confusion, wrapAsync should be used on
// functions where the callback is the only argument of type Function.

/**
 * @memberOf Meteor
 * @summary Wrap a function that takes a callback function as its final parameter. The signature of the callback of the wrapped function should be `function(error, result){}`. On the server, the wrapped function can be used either synchronously (without passing a callback) or asynchronously (when a callback is passed). On the client, a callback is always required; errors will be logged if there is no callback. If a callback is provided, the environment captured when the original function was called will be restored in the callback.
 * @locus Anywhere
 * @param {Function} func A function that takes a callback as its final parameter
 * @param {Object} [context] Optional `this` object against which the original function will be invoked
 */
var wrapAsync = function wrapAsync(fn, context) {
  return function () /* arguments */{
    var self = context || this;
    var newArgs = Array.prototype.slice.call(arguments);
    var callback;

    for (var i = newArgs.length - 1; i >= 0; --i) {
      var arg = newArgs[i];
      var type = typeof arg === 'undefined' ? 'undefined' : (0, _typeof3.default)(arg);
      if (type !== 'undefined') {
        if (type === 'function') {
          callback = arg;
        }
        break;
      }
    }

    return new _promise2.default(function (resolve, reject) {
      var newCallback = function newCallback(err, results) {
        if (callback) {
          bindAndCatch(callback)(err, results);
        }

        if (err) {
          console.log('Exception in callback of async function', err.stack ? err.stack : err);
          reject(err);
        }
        resolve(results);
      };

      if (!callback) ++i;

      newArgs[i] = newCallback;

      _promise2.default.resolve(fn.apply(self, newArgs)).catch(function (err) {
        console.log('Exception in async function', err.stack ? err.stack : err);
        reject(err);
      });
    });
  };
};

// Sets child's prototype to a new object whose prototype is parent's
// prototype. Used as:
//   Meteor._inherits(ClassB, ClassA).
//   _.extend(ClassB.prototype, { ... })
// Inspired by CoffeeScript's `extend` and Google Closure's `goog.inherits`.
var hasOwn = Object.prototype.hasOwnProperty;
var _inherits = function _inherits(Child, Parent) {
  // copy Parent static properties
  for (var key in Parent) {
    // make sure we only copy hasOwnProperty properties vs. prototype
    // properties
    if (hasOwn.call(Parent, key)) {
      Child[key] = Parent[key];
    }
  }

  // a middle member of prototype chain: takes the prototype from the Parent
  var Middle = function Middle() {
    this.constructor = Child;
  };
  Middle.prototype = Parent.prototype;
  Child.prototype = new Middle();
  Child.__super__ = Parent.prototype;
  return Child;
};

var bindAndCatch = function bindAndCatch(func, onException, _this) {
  if (!onException || typeof onException === 'string') {
    var description = onException || 'callback of async function';
    onException = function onException(error) {
      debug('Exception in ' + description + ':', error && error.stack || error);
    };
  }

  return function () {
    try {
      for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      var ret = func.apply(_this, args);
      if (ret instanceof _promise2.default || ret && ret.then) {
        return ret.catch(onException);
      }
    } catch (e) {
      onException(e);
    }
    return ret;
  };
};

module.exports = {
  get: _get,
  ensure: _ensure,
  detele: _delete,
  wrapAsync: wrapAsync,
  inherits: _inherits,
  bindAndCatch: bindAndCatch
};