'use strict';

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// meteor/packages/ejson/stringify.js
// c6bddb331a8fd8c24336c6a7232406a3f108a6b5

// Based on json2.js from https://github.com/douglascrockford/JSON-js
//
//    json2.js
//    2012-10-08
//
//    Public Domain.
//
//    NO WARRANTY EXPRESSED OR IMPLIED. USE AT YOUR OWN RISK.

function quote(string) {
  return (0, _stringify2.default)(string);
}

var str = function str(key, holder, singleIndent, outerIndent, canonical) {
  var value = holder[key];

  // What happens next depends on the value's type.
  switch (typeof value === 'undefined' ? 'undefined' : (0, _typeof3.default)(value)) {
    case 'string':
      return quote(value);
    case 'number':
      // JSON numbers must be finite. Encode non-finite numbers as null.
      return isFinite(value) ? String(value) : 'null';
    case 'boolean':
      return String(value);
    // If the type is 'object', we might be dealing with an object or an array or
    // null.
    case 'object':
      // Due to a specification blunder in ECMAScript, typeof null is 'object',
      // so watch out for that case.
      if (!value) {
        return 'null';
      }
      // Make an array to hold the partial results of stringifying this object
      // value.
      var innerIndent = outerIndent + singleIndent;
      var partial = [];

      // Is the value an array?
      if (Array.isArray(value) || {}.hasOwnProperty.call(value, 'callee')) {
        // The value is an array. Stringify every element. Use null as a
        // placeholder for non-JSON values.
        var length = value.length;
        for (var i = 0; i < length; i += 1) {
          partial[i] = str(i, value, singleIndent, innerIndent, canonical) || 'null';
        }

        // Join all of the elements together, separated with commas, and wrap
        // them in brackets.
        var _v = void 0;
        if (partial.length === 0) {
          _v = '[]';
        } else if (innerIndent) {
          _v = '[\n' + innerIndent + partial.join(',\n' + innerIndent) + '\n' + outerIndent + ']';
        } else {
          _v = '[' + partial.join(',') + ']';
        }
        return _v;
      }

      // Iterate through all of the keys in the object.
      var keys = (0, _keys2.default)(value);
      if (canonical) {
        keys = keys.sort();
      }
      keys.forEach(function (k) {
        var v = str(k, value, singleIndent, innerIndent, canonical);
        if (v) {
          partial.push(quote(k) + (innerIndent ? ': ' : ':') + v);
        }
      });

      // Join all of the member texts together, separated with commas,
      // and wrap them in braces.
      var v;
      if (partial.length === 0) {
        v = '{}';
      } else if (innerIndent) {
        v = '{\n' + innerIndent + partial.join(',\n' + innerIndent) + '\n' + outerIndent + '}';
      } else {
        v = '{' + partial.join(',') + '}';
      }
      return v;

    default: // Do nothing
  }
};

// If the JSON object does not yet have a stringify method, give it one.
var canonicalStringify = function canonicalStringify(value, options) {
  // Make a fake root object containing our value under the key of ''.
  // Return the result of stringifying the value.
  var allOptions = (0, _assign2.default)({
    indent: '',
    canonical: false
  }, options);
  if (allOptions.indent === true) {
    allOptions.indent = '  ';
  } else if (typeof allOptions.indent === 'number') {
    var newIndent = '';
    for (var i = 0; i < allOptions.indent; i++) {
      newIndent += ' ';
    }
    allOptions.indent = newIndent;
  }
  return str('', { '': value }, allOptions.indent, '', allOptions.canonical);
};

module.exports = canonicalStringify;