'use strict';

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// meteor/packages/email/email.js
// ae1783bd4bcdf7aaf6efaa5db2ca824fed136d80

var _require = require('../common'),
    debug = _require.debug,
    wrapAsync = _require.wrapAsync;

/* eslint-disable camelcase */

var urlModule = require('url');
var nodemailer = require('node4mailer');

var Email = {};
var EmailTest = {};

var EmailInternals = {
  NpmModules: {
    mailcomposer: {
      version: require('node4mailer/package.json').version,
      module: require('node4mailer/lib/mail-composer')
    },
    nodemailer: {
      version: require('node4mailer/package.json').version,
      module: require('node4mailer')
    }
  }
};

var MailComposer = EmailInternals.NpmModules.mailcomposer.module;

var makeTransport = function makeTransport(mailUrlString) {
  var mailUrl = urlModule.parse(mailUrlString, true);

  if (mailUrl.protocol !== 'smtp:' && mailUrl.protocol !== 'smtps:') {
    throw new Error('Email protocol in $MAIL_URL (' + mailUrlString + ") must be 'smtp' or 'smtps'");
  }

  if (mailUrl.protocol === 'smtp:' && mailUrl.port === '465') {
    debug("The $MAIL_URL is 'smtp://...:465'.  " + "You probably want 'smtps://' (The 's' enables TLS/SSL) " + "since '465' is typically a secure port.");
  }

  // Allow overriding pool setting, but default to true.
  if (!mailUrl.query) {
    mailUrl.query = {};
  }

  if (!mailUrl.query.pool) {
    mailUrl.query.pool = 'true';
  }

  var transport = nodemailer.createTransport(urlModule.format(mailUrl));

  transport._asyncSendMail = wrapAsync(transport.sendMail, transport);
  return transport;
};

var cacheKey;
var cache;
var getTransport = function getTransport() {
  // We delay this check until the first call to Email.send, in case someone
  // set process.env.MAIL_URL in startup code. Then we store in a cache until
  // process.env.MAIL_URL changes.
  var url = process.env.MAIL_URL;
  if (cacheKey === undefined || cacheKey !== url) {
    cacheKey = url;
    cache = url ? makeTransport(url) : null;
  }
  return cache;
};

var nextDevModeMailId = 0;
var output_stream = process.stdout;

// Testing hooks
EmailTest.overrideOutputStream = function (stream) {
  nextDevModeMailId = 0;
  output_stream = stream;
};

EmailTest.restoreOutputStream = function () {
  output_stream = process.stdout;
};

var devModeSend = function () {
  var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(mail) {
    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            return _context.abrupt('return', new _promise2.default(function (resolve) {
              var devModeMailId = nextDevModeMailId++;

              var stream = output_stream;

              // This approach does not prevent other writers to stdout from interleaving.
              stream.write('====== BEGIN MAIL #' + devModeMailId + ' ======\n');
              stream.write('(Mail not sent; to enable sending, set the MAIL_URL ' + 'environment variable.)\n');
              var readStream = new MailComposer(mail).compile().createReadStream();
              readStream.pipe(stream, { end: false });

              readStream.on('end', function () {
                stream.write('====== END MAIL #' + devModeMailId + ' ======\n');
                resolve();
              });
            }));

          case 1:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, this);
  }));

  return function devModeSend(_x) {
    return _ref.apply(this, arguments);
  };
}();

var smtpSend = function () {
  var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(transport, mail) {
    return _regenerator2.default.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            return _context2.abrupt('return', transport._asyncSendMail(mail));

          case 1:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, this);
  }));

  return function smtpSend(_x2, _x3) {
    return _ref2.apply(this, arguments);
  };
}();

/**
 * Mock out email sending (eg, during a test.) This is private for now.
 *
 * f receives the arguments to Email.send and should return true to go
 * ahead and send the email (or at least, try subsequent hooks), or
 * false to skip sending.
 */
var sendHooks = [];
EmailTest.hookSend = function (f) {
  sendHooks.push(f);
};

/**
 * @summary Send an email. Throws an `Error` on failure to contact mail server
 * or if mail server returns an error. All fields should match
 * [RFC5322](http://tools.ietf.org/html/rfc5322) specification.
 *
 * If the `MAIL_URL` environment variable is set, actually sends the email.
 * Otherwise, prints the contents of the email to standard out.
 *
 * Note that this package is based on **mailcomposer 4**, so make sure to refer to
 * [the documentation](https://github.com/nodemailer/mailcomposer/blob/v4.0.1/README.md)
 * for that version when using the `attachments` or `mailComposer` options.
 *
 * @locus Server
 * @param {Object} options
 * @param {String} [options.from] "From:" address (required)
 * @param {String|String[]} options.to,cc,bcc,replyTo
 *   "To:", "Cc:", "Bcc:", and "Reply-To:" addresses
 * @param {String} [options.inReplyTo] Message-ID this message is replying to
 * @param {String|String[]} [options.references] Array (or space-separated string) of Message-IDs to refer to
 * @param {String} [options.messageId] Message-ID for this message; otherwise, will be set to a random value
 * @param {String} [options.subject]  "Subject:" line
 * @param {String} [options.text|html] Mail body (in plain text and/or HTML)
 * @param {String} [options.watchHtml] Mail body in HTML specific for Apple Watch
 * @param {String} [options.icalEvent] iCalendar event attachment
 * @param {Object} [options.headers] Dictionary of custom headers - e.g. `{ "header name": "header value" }`. To set an object under a header name, use `JSON.stringify` - e.g. `{ "header name": JSON.stringify({ tracking: { level: 'full' } }) }`.
 * @param {Object[]} [options.attachments] Array of attachment objects, as
 * described in the [mailcomposer documentation](https://github.com/nodemailer/mailcomposer/blob/v4.0.1/README.md#attachments).
 * @param {MailComposer} [options.mailComposer] A [MailComposer](https://nodemailer.com/extras/mailcomposer/#e-mail-message-fields)
 * object representing the message to be sent.  Overrides all other options.
 * You can create a `MailComposer` object via
 * `new EmailInternals.NpmModules.mailcomposer.module`.
 */
Email.send = function () {
  var _ref3 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3(options) {
    var i, transport;
    return _regenerator2.default.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            i = 0;

          case 1:
            if (!(i < sendHooks.length)) {
              _context3.next = 7;
              break;
            }

            if (sendHooks[i](options)) {
              _context3.next = 4;
              break;
            }

            return _context3.abrupt('return');

          case 4:
            i++;
            _context3.next = 1;
            break;

          case 7:

            if (options.mailComposer) {
              options = options.mailComposer.mail;
            }

            transport = getTransport();

            if (!transport) {
              _context3.next = 14;
              break;
            }

            _context3.next = 12;
            return smtpSend(transport, options);

          case 12:
            _context3.next = 16;
            break;

          case 14:
            _context3.next = 16;
            return devModeSend(options);

          case 16:
          case 'end':
            return _context3.stop();
        }
      }
    }, _callee3, this);
  }));

  return function (_x4) {
    return _ref3.apply(this, arguments);
  };
}();

module.exports = {
  Email: Email,
  EmailTest: EmailTest,
  EmailInternals: EmailInternals
};