'use strict';

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// meteor/packages/http/httpcall_server.js
// 21275011bdb76d28db4ccd9556d9507e78d94f1d

/* eslint-disable camelcase */

var _require = require('../common'),
    wrapAsync = _require.wrapAsync;

var request = require('request');

var _require2 = require('../url/server'),
    URL = _require2.URL;

var MAX_LENGTH = 500;
var slice = Array.prototype.slice;

var makeErrorByStatus = function makeErrorByStatus(statusCode, content) {
  var message = 'failed [' + statusCode + ']';

  if (content) {
    var stringContent = typeof content === 'string' ? content : content.toString();

    message += ' ' + truncate(stringContent.replace(/\n/g, ' '), MAX_LENGTH);
  }

  return new Error(message);
};

function truncate(str, length) {
  return str.length > length ? str.slice(0, length) + '...' : str;
}

// Fill in `response.data` if the content-type is JSON.
var populateData = function populateData(response) {
  // Read Content-Type header, up to a ';' if there is one.
  // A typical header might be "application/json; charset=utf-8"
  // or just "application/json".
  var contentType = (response.headers['content-type'] || ';').split(';')[0];

  // Only try to parse data as JSON if server sets correct content type.
  if (['application/json', 'text/javascript', 'application/javascript', 'application/x-javascript'].indexOf(contentType) >= 0) {
    try {
      response.data = JSON.parse(response.content);
    } catch (err) {
      response.data = null;
    }
  } else {
    response.data = null;
  }
};

var HTTP = {};

HTTP.get = function () /* varargs */{
  return HTTP.call.apply(this, ['GET'].concat(slice.call(arguments)));
};

HTTP.post = function () /* varargs */{
  return HTTP.call.apply(this, ['POST'].concat(slice.call(arguments)));
};

HTTP.put = function () /* varargs */{
  return HTTP.call.apply(this, ['PUT'].concat(slice.call(arguments)));
};

HTTP.del = function () /* varargs */{
  return HTTP.call.apply(this, ['DELETE'].concat(slice.call(arguments)));
};

HTTP.patch = function () /* varargs */{
  return HTTP.call.apply(this, ['PATCH'].concat(slice.call(arguments)));
};

var hasOwn = Object.prototype.hasOwnProperty;

var HTTPInternals = {
  NpmModules: {
    request: {
      version: require('request/package.json').version,
      module: request
    }
  }

  // _call always runs asynchronously; HTTP.call, defined below,
  // wraps _call and runs synchronously when no callback is provided.
};function _call(method, url, options, callback) {
  // Process arguments

  if (!callback && typeof options === 'function') {
    // support (method, url, callback) argument list
    callback = options;
    options = null;
  }

  options = options || {};

  if (hasOwn.call(options, 'beforeSend')) {
    throw new Error('Option beforeSend not supported on server.');
  }

  method = (method || '').toUpperCase();

  if (!/^https?:\/\//.test(url)) {
    throw new Error('url must be absolute and start with http:// or https://');
  }

  var headers = {};

  var content = options.content;
  if (options.data) {
    content = (0, _stringify2.default)(options.data);
    headers['Content-Type'] = 'application/json';
  }

  var paramsForUrl, paramsForBody;
  if (content || method === 'GET' || method === 'HEAD') {
    paramsForUrl = options.params;
  } else {
    paramsForBody = options.params;
  }

  var newUrl = URL._constructUrl(url, options.query, paramsForUrl);

  if (options.auth) {
    if (options.auth.indexOf(':') < 0) {
      throw new Error('auth option should be of the form "username:password"');
    }
    headers['Authorization'] = 'Basic ' + Buffer.from(options.auth, 'ascii').toString('base64');
  }

  if (paramsForBody) {
    content = URL._encodeParams(paramsForBody);
    headers['Content-Type'] = 'application/x-www-form-urlencoded';
  }

  if (options.headers) {
    (0, _keys2.default)(options.headers).forEach(function (key) {
      headers[key] = options.headers[key];
    });
  }

  // wrap callback to add a 'response' property on an error, in case
  // we have both (http 4xx/5xx error, which has a response payload)
  callback = function (callback) {
    var called = false;
    return function (error, response) {
      if (!called) {
        called = true;
        if (error && response) {
          error.response = response;
        }
        callback(error, response);
      }
    };
  }(callback);

  // Kickoff!

  // Allow users to override any request option with the npmRequestOptions
  // option.
  var reqOptions = (0, _assign2.default)({
    url: newUrl,
    method: method,
    encoding: 'utf8',
    jar: false,
    timeout: options.timeout,
    body: content,
    followRedirect: options.followRedirects,
    // Follow redirects on non-GET requests
    // also. (https://github.com/meteor/meteor/issues/2808)
    followAllRedirects: options.followRedirects,
    headers: headers
  }, options.npmRequestOptions || null);

  request(reqOptions, function (error, res, body) {
    var response = null;

    if (!error) {
      response = {};
      response.statusCode = res.statusCode;
      response.content = body;
      response.headers = res.headers;

      populateData(response);

      if (response.statusCode >= 400) {
        error = makeErrorByStatus(response.statusCode, response.content);
      }
    }

    callback(error, response);
  });
}

HTTP.call = wrapAsync(_call);

module.exports = {
  HTTP: HTTP,
  HTTPInternals: HTTPInternals
};