'use strict';

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _create = require('babel-runtime/core-js/object/create');

var _create2 = _interopRequireDefault(_create);

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// meteor/packages/id-map/id-map.js
// e84ae21a1e60b0870b5638ecf56f0f1e9da13545

/* eslint-disable no-useless-call */

var _require = require('../ejson'),
    EJSON = _require.EJSON;

var hasOwn = Object.prototype.hasOwnProperty;

var IdMap = function () {
  function IdMap(idStringify, idParse) {
    (0, _classCallCheck3.default)(this, IdMap);

    this.clear();
    this._idStringify = idStringify || _stringify2.default;
    this._idParse = idParse || JSON.parse;
  }

  // Some of these methods are designed to match methods on OrderedDict, since
  // (eg) ObserveMultiplex and _CachingChangeObserver use them interchangeably.
  // (Conceivably, this should be replaced with "UnorderedDict" with a specific
  // set of methods that overlap between the two.)

  (0, _createClass3.default)(IdMap, [{
    key: 'get',
    value: function get(id) {
      var key = this._idStringify(id);
      return this._map[key];
    }
  }, {
    key: 'set',
    value: function set(id, value) {
      var key = this._idStringify(id);
      this._map[key] = value;
    }
  }, {
    key: 'remove',
    value: function remove(id) {
      var key = this._idStringify(id);
      delete this._map[key];
    }
  }, {
    key: 'has',
    value: function has(id) {
      var key = this._idStringify(id);
      return hasOwn.call(this._map, key);
    }
  }, {
    key: 'empty',
    value: function empty() {
      for (var key in this._map) {
        return false;
      }
      return true;
    }
  }, {
    key: 'clear',
    value: function clear() {
      this._map = (0, _create2.default)(null);
    }

    // Iterates over the items in the map. Return `false` to break the loop.

  }, {
    key: 'forEach',
    value: function forEach(iterator) {
      // don't use _.each, because we can't break out of it.
      var keys = (0, _keys2.default)(this._map);
      for (var i = 0; i < keys.length; i++) {
        var breakIfFalse = iterator.call(null, this._map[keys[i]], this._idParse(keys[i]));
        if (breakIfFalse === false) {
          return;
        }
      }
    }
  }, {
    key: 'size',
    value: function size() {
      return (0, _keys2.default)(this._map).length;
    }
  }, {
    key: 'setDefault',
    value: function setDefault(id, def) {
      var key = this._idStringify(id);
      if (hasOwn.call(this._map, key)) {
        return this._map[key];
      }
      this._map[key] = def;
      return def;
    }

    // Assumes that values are EJSON-cloneable, and that we don't need to clone
    // IDs (ie, that nobody is going to mutate an ObjectId).

  }, {
    key: 'clone',
    value: function clone() {
      var clone = new IdMap(this._idStringify, this._idParse);
      this.forEach(function (value, id) {
        clone.set(id, EJSON.clone(value));
      });
      return clone;
    }
  }]);
  return IdMap;
}();

module.exports = {
  IdMap: IdMap
};