'use strict';

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// meteor/packages/mongo-id/id.js
// b3096e93661bc79bab73a63bae0e14643030a9a3

var _require = require('../random'),
    Random = _require.Random;

var _require2 = require('../ejson'),
    EJSON = _require2.EJSON;

var MongoID = {};

MongoID._looksLikeObjectID = function (str) {
  return str.length === 24 && str.match(/^[0-9a-f]*$/);
};

MongoID.ObjectID = function (hexString) {
  // random-based impl of Mongo ObjectID
  var self = this;
  if (hexString) {
    hexString = hexString.toLowerCase();
    if (!MongoID._looksLikeObjectID(hexString)) {
      throw new Error('Invalid hexadecimal string for creating an ObjectID');
    }
    // meant to work with _.isEqual(), which relies on structural equality
    self._str = hexString;
  } else {
    self._str = Random.hexString(24);
  }
};

MongoID.ObjectID.prototype.toString = function () {
  var self = this;
  return 'ObjectID("' + self._str + '")';
};

MongoID.ObjectID.prototype.equals = function (other) {
  var self = this;
  return other instanceof MongoID.ObjectID && self.valueOf() === other.valueOf();
};

MongoID.ObjectID.prototype.clone = function () {
  var self = this;
  return new MongoID.ObjectID(self._str);
};

MongoID.ObjectID.prototype.typeName = function () {
  return 'oid';
};

MongoID.ObjectID.prototype.getTimestamp = function () {
  var self = this;
  return parseInt(self._str.substr(0, 8), 16);
};

MongoID.ObjectID.prototype.valueOf = MongoID.ObjectID.prototype.toJSONValue = MongoID.ObjectID.prototype.toHexString = function () {
  return this._str;
};

EJSON.addType('oid', function (str) {
  return new MongoID.ObjectID(str);
});

MongoID.idStringify = function (id) {
  if (id instanceof MongoID.ObjectID) {
    return id.valueOf();
  } else if (typeof id === 'string') {
    if (id === '') {
      return id;
    } else if (id.substr(0, 1) === '-' || // escape previously dashed strings
    id.substr(0, 1) === '~' || // escape escaped numbers, true, false
    MongoID._looksLikeObjectID(id) || // escape object-id-form strings
    id.substr(0, 1) === '{') {
      // escape object-form strings, for maybe implementing later
      return '-' + id;
    } else {
      return id; // other strings go through unchanged.
    }
  } else if (id === undefined) {
    return '-';
  } else if ((typeof id === 'undefined' ? 'undefined' : (0, _typeof3.default)(id)) === 'object' && id !== null) {
    throw new Error('does not currently support objects other than ObjectID as ids');
  } else {
    // Numbers, true, false, null
    return '~' + (0, _stringify2.default)(id);
  }
};

MongoID.idParse = function (id) {
  if (id === '') {
    return id;
  } else if (id === '-') {
    return undefined;
  } else if (id.substr(0, 1) === '-') {
    return id.substr(1);
  } else if (id.substr(0, 1) === '~') {
    return JSON.parse(id.substr(1));
  } else if (MongoID._looksLikeObjectID(id)) {
    return new MongoID.ObjectID(id);
  } else {
    return id;
  }
};

module.exports = {
  MongoID: MongoID
};