'use strict';

var _defineProperty = require('babel-runtime/core-js/object/define-property');

var _defineProperty2 = _interopRequireDefault(_defineProperty);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _getOwnPropertyDescriptors = require('babel-runtime/core-js/object/get-own-property-descriptors');

var _getOwnPropertyDescriptors2 = _interopRequireDefault(_getOwnPropertyDescriptors);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _create = require('babel-runtime/core-js/object/create');

var _create2 = _interopRequireDefault(_create);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// meteor/packages/mongo/collection.js
// b06a6af335427d57ac47963a6989ced482f5e5f9

var _require = require('../random'),
    Random = _require.Random;

var _require2 = require('../check'),
    check = _require2.check,
    Match = _require2.Match;

var _require3 = require('../mongo-id'),
    MongoID = _require3.MongoID;

var _require4 = require('../common'),
    debug = _require4.debug;

var _require5 = require('./namespace'),
    Mongo = _require5.Mongo,
    MongoInternals = _require5.MongoInternals;

var _require6 = require('./utils'),
    wrapTransform = _require6.wrapTransform,
    _selectorIsId = _require6._selectorIsId;

// options.connection, if given, is a LivedataClient or LivedataServer
// XXX presently there is no way to destroy/clean up a Collection

/**
 * @summary Constructor for a Collection
 * @locus Anywhere
 * @instancename collection
 * @class
 * @param {String} name The name of the collection.  If null, creates an unmanaged (unsynchronized) local collection.
 * @param {Object} [options]
 * @param {Object} options.connection The server connection that will manage this collection. Uses the default connection if not specified.  Pass the return value of calling [`DDP.connect`](#ddp_connect) to specify a different server. Pass `null` to specify no connection. Unmanaged (`name` is null) collections cannot specify a connection.
 * @param {String} options.idGeneration The method of generating the `_id` fields of new documents in this collection.  Possible values:
 - **`'STRING'`**: random strings
 - **`'MONGO'`**:  random [`Mongo.ObjectID`](#mongo_object_id) values
The default id generation technique is `'STRING'`.
 * @param {Function} options.transform An optional transformation function. Documents will be passed through this function before being returned from `fetch` or `findOne`, and before being passed to callbacks of `observe`, `map`, `forEach`, `allow`, and `deny`. Transforms are *not* applied for the callbacks of `observeChanges` or to cursors returned from publish functions.
 * @param {Boolean} options.defineMutationMethods Set to `false` to skip setting up the mutation methods that enable insert/update/remove from client code. Default `true`.
 */


Mongo.Collection = function () {
  function Collection(name, options) {
    (0, _classCallCheck3.default)(this, Collection);

    this.init(name, options);
  }

  // Allow function-based subclasses to invoke the body of the
  // Mongo.Collection constructor using .call or .apply, since that's how
  // super(...) was implemented before native ECMAScript class syntax.


  (0, _createClass3.default)(Collection, [{
    key: 'init',
    value: function init(name, options) {
      if (!name && name !== null) {
        debug('Warning: creating anonymous collection. It will not be ' + 'saved or synchronized over the network. (Pass null for ' + 'the collection name to turn off this warning.)');
        name = null;
      }

      if (name !== null && typeof name !== 'string') {
        throw new Error('First argument to new Mongo.Collection must be a string or null');
      }

      options = (0, _extends3.default)({
        idGeneration: 'STRING',
        transform: null,
        _driver: undefined
      }, options);

      switch (options.idGeneration) {
        case 'MONGO':
          this._makeNewID = function () {
            var src = Random.insecure;
            return new Mongo.ObjectID(src.hexString(24));
          };
          break;
        case 'STRING':
        default:
          this._makeNewID = function () {
            var src = Random.insecure;
            return src.id();
          };
          break;
      }

      this._transform = wrapTransform(options.transform);

      if (!options._driver) {
        // XXX This check assumes that webapp is loaded so that Meteor.server !==
        // null. We should fully support the case of "want to use a Mongo-backed
        // collection from Node code without webapp", but we don't yet.
        // #MeteorServerNull
        if (name && typeof MongoInternals !== 'undefined' && MongoInternals.defaultRemoteCollectionDriver) {
          options._driver = MongoInternals.defaultRemoteCollectionDriver();
        } else {
          throw new Error('Remote collection driver required');
        }
      }

      this._collection = options._driver.open(name);
      this._name = name;
      this._driver = options._driver;
    }

    //
    /// Main collection API
    //

  }, {
    key: '_getFindSelector',
    value: function _getFindSelector(args) {
      if (args.length === 0) {
        return {};
      } else {
        return args[0];
      }
    }
  }, {
    key: '_getFindOptions',
    value: function _getFindOptions(args) {
      var self = this;
      if (args.length < 2) {
        return { transform: self._transform };
      } else {
        check(args[1], Match.Optional(Match.ObjectIncluding({
          fields: Match.Optional(Match.OneOf(Object, undefined)),
          sort: Match.Optional(Match.OneOf(Object, Array, Function, undefined)),
          limit: Match.Optional(Match.OneOf(Number, undefined)),
          skip: Match.Optional(Match.OneOf(Number, undefined))
        })));

        return (0, _extends3.default)({
          transform: self._transform
        }, args[1]);
      }
    }

    /**
     * @summary Find the documents in a collection that match the selector.
     * @locus Anywhere
     * @method find
     * @memberOf Mongo.Collection
     * @instance
     * @param {MongoSelector} [selector] A query describing the documents to find
     * @param {Object} [options]
     * @param {MongoSortSpecifier} options.sort Sort order (default: natural order)
     * @param {Number} options.skip Number of results to skip at the beginning
     * @param {Number} options.limit Maximum number of results to return
     * @param {MongoFieldSpecifier} options.fields Dictionary of fields to return or exclude.
     * @param {Boolean} options.reactive (Client only) Default `true`; pass `false` to disable reactivity
     * @param {Function} options.transform Overrides `transform` on the  [`Collection`](#collections) for this cursor.  Pass `null` to disable transformation.
     * @param {Boolean} options.disableOplog (Server only) Pass true to disable oplog-tailing on this query. This affects the way server processes calls to `observe` on this query. Disabling the oplog can be useful when working with data that updates in large batches.
     * @param {Number} options.pollingIntervalMs (Server only) When oplog is disabled (through the use of `disableOplog` or when otherwise not available), the frequency (in milliseconds) of how often to poll this query when observing on the server. Defaults to 10000ms (10 seconds).
     * @param {Number} options.pollingThrottleMs (Server only) When oplog is disabled (through the use of `disableOplog` or when otherwise not available), the minimum time (in milliseconds) to allow between re-polling when observing on the server. Increasing this will save CPU and mongo load at the expense of slower updates to users. Decreasing this is not recommended. Defaults to 50ms.
     * @param {Number} options.maxTimeMs (Server only) If set, instructs MongoDB to set a time limit for this cursor's operations. If the operation reaches the specified time limit (in milliseconds) without the having been completed, an exception will be thrown. Useful to prevent an (accidental or malicious) unoptimized query from causing a full collection scan that would disrupt other database users, at the expense of needing to handle the resulting error.
     * @param {String|Object} options.hint (Server only) Overrides MongoDB's default index selection and query optimization process. Specify an index to force its use, either by its name or index specification. You can also specify `{ $natural : 1 }` to force a forwards collection scan, or `{ $natural : -1 }` for a reverse collection scan. Setting this is only recommended for advanced users.
     * @returns {Mongo.Cursor}
     */

  }, {
    key: 'find',
    value: function find() {
      for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      // Collection.find() (return all docs) behaves differently
      // from Collection.find(undefined) (return 0 docs).  so be
      // careful about the length of arguments.
      return this._collection.find(this._getFindSelector(args), this._getFindOptions(args));
    }

    /**
     * @summary Finds the first document that matches the selector, as ordered by sort and skip options. Returns `undefined` if no matching document is found.
     * @locus Anywhere
     * @method findOne
     * @memberOf Mongo.Collection
     * @instance
     * @param {MongoSelector} [selector] A query describing the documents to find
     * @param {Object} [options]
     * @param {MongoSortSpecifier} options.sort Sort order (default: natural order)
     * @param {Number} options.skip Number of results to skip at the beginning
     * @param {MongoFieldSpecifier} options.fields Dictionary of fields to return or exclude.
     * @param {Boolean} options.reactive (Client only) Default true; pass false to disable reactivity
     * @param {Function} options.transform Overrides `transform` on the [`Collection`](#collections) for this cursor.  Pass `null` to disable transformation.
     * @returns {Object}
     */

  }, {
    key: 'findOne',
    value: function () {
      var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee() {
        for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
          args[_key2] = arguments[_key2];
        }

        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                return _context.abrupt('return', this._collection.findOne(this._getFindSelector(args), this._getFindOptions(args)));

              case 1:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function findOne() {
        return _ref.apply(this, arguments);
      }

      return findOne;
    }()

    // protect against dangerous selectors.  falsey and {_id: falsey} are both
    // likely programmer error, and not what you want, particularly for destructive
    // operations. If a falsey _id is sent in, a new string _id will be
    // generated and returned; if a fallbackId is provided, it will be returned
    // instead.

  }, {
    key: 'insert',


    // 'insert' immediately returns the inserted document's new _id.
    // The others return values immediately if you are in a stub, an in-memory
    // unmanaged collection, or a mongo-backed collection and you don't pass a
    // callback. 'update' and 'remove' return the number of affected
    // documents. 'upsert' returns an object with keys 'numberAffected' and, if an
    // insert happened, 'insertedId'.
    //
    // Otherwise, the semantics are exactly like other methods: they take
    // a callback as an optional last argument; if no callback is
    // provided, they block until the operation is complete, and throw an
    // exception if it fails; if a callback is provided, then they don't
    // necessarily block, and they call the callback when they finish with error and
    // result arguments.  (The insert method provides the document ID as its result
    // update and remove provide the number of affected docs as the result; upsert
    // provides an object with numberAffected and maybe insertedId.)
    //
    // On the client, blocking is impossible, so if a callback
    // isn't provided, they just return immediately and any error
    // information is lost.
    //
    // There's one more tweak. On the client, if you don't provide a
    // callback, then if there is an error, a message will be logged with
    // Meteor._debug.
    //
    // The intent (though this is actually determined by the underlying
    // drivers) is that the operations should be done synchronously, not
    // generating their result until the database has acknowledged
    // them. In the future maybe we should provide a flag to turn this
    // off.

    /**
     * @summary Insert a document in the collection.  Returns its unique _id.
     * @locus Anywhere
     * @method  insert
     * @memberOf Mongo.Collection
     * @instance
     * @param {Object} doc The document to insert. May not yet have an _id attribute, in which case Meteor will generate one for you.
     * @param {Function} [callback] Optional.  If present, called with an error object as the first argument and, if no error, the _id as the second.
     */
    value: function () {
      var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(doc, callback) {
        var chooseReturnValueFromCollectionResult, wrappedCallback, result;
        return _regenerator2.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                if (doc) {
                  _context2.next = 2;
                  break;
                }

                throw new Error('insert requires an argument');

              case 2:

                // Make a shallow clone of the document, preserving its prototype.
                doc = (0, _create2.default)((0, _getPrototypeOf2.default)(doc), (0, _getOwnPropertyDescriptors2.default)(doc));

                if (!('_id' in doc)) {
                  _context2.next = 8;
                  break;
                }

                if (!(!doc._id || !(typeof doc._id === 'string' || doc._id instanceof Mongo.ObjectID))) {
                  _context2.next = 6;
                  break;
                }

                throw new Error('requires document _id fields to be non-empty strings or ObjectIDs');

              case 6:
                _context2.next = 9;
                break;

              case 8:
                doc._id = this._makeNewID();

              case 9:

                // On inserts, always return the id that we generated; on all other
                // operations, just return the result from the collection.
                chooseReturnValueFromCollectionResult = function chooseReturnValueFromCollectionResult(result) {
                  if (doc._id) {
                    return doc._id;
                  }

                  // XXX what is this for??
                  // It's some iteraction between the callback to _callMutatorMethod and
                  // the return value conversion
                  doc._id = result;

                  return result;
                };

                wrappedCallback = wrapCallback(callback, chooseReturnValueFromCollectionResult);

                // it's my collection.  descend into the collection object
                // and propagate any exception.

                _context2.prev = 11;
                _context2.next = 14;
                return this._collection.insert(doc, wrappedCallback);

              case 14:
                result = _context2.sent;
                return _context2.abrupt('return', chooseReturnValueFromCollectionResult(result));

              case 18:
                _context2.prev = 18;
                _context2.t0 = _context2['catch'](11);

                if (!callback) {
                  _context2.next = 23;
                  break;
                }

                callback(_context2.t0);
                return _context2.abrupt('return', null);

              case 23:
                throw _context2.t0;

              case 24:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this, [[11, 18]]);
      }));

      function insert(_x, _x2) {
        return _ref2.apply(this, arguments);
      }

      return insert;
    }()

    /**
     * @summary Modify one or more documents in the collection. Returns the number of matched documents.
     * @locus Anywhere
     * @method update
     * @memberOf Mongo.Collection
     * @instance
     * @param {MongoSelector} selector Specifies which documents to modify
     * @param {MongoModifier} modifier Specifies how to modify the documents
     * @param {Object} [options]
     * @param {Boolean} options.multi True to modify all matching documents; false to only modify one of the matching documents (the default).
     * @param {Boolean} options.upsert True to insert a document if no matching documents are found.
     * @param {Function} [callback] Optional.  If present, called with an error object as the first argument and, if no error, the number of affected documents as the second.
     */

  }, {
    key: 'update',
    value: function () {
      var _ref3 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3(selector, modifier) {
        for (var _len3 = arguments.length, optionsAndCallback = Array(_len3 > 2 ? _len3 - 2 : 0), _key3 = 2; _key3 < _len3; _key3++) {
          optionsAndCallback[_key3 - 2] = arguments[_key3];
        }

        var callback, options, insertedId, wrappedCallback;
        return _regenerator2.default.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                callback = popCallbackFromArgs(optionsAndCallback);

                // We've already popped off the callback, so we are left with an array
                // of one or zero items

                options = (0, _extends3.default)({}, optionsAndCallback[0] || null);
                insertedId = void 0;

                if (!(options && options.upsert)) {
                  _context3.next = 11;
                  break;
                }

                if (!options.insertedId) {
                  _context3.next = 10;
                  break;
                }

                if (typeof options.insertedId === 'string' || options.insertedId instanceof Mongo.ObjectID) {
                  _context3.next = 7;
                  break;
                }

                throw new Error('insertedId must be string or ObjectID');

              case 7:
                insertedId = options.insertedId;
                _context3.next = 11;
                break;

              case 10:
                if (!selector || !selector._id) {
                  insertedId = this._makeNewID();
                  options.generatedId = true;
                  options.insertedId = insertedId;
                }

              case 11:

                selector = Mongo.Collection._rewriteSelector(selector, {
                  fallbackId: insertedId
                });

                wrappedCallback = wrapCallback(callback);

                // it's my collection.  descend into the collection object
                // and propagate any exception.

                _context3.prev = 13;
                return _context3.abrupt('return', this._collection.update(selector, modifier, options, wrappedCallback));

              case 17:
                _context3.prev = 17;
                _context3.t0 = _context3['catch'](13);

                if (!callback) {
                  _context3.next = 22;
                  break;
                }

                callback(_context3.t0);
                return _context3.abrupt('return', null);

              case 22:
                throw _context3.t0;

              case 23:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, this, [[13, 17]]);
      }));

      function update(_x3, _x4) {
        return _ref3.apply(this, arguments);
      }

      return update;
    }()

    /**
     * @summary Remove documents from the collection
     * @locus Anywhere
     * @method remove
     * @memberOf Mongo.Collection
     * @instance
     * @param {MongoSelector} selector Specifies which documents to remove
     * @param {Function} [callback] Optional.  If present, called with an error object as its argument.
     */

  }, {
    key: 'remove',
    value: function () {
      var _ref4 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee4(selector, callback) {
        var wrappedCallback;
        return _regenerator2.default.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                selector = Mongo.Collection._rewriteSelector(selector);

                wrappedCallback = wrapCallback(callback);

                // it's my collection.  descend into the collection object
                // and propagate any exception.

                _context4.prev = 2;
                return _context4.abrupt('return', this._collection.remove(selector, wrappedCallback));

              case 6:
                _context4.prev = 6;
                _context4.t0 = _context4['catch'](2);

                if (!callback) {
                  _context4.next = 11;
                  break;
                }

                callback(_context4.t0);
                return _context4.abrupt('return', null);

              case 11:
                throw _context4.t0;

              case 12:
              case 'end':
                return _context4.stop();
            }
          }
        }, _callee4, this, [[2, 6]]);
      }));

      function remove(_x5, _x6) {
        return _ref4.apply(this, arguments);
      }

      return remove;
    }()

    /**
     * @summary Modify one or more documents in the collection, or insert one if no matching documents were found. Returns an object with keys `numberAffected` (the number of documents modified)  and `insertedId` (the unique _id of the document that was inserted, if any).
     * @locus Anywhere
     * @param {MongoSelector} selector Specifies which documents to modify
     * @param {MongoModifier} modifier Specifies how to modify the documents
     * @param {Object} [options]
     * @param {Boolean} options.multi True to modify all matching documents; false to only modify one of the matching documents (the default).
     * @param {Function} [callback] Optional.  If present, called with an error object as the first argument and, if no error, the number of affected documents as the second.
     */

  }, {
    key: 'upsert',
    value: function upsert(selector, modifier, options, callback) {
      if (!callback && typeof options === 'function') {
        callback = options;
        options = {};
      }

      return this.update(selector, modifier, (0, _extends3.default)({}, options, {
        _returnObject: true,
        upsert: true
      }), callback);
    }

    // We'll actually design an index API later. For now, we just pass through to
    // Mongo's, but make it synchronous.

  }, {
    key: '_ensureIndex',
    value: function () {
      var _ref5 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee5(index, options) {
        var self;
        return _regenerator2.default.wrap(function _callee5$(_context5) {
          while (1) {
            switch (_context5.prev = _context5.next) {
              case 0:
                self = this;

                if (self._collection._ensureIndex) {
                  _context5.next = 3;
                  break;
                }

                throw new Error('Can only call _ensureIndex on server collections');

              case 3:
                _context5.next = 5;
                return self._collection._ensureIndex(index, options);

              case 5:
              case 'end':
                return _context5.stop();
            }
          }
        }, _callee5, this);
      }));

      function _ensureIndex(_x7, _x8) {
        return _ref5.apply(this, arguments);
      }

      return _ensureIndex;
    }()
  }, {
    key: '_dropIndex',
    value: function () {
      var _ref6 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee6(index) {
        var self;
        return _regenerator2.default.wrap(function _callee6$(_context6) {
          while (1) {
            switch (_context6.prev = _context6.next) {
              case 0:
                self = this;

                if (self._collection._dropIndex) {
                  _context6.next = 3;
                  break;
                }

                throw new Error('Can only call _dropIndex on server collections');

              case 3:
                _context6.next = 5;
                return self._collection._dropIndex(index);

              case 5:
              case 'end':
                return _context6.stop();
            }
          }
        }, _callee6, this);
      }));

      function _dropIndex(_x9) {
        return _ref6.apply(this, arguments);
      }

      return _dropIndex;
    }()
  }, {
    key: '_dropCollection',
    value: function () {
      var _ref7 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee7() {
        var self;
        return _regenerator2.default.wrap(function _callee7$(_context7) {
          while (1) {
            switch (_context7.prev = _context7.next) {
              case 0:
                self = this;

                if (self._collection.dropCollection) {
                  _context7.next = 3;
                  break;
                }

                throw new Error('Can only call _dropCollection on server collections');

              case 3:
                _context7.next = 5;
                return self._collection.dropCollection();

              case 5:
              case 'end':
                return _context7.stop();
            }
          }
        }, _callee7, this);
      }));

      function _dropCollection() {
        return _ref7.apply(this, arguments);
      }

      return _dropCollection;
    }()
  }, {
    key: '_createCappedCollection',
    value: function () {
      var _ref8 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee8(byteSize, maxDocuments) {
        var self;
        return _regenerator2.default.wrap(function _callee8$(_context8) {
          while (1) {
            switch (_context8.prev = _context8.next) {
              case 0:
                self = this;

                if (self._collection._createCappedCollection) {
                  _context8.next = 3;
                  break;
                }

                throw new Error('Can only call _createCappedCollection on server collections');

              case 3:
                _context8.next = 5;
                return self._collection._createCappedCollection(byteSize, maxDocuments);

              case 5:
              case 'end':
                return _context8.stop();
            }
          }
        }, _callee8, this);
      }));

      function _createCappedCollection(_x10, _x11) {
        return _ref8.apply(this, arguments);
      }

      return _createCappedCollection;
    }()

    /**
     * @summary Returns the [`Collection`](http://mongodb.github.io/node-mongodb-native/2.2/api/Collection.html) object corresponding to this collection from the [npm `mongodb` driver module](https://www.npmjs.com/package/mongodb) which is wrapped by `Mongo.Collection`.
     * @locus Server
     */

  }, {
    key: 'rawCollection',
    value: function () {
      var _ref9 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee9() {
        var self;
        return _regenerator2.default.wrap(function _callee9$(_context9) {
          while (1) {
            switch (_context9.prev = _context9.next) {
              case 0:
                self = this;

                if (self._collection.rawCollection) {
                  _context9.next = 3;
                  break;
                }

                throw new Error('Can only call rawCollection on server collections');

              case 3:
                return _context9.abrupt('return', self._collection.rawCollection());

              case 4:
              case 'end':
                return _context9.stop();
            }
          }
        }, _callee9, this);
      }));

      function rawCollection() {
        return _ref9.apply(this, arguments);
      }

      return rawCollection;
    }()

    /**
     * @summary Returns the [`Db`](http://mongodb.github.io/node-mongodb-native/2.2/api/Db.html) object corresponding to this collection's database connection from the [npm `mongodb` driver module](https://www.npmjs.com/package/mongodb) which is wrapped by `Mongo.Collection`.
     * @locus Server
     */

  }, {
    key: 'rawDatabase',
    value: function rawDatabase() {
      var self = this;
      if (!(self._driver.mongo && self._driver.mongo.db)) {
        throw new Error('Can only call rawDatabase on server collections');
      }
      return self._driver.mongo.db;
    }
  }], [{
    key: 'call',
    value: function call(obj) {
      if (obj instanceof this) {
        for (var _len4 = arguments.length, args = Array(_len4 > 1 ? _len4 - 1 : 0), _key4 = 1; _key4 < _len4; _key4++) {
          args[_key4 - 1] = arguments[_key4];
        }

        return this.prototype.init.apply(obj, args);
      }
      throw new Error('Called Mongo.Collection constructor with non-instance');
    }
  }, {
    key: 'apply',
    value: function apply(obj, args) {
      if (obj instanceof this) {
        return this.prototype.init.apply(obj, args);
      }
      throw new Error('Applied Mongo.Collection constructor with non-instance');
    }
  }, {
    key: '_rewriteSelector',
    value: function _rewriteSelector(selector) {
      var _ref10 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
          fallbackId = _ref10.fallbackId;

      // shorthand -- scalars match _id
      if (_selectorIsId(selector)) {
        selector = { _id: selector };
      }

      if (Array.isArray(selector)) {
        // This is consistent with the Mongo console itself; if we don't do this
        // check passing an empty array ends up selecting all items
        throw new Error("Mongo selector can't be an array.");
      }

      if (!selector || '_id' in selector && !selector._id) {
        // can't match anything
        return { _id: fallbackId || Random.id() };
      }

      return selector;
    }
  }]);
  return Collection;
}();

// Make static methods of Mongo.Collection enumerable, so that code
// attempting to subclass Mongo.Collection without using native ECMAScript
// class syntax can more easily inherit static properties. For example:
// https://github.com/matb33/meteor-collection-hooks/blob/d791a697a5a4c92dbb17b6e62ad265400e0ed162/collection-hooks.js#L280-L284

var staticDescriptors = (0, _getOwnPropertyDescriptors2.default)(Mongo.Collection);

(0, _keys2.default)(staticDescriptors).forEach(function (name) {
  if (name === 'call' || name === 'apply') {
    // Don't bother making Mongo.Collection.{call,apply} enumerable, since
    // they need not be inherited by non-native function-based classes.
    return;
  }
  var desc = staticDescriptors[name];
  if (desc.configurable && !desc.enumerable) {
    desc.enumerable = true;
    (0, _defineProperty2.default)(Mongo.Collection, name, desc);
  }
});

// Convert the callback to not return a result if there is an error
function wrapCallback(callback, convertResult) {
  return callback && function (error, result) {
    if (error) {
      callback(error);
    } else if (typeof convertResult === 'function') {
      callback(null, convertResult(result));
    } else {
      callback(null, result);
    }
  };
}

/**
 * @summary Create a Mongo-style `ObjectID`.  If you don't specify a `hexString`, the `ObjectID` will generated randomly (not using MongoDB's ID construction rules).
 * @locus Anywhere
 * @class
 * @param {String} [hexString] Optional.  The 24-character hexadecimal contents of the ObjectID to create
 */
Mongo.ObjectID = MongoID.ObjectID;

function popCallbackFromArgs(args) {
  // Pull off any callback (or perhaps a 'callback' variable that was passed
  // in undefined, like how 'upsert' does it).
  if (args.length && (args[args.length - 1] === undefined || args[args.length - 1] instanceof Function)) {
    return args.pop();
  }
}