'use strict';

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// meteor/packages/mongo/mongo_driver.js
// ed98a07125cd072552482ca2244239f034857814

/* eslint-disable camelcase */

var isArray = require('lodash/isArray');
var map = require('lodash/map');
var bind = require('lodash/bind');
var each = require('lodash/each');
var size = require('lodash/size');
var has = require('lodash/has');
var extend = require('lodash/extend');
var pick = require('lodash/pick');
var identity = require('lodash/identity');
var clone = require('lodash/clone');

var _require = require('../ejson'),
    EJSON = _require.EJSON;

var _require2 = require('../callback-hook'),
    Hook = _require2.Hook;

var _require3 = require('../common'),
    bindAndCatch = _require3.bindAndCatch,
    wrapAsync = _require3.wrapAsync;

var _require4 = require('./namespace'),
    Mongo = _require4.Mongo,
    MongoInternals = _require4.MongoInternals;

var _require5 = require('./utils'),
    _isPlainObject = _require5._isPlainObject,
    _isModificationMod = _require5._isModificationMod,
    wrapTransform = _require5.wrapTransform,
    _IdMap = _require5._IdMap;

/**
 * Provide a synchronous Collection API using fibers, backed by
 * MongoDB.  This is only for use on the server, and mostly identical
 * to the client API.
 *
 * NOTE: the public API methods must be run within a fiber. If you call
 * these outside of a fiber they will explode!
 */

var MongoDB = require('mongodb');

MongoInternals.NpmModules = {
  mongodb: {
    version: require('mongodb/package.json').version,
    module: MongoDB
  }

  // Older version of what is now available via
  // MongoInternals.NpmModules.mongodb.module.  It was never documented, but
  // people do use it.
  // XXX COMPAT WITH 1.0.3.2
};MongoInternals.NpmModule = MongoDB;

// This is used to add or remove EJSON from the beginning of everything nested
// inside an EJSON custom type. It should only be called on pure JSON!
var replaceNames = function replaceNames(filter, thing) {
  if ((typeof thing === 'undefined' ? 'undefined' : (0, _typeof3.default)(thing)) === 'object') {
    if (isArray(thing)) {
      return map(thing, bind(replaceNames, null, filter));
    }
    var ret = {};
    each(thing, function (value, key) {
      ret[filter(key)] = replaceNames(filter, value);
    });
    return ret;
  }
  return thing;
};

// Ensure that EJSON.clone keeps a Timestamp as a Timestamp (instead of just
// doing a structural clone).
// XXX how ok is this? what if there are multiple copies of MongoDB loaded?
MongoDB.Timestamp.prototype.clone = function () {
  // Timestamps should be immutable.
  return this;
};

var makeMongoLegal = function makeMongoLegal(name) {
  return 'EJSON' + name;
};
var unmakeMongoLegal = function unmakeMongoLegal(name) {
  return name.substr(5);
};

var replaceMongoAtomWithMeteor = function replaceMongoAtomWithMeteor(document) {
  if (document instanceof MongoDB.Binary) {
    var buffer = document.value(true);
    return new Uint8Array(buffer);
  }
  if (document instanceof MongoDB.ObjectID) {
    return new Mongo.ObjectID(document.toHexString());
  }
  if (document['EJSON$type'] && document['EJSON$value'] && size(document) === 2) {
    return EJSON.fromJSONValue(replaceNames(unmakeMongoLegal, document));
  }
  if (document instanceof MongoDB.Timestamp) {
    // For now, the Meteor representation of a Mongo timestamp type (not a date!
    // this is a weird internal thing used in the oplog!) is the same as the
    // Mongo representation. We need to do this explicitly or else we would do a
    // structural clone and lose the prototype.
    return document;
  }
  return undefined;
};

var replaceMeteorAtomWithMongo = function replaceMeteorAtomWithMongo(document) {
  if (EJSON.isBinary(document)) {
    // This does more copies than we'd like, but is necessary because
    // MongoDB.BSON only looks like it takes a Uint8Array (and doesn't actually
    // serialize it correctly).
    return new MongoDB.Binary(Buffer.from(document));
  }
  if (document instanceof Mongo.ObjectID) {
    return new MongoDB.ObjectID(document.toHexString());
  }
  if (document instanceof MongoDB.Timestamp) {
    // For now, the Meteor representation of a Mongo timestamp type (not a date!
    // this is a weird internal thing used in the oplog!) is the same as the
    // Mongo representation. We need to do this explicitly or else we would do a
    // structural clone and lose the prototype.
    return document;
  }
  if (EJSON._isCustomType(document)) {
    return replaceNames(makeMongoLegal, EJSON.toJSONValue(document));
  }
  // It is not ordinarily possible to stick dollar-sign keys into mongo
  // so we don't bother checking for things that need escaping at this time.
  return undefined;
};

var replaceTypes = function replaceTypes(document, atomTransformer) {
  if ((typeof document === 'undefined' ? 'undefined' : (0, _typeof3.default)(document)) !== 'object' || document === null) {
    return document;
  }

  var replacedTopLevelAtom = atomTransformer(document);
  if (replacedTopLevelAtom !== undefined) {
    return replacedTopLevelAtom;
  }

  var ret = document;
  each(document, function (val, key) {
    var valReplaced = replaceTypes(val, atomTransformer);
    if (val !== valReplaced) {
      // Lazy clone. Shallow copy.
      if (ret === document) {
        ret = clone(document);
      }
      ret[key] = valReplaced;
    }
  });
  return ret;
};

var MongoConnection = exports.MongoConnection = function (url, options) {
  var self = this;
  options = options || {};
  self._onFailoverHook = new Hook();

  var mongoOptions = (0, _assign2.default)({
    // Reconnect on error.
    autoReconnect: true,
    // Try to reconnect forever, instead of stopping after 30 tries (the
    // default), with each attempt separated by 1000ms.
    reconnectTries: Infinity,
    ignoreUndefined: true
  }, Mongo._connectionOptions);

  // Disable the native parser by default, unless specifically enabled
  // in the mongo URL.
  // - The native driver can cause errors which normally would be
  //   thrown, caught, and handled into segfaults that take down the
  //   whole app.
  // - Binary modules don't yet work when you bundle and move the bundle
  //   to a different platform (aka deploy)
  // We should revisit this after binary npm module support lands.
  if (!/[?&]native_?[pP]arser=/.test(url)) {
    mongoOptions.native_parser = false;
  }

  // Internally the oplog connections specify their own poolSize
  // which we don't want to overwrite with any user defined value
  if (has(options, 'poolSize')) {
    // If we just set this for "server", replSet will override it. If we just
    // set it for replSet, it will be ignored if we're not using a replSet.
    mongoOptions.poolSize = options.poolSize;
  }

  self.db = null;
  // We keep track of the ReplSet's primary, so that we can trigger hooks when
  // it changes.  The Node driver's joined callback seems to fire way too
  // often, which is why we need to track it ourselves.
  self._primary = null;

  self.db = new _promise2.default(function (resolve, reject) {
    MongoDB.connect(url, mongoOptions, bindAndCatch(function (err, db) {
      if (err) {
        throw err;
      }

      // First, figure out what the current primary is, if any.
      if (db.serverConfig.isMasterDoc) {
        self._primary = db.serverConfig.isMasterDoc.primary;
      }

      db.serverConfig.on('joined', bindAndCatch(function (kind, doc) {
        if (kind === 'primary') {
          if (doc.primary !== self._primary) {
            self._primary = doc.primary;
            self._onFailoverHook.each(function (callback) {
              callback();
              return true;
            });
          }
        } else if (doc.me === self._primary) {
          // The thing we thought was primary is now something other than
          // primary.  Forget that we thought it was primary.  (This means
          // that if a server stops being primary and then starts being
          // primary again without another server becoming primary in the
          // middle, we'll correctly count it as a failover.)
          self._primary = null;
        }
      }));

      // Allow the constructor to return.
      resolve(db);
    }, reject // onException
    ));
  });
};

MongoConnection.prototype.close = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee() {
  var self, db, close;
  return _regenerator2.default.wrap(function _callee$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          self = this;
          _context.next = 3;
          return self.db;

        case 3:
          db = _context.sent;

          if (db) {
            _context.next = 6;
            break;
          }

          throw Error('close called before Connection created?');

        case 6:

          // Use Future.wrap so that errors get thrown. This happens to
          // work even outside a fiber since the 'close' method is not
          // actually asynchronous.
          close = wrapAsync(db.close, db);
          _context.next = 9;
          return close(true);

        case 9:
        case 'end':
          return _context.stop();
      }
    }
  }, _callee, this);
}));

// Returns the Mongo Collection object; may yield.
MongoConnection.prototype.rawCollection = function () {
  var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(collectionName) {
    var self, db;
    return _regenerator2.default.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            self = this;
            _context2.next = 3;
            return self.db;

          case 3:
            db = _context2.sent;

            if (db) {
              _context2.next = 6;
              break;
            }

            throw Error('rawCollection called before Connection created?');

          case 6:
            return _context2.abrupt('return', new _promise2.default(function (resolve, reject) {
              db.collection(collectionName, function (err, collection) {
                if (err) reject(err);
                resolve(collection);
              });
            }));

          case 7:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, this);
  }));

  return function (_x) {
    return _ref2.apply(this, arguments);
  };
}();

MongoConnection.prototype._createCappedCollection = function () {
  var _ref3 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3(collectionName, byteSize, maxDocuments) {
    var self, db;
    return _regenerator2.default.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            self = this;
            _context3.next = 3;
            return self.db;

          case 3:
            db = _context3.sent;

            if (db) {
              _context3.next = 6;
              break;
            }

            throw Error('_createCappedCollection called before Connection created?');

          case 6:
            return _context3.abrupt('return', new _promise2.default(function (resolve, reject) {
              db.createCollection(collectionName, { capped: true, size: byteSize, max: maxDocuments }, function (err, collection) {
                if (err) reject(err);
                resolve(collection);
              });
            }));

          case 7:
          case 'end':
            return _context3.stop();
        }
      }
    }, _callee3, this);
  }));

  return function (_x2, _x3, _x4) {
    return _ref3.apply(this, arguments);
  };
}();

// Internal interface: adds a callback which is called when the Mongo primary
// changes. Returns a stop handle.
MongoConnection.prototype._onFailover = function (callback) {
  return this._onFailoverHook.register(callback);
};

// Public API

var writeCallback = function writeCallback(callback) {
  return function (err, result) {
    if (callback) {
      callback(err, result);
    } else if (err) {
      throw err;
    }
  };
};

var bindAndCatchForWrite = function bindAndCatchForWrite(callback) {
  return bindAndCatch(callback, 'Mongo write');
};

MongoConnection.prototype._insert = function () {
  var _ref4 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee4(collection_name, document, callback) {
    var self, sendError, collection;
    return _regenerator2.default.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            self = this;

            sendError = function sendError(e) {
              if (callback) {
                return callback(e);
              }
              throw e;
            };

            if (_isPlainObject(document) && !EJSON._isCustomType(document)) {
              _context4.next = 5;
              break;
            }

            sendError(new Error('Only plain objects may be inserted into MongoDB'));
            return _context4.abrupt('return');

          case 5:

            callback = bindAndCatchForWrite(writeCallback(callback));
            _context4.prev = 6;
            _context4.next = 9;
            return self.rawCollection(collection_name);

          case 9:
            collection = _context4.sent;
            _context4.next = 12;
            return collection.insert(replaceTypes(document, replaceMeteorAtomWithMongo), { safe: true }, callback);

          case 12:
            _context4.next = 17;
            break;

          case 14:
            _context4.prev = 14;
            _context4.t0 = _context4['catch'](6);
            throw _context4.t0;

          case 17:
          case 'end':
            return _context4.stop();
        }
      }
    }, _callee4, this, [[6, 14]]);
  }));

  return function (_x5, _x6, _x7) {
    return _ref4.apply(this, arguments);
  };
}();

MongoConnection.prototype._remove = function () {
  var _ref5 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee5(collection_name, selector, callback) {
    var self, collection, wrappedCallback;
    return _regenerator2.default.wrap(function _callee5$(_context5) {
      while (1) {
        switch (_context5.prev = _context5.next) {
          case 0:
            self = this;


            callback = bindAndCatchForWrite(writeCallback(callback));

            _context5.prev = 2;
            _context5.next = 5;
            return self.rawCollection(collection_name);

          case 5:
            collection = _context5.sent;

            wrappedCallback = function wrappedCallback(err, driverResult) {
              callback(err, transformResult(driverResult).numberAffected);
            };

            _context5.next = 9;
            return collection.remove(replaceTypes(selector, replaceMeteorAtomWithMongo), { safe: true }, wrappedCallback);

          case 9:
            _context5.next = 14;
            break;

          case 11:
            _context5.prev = 11;
            _context5.t0 = _context5['catch'](2);
            throw _context5.t0;

          case 14:
          case 'end':
            return _context5.stop();
        }
      }
    }, _callee5, this, [[2, 11]]);
  }));

  return function (_x8, _x9, _x10) {
    return _ref5.apply(this, arguments);
  };
}();

MongoConnection.prototype._dropCollection = function () {
  var _ref6 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee6(collectionName, cb) {
    var self, collection;
    return _regenerator2.default.wrap(function _callee6$(_context6) {
      while (1) {
        switch (_context6.prev = _context6.next) {
          case 0:
            self = this;


            cb = bindAndCatchForWrite(writeCallback(cb));

            _context6.prev = 2;
            _context6.next = 5;
            return self.rawCollection(collectionName);

          case 5:
            collection = _context6.sent;
            _context6.next = 8;
            return collection.drop(cb);

          case 8:
            _context6.next = 13;
            break;

          case 10:
            _context6.prev = 10;
            _context6.t0 = _context6['catch'](2);
            throw _context6.t0;

          case 13:
          case 'end':
            return _context6.stop();
        }
      }
    }, _callee6, this, [[2, 10]]);
  }));

  return function (_x11, _x12) {
    return _ref6.apply(this, arguments);
  };
}();

// For testing only.  Slightly better than `c.rawDatabase().dropDatabase()`
// because it lets the test's fence wait for it to be complete.
MongoConnection.prototype._dropDatabase = function () {
  var _ref7 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee7(cb) {
    var self, db;
    return _regenerator2.default.wrap(function _callee7$(_context7) {
      while (1) {
        switch (_context7.prev = _context7.next) {
          case 0:
            self = this;
            _context7.next = 3;
            return self.db;

          case 3:
            db = _context7.sent;


            cb = bindAndCatchForWrite(writeCallback(cb));

            _context7.prev = 5;
            _context7.next = 8;
            return db.dropDatabase(cb);

          case 8:
            _context7.next = 13;
            break;

          case 10:
            _context7.prev = 10;
            _context7.t0 = _context7['catch'](5);
            throw _context7.t0;

          case 13:
          case 'end':
            return _context7.stop();
        }
      }
    }, _callee7, this, [[5, 10]]);
  }));

  return function (_x13) {
    return _ref7.apply(this, arguments);
  };
}();

MongoConnection.prototype._update = function () {
  var _ref8 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee8(collection_name, selector, mod, options, callback) {
    var self, collection, mongoOpts, mongoSelector, mongoMod, isModify, err, knownId;
    return _regenerator2.default.wrap(function _callee8$(_context8) {
      while (1) {
        switch (_context8.prev = _context8.next) {
          case 0:
            self = this;


            if (!callback && options instanceof Function) {
              callback = options;
              options = null;
            }

            // explicit safety check. null and undefined can crash the mongo
            // driver. Although the node driver and minimongo do 'support'
            // non-object modifier in that they don't crash, they are not
            // meaningful operations and do not do anything. Defensively throw an
            // error here.

            if (!(!mod || (typeof mod === 'undefined' ? 'undefined' : (0, _typeof3.default)(mod)) !== 'object')) {
              _context8.next = 4;
              break;
            }

            throw new Error('Invalid modifier. Modifier must be an object.');

          case 4:
            if (_isPlainObject(mod) && !EJSON._isCustomType(mod)) {
              _context8.next = 6;
              break;
            }

            throw new Error('Only plain objects may be used as replacement' + ' documents in MongoDB');

          case 6:

            if (!options) options = {};

            callback = writeCallback(callback);
            _context8.prev = 8;
            _context8.next = 11;
            return self.rawCollection(collection_name);

          case 11:
            collection = _context8.sent;
            mongoOpts = { safe: true
              // explictly enumerate options that minimongo supports
            };
            if (options.upsert) mongoOpts.upsert = true;
            if (options.multi) mongoOpts.multi = true;
            // Lets you get a more more full result from MongoDB. Use with caution:
            // might not work with C.upsert (as opposed to C.update({upsert:true}) or
            // with simulated upsert.
            if (options.fullResult) mongoOpts.fullResult = true;

            mongoSelector = replaceTypes(selector, replaceMeteorAtomWithMongo);
            mongoMod = replaceTypes(mod, replaceMeteorAtomWithMongo);
            isModify = _isModificationMod(mongoMod);

            if (!(options._forbidReplace && !isModify)) {
              _context8.next = 26;
              break;
            }

            err = new Error('Invalid modifier. Replacements are forbidden.');

            if (!callback) {
              _context8.next = 25;
              break;
            }

            return _context8.abrupt('return', callback(err));

          case 25:
            throw err;

          case 26:
            knownId = void 0;

            if (!(options.upsert && !isModify && !knownId && options.insertedId && !(options.insertedId instanceof Mongo.ObjectID && options.generatedId))) {
              _context8.next = 32;
              break;
            }

            _context8.next = 30;
            return simulateUpsertWithInsertedId(collection, mongoSelector, mongoMod, options,
            // This callback does not need to be bindEnvironment'ed because
            // simulateUpsertWithInsertedId() wraps it and then passes it through
            // bindAndCatchForWrite.
            function (error, result) {
              // If we got here via a upsert() call, then options._returnObject will
              // be set and we should return the whole object. Otherwise, we should
              // just return the number of affected docs to match the mongo API.
              if (result && !options._returnObject) {
                callback(error, result.numberAffected);
              } else {
                callback(error, result);
              }
            });

          case 30:
            _context8.next = 35;
            break;

          case 32:
            if (options.upsert && !knownId && options.insertedId && isModify) {
              if (!mongoMod.hasOwnProperty('$setOnInsert')) {
                mongoMod.$setOnInsert = {};
              }
              knownId = options.insertedId;
              (0, _assign2.default)(mongoMod.$setOnInsert, replaceTypes({ _id: options.insertedId }, replaceMeteorAtomWithMongo));
            }

            _context8.next = 35;
            return collection.update(mongoSelector, mongoMod, mongoOpts, bindAndCatchForWrite(function (err, result) {
              if (!err) {
                var meteorResult = transformResult(result);
                if (meteorResult && options._returnObject) {
                  // If this was an upsert() call, and we ended up
                  // inserting a new doc and we know its id, then
                  // return that id as well.
                  if (options.upsert && meteorResult.insertedId) {
                    if (knownId) {
                      meteorResult.insertedId = knownId;
                    } else if (meteorResult.insertedId instanceof MongoDB.ObjectID) {
                      meteorResult.insertedId = new Mongo.ObjectID(meteorResult.insertedId.toHexString());
                    }
                  }

                  callback(err, meteorResult);
                } else {
                  callback(err, meteorResult.numberAffected);
                }
              } else {
                callback(err);
              }
            }));

          case 35:
            _context8.next = 40;
            break;

          case 37:
            _context8.prev = 37;
            _context8.t0 = _context8['catch'](8);
            throw _context8.t0;

          case 40:
          case 'end':
            return _context8.stop();
        }
      }
    }, _callee8, this, [[8, 37]]);
  }));

  return function (_x14, _x15, _x16, _x17, _x18) {
    return _ref8.apply(this, arguments);
  };
}();

var transformResult = function transformResult(driverResult) {
  var meteorResult = { numberAffected: 0 };
  if (driverResult) {
    var mongoResult = driverResult.result;

    // On updates with upsert:true, the inserted values come as a list of
    // upserted values -- even with options.multi, when the upsert does insert,
    // it only inserts one element.
    if (mongoResult.upserted) {
      meteorResult.numberAffected += mongoResult.upserted.length;

      if (mongoResult.upserted.length === 1) {
        meteorResult.insertedId = mongoResult.upserted[0]._id;
      }
    } else {
      meteorResult.numberAffected = mongoResult.n;
    }
  }

  return meteorResult;
};

var NUM_OPTIMISTIC_TRIES = 3;

// exposed for testing
MongoConnection._isCannotChangeIdError = function (err) {
  // Mongo 3.2.* returns error as next Object:
  // {name: String, code: Number, errmsg: String}
  // Older Mongo returns:
  // {name: String, code: Number, err: String}
  var error = err.errmsg || err.err;

  // We don't use the error code here
  // because the error code we observed it producing (16837) appears to be
  // a far more generic error code based on examining the source.
  if (error.indexOf('The _id field cannot be changed') === 0 || error.indexOf("the (immutable) field '_id' was found to have been altered to _id") !== -1) {
    return true;
  }

  return false;
};

var simulateUpsertWithInsertedId = function () {
  var _ref9 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee13(collection, selector, mod, options, callback) {
    var insertedId, mongoOptsForUpdate, mongoOptsForInsert, replacementWithId, tries, doUpdate, doConditionalInsert;
    return _regenerator2.default.wrap(function _callee13$(_context13) {
      while (1) {
        switch (_context13.prev = _context13.next) {
          case 0:
            // STRATEGY: First try doing an upsert with a generated ID.
            // If this throws an error about changing the ID on an existing document
            // then without affecting the database, we know we should probably try
            // an update without the generated ID. If it affected 0 documents,
            // then without affecting the database, we the document that first
            // gave the error is probably removed and we need to try an insert again
            // We go back to step one and repeat.
            // Like all "optimistic write" schemes, we rely on the fact that it's
            // unlikely our writes will continue to be interfered with under normal
            // circumstances (though sufficiently heavy contention with writers
            // disagreeing on the existence of an object will cause writes to fail
            // in theory).

            insertedId = options.insertedId; // must exist

            mongoOptsForUpdate = {
              safe: true,
              multi: options.multi
            };
            mongoOptsForInsert = {
              safe: true,
              upsert: true
            };
            replacementWithId = (0, _assign2.default)(replaceTypes({ _id: insertedId }, replaceMeteorAtomWithMongo), mod);
            tries = NUM_OPTIMISTIC_TRIES;

            doUpdate = function () {
              var _ref10 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee10() {
                return _regenerator2.default.wrap(function _callee10$(_context10) {
                  while (1) {
                    switch (_context10.prev = _context10.next) {
                      case 0:
                        tries--;

                        if (tries) {
                          _context10.next = 5;
                          break;
                        }

                        callback(new Error('Upsert failed after ' + NUM_OPTIMISTIC_TRIES + ' tries.'));
                        _context10.next = 7;
                        break;

                      case 5:
                        _context10.next = 7;
                        return collection.update(selector, mod, mongoOptsForUpdate, bindAndCatchForWrite(function () {
                          var _ref11 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee9(err, result) {
                            return _regenerator2.default.wrap(function _callee9$(_context9) {
                              while (1) {
                                switch (_context9.prev = _context9.next) {
                                  case 0:
                                    if (!err) {
                                      _context9.next = 4;
                                      break;
                                    }

                                    callback(err);
                                    _context9.next = 10;
                                    break;

                                  case 4:
                                    if (!(result && result.result.n !== 0)) {
                                      _context9.next = 8;
                                      break;
                                    }

                                    callback(null, {
                                      numberAffected: result.result.n
                                    });
                                    _context9.next = 10;
                                    break;

                                  case 8:
                                    _context9.next = 10;
                                    return doConditionalInsert();

                                  case 10:
                                  case 'end':
                                    return _context9.stop();
                                }
                              }
                            }, _callee9, this);
                          }));

                          return function (_x24, _x25) {
                            return _ref11.apply(this, arguments);
                          };
                        }()));

                      case 7:
                      case 'end':
                        return _context10.stop();
                    }
                  }
                }, _callee10, this);
              }));

              return function doUpdate() {
                return _ref10.apply(this, arguments);
              };
            }();

            doConditionalInsert = function () {
              var _ref12 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee12() {
                return _regenerator2.default.wrap(function _callee12$(_context12) {
                  while (1) {
                    switch (_context12.prev = _context12.next) {
                      case 0:
                        _context12.next = 2;
                        return collection.update(selector, replacementWithId, mongoOptsForInsert, bindAndCatchForWrite(function () {
                          var _ref13 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee11(err, result) {
                            return _regenerator2.default.wrap(function _callee11$(_context11) {
                              while (1) {
                                switch (_context11.prev = _context11.next) {
                                  case 0:
                                    if (!err) {
                                      _context11.next = 9;
                                      break;
                                    }

                                    if (!MongoConnection._isCannotChangeIdError(err)) {
                                      _context11.next = 6;
                                      break;
                                    }

                                    _context11.next = 4;
                                    return doUpdate();

                                  case 4:
                                    _context11.next = 7;
                                    break;

                                  case 6:
                                    callback(err);

                                  case 7:
                                    _context11.next = 10;
                                    break;

                                  case 9:
                                    callback(null, {
                                      numberAffected: result.result.upserted.length,
                                      insertedId: insertedId
                                    });

                                  case 10:
                                  case 'end':
                                    return _context11.stop();
                                }
                              }
                            }, _callee11, this);
                          }));

                          return function (_x26, _x27) {
                            return _ref13.apply(this, arguments);
                          };
                        }()));

                      case 2:
                      case 'end':
                        return _context12.stop();
                    }
                  }
                }, _callee12, this);
              }));

              return function doConditionalInsert() {
                return _ref12.apply(this, arguments);
              };
            }();

            _context13.next = 9;
            return doUpdate();

          case 9:
          case 'end':
            return _context13.stop();
        }
      }
    }, _callee13, this);
  }));

  return function simulateUpsertWithInsertedId(_x19, _x20, _x21, _x22, _x23) {
    return _ref9.apply(this, arguments);
  };
}();

each(['insert', 'update', 'remove', 'dropCollection', 'dropDatabase'], function (method) {
  MongoConnection.prototype[method] = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee14() {
    var self,
        _args14 = arguments;
    return _regenerator2.default.wrap(function _callee14$(_context14) {
      while (1) {
        switch (_context14.prev = _context14.next) {
          case 0:
            self = this;
            return _context14.abrupt('return', wrapAsync(self['_' + method]).apply(self, _args14));

          case 2:
          case 'end':
            return _context14.stop();
        }
      }
    }, _callee14, this);
  }));
});

// XXX MongoConnection.upsert() does not return the id of the inserted document
// unless you set it explicitly in the selector or modifier (as a replacement
// doc).
MongoConnection.prototype.upsert = function () {
  var _ref15 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee15(collectionName, selector, mod, options, callback) {
    var self;
    return _regenerator2.default.wrap(function _callee15$(_context15) {
      while (1) {
        switch (_context15.prev = _context15.next) {
          case 0:
            self = this;

            if (typeof options === 'function' && !callback) {
              callback = options;
              options = {};
            }

            return _context15.abrupt('return', self.update(collectionName, selector, mod, extend({}, options, {
              upsert: true,
              _returnObject: true
            }), callback));

          case 3:
          case 'end':
            return _context15.stop();
        }
      }
    }, _callee15, this);
  }));

  return function (_x28, _x29, _x30, _x31, _x32) {
    return _ref15.apply(this, arguments);
  };
}();

MongoConnection.prototype.find = function (collectionName, selector, options) {
  var self = this;

  if (arguments.length === 1) {
    selector = {};
  }

  return new Cursor(self, new CursorDescription(collectionName, selector, options));
};

MongoConnection.prototype.findOne = function () {
  var _ref16 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee16(collection_name, selector, options) {
    var self,
        _args16 = arguments;
    return _regenerator2.default.wrap(function _callee16$(_context16) {
      while (1) {
        switch (_context16.prev = _context16.next) {
          case 0:
            self = this;

            if (_args16.length === 1) {
              selector = {};
            }

            options = options || {};
            options.limit = 1;
            _context16.next = 6;
            return self.find(collection_name, selector, options).fetch();

          case 6:
            return _context16.abrupt('return', _context16.sent[0]);

          case 7:
          case 'end':
            return _context16.stop();
        }
      }
    }, _callee16, this);
  }));

  return function (_x33, _x34, _x35) {
    return _ref16.apply(this, arguments);
  };
}();

// We'll actually design an index API later. For now, we just pass through to
// Mongo's, but make it synchronous.
MongoConnection.prototype._ensureIndex = function () {
  var _ref17 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee17(collectionName, index, options) {
    var self, collection;
    return _regenerator2.default.wrap(function _callee17$(_context17) {
      while (1) {
        switch (_context17.prev = _context17.next) {
          case 0:
            self = this;

            // We expect this function to be called at startup, not from within a method,
            // so we don't interact with the write fence.

            _context17.next = 3;
            return self.rawCollection(collectionName);

          case 3:
            collection = _context17.sent;
            return _context17.abrupt('return', new _promise2.default(function (resolve, reject) {
              collection.ensureIndex(index, options, function (err, results) {
                if (err) reject(err);
                resolve(results);
              });
            }));

          case 5:
          case 'end':
            return _context17.stop();
        }
      }
    }, _callee17, this);
  }));

  return function (_x36, _x37, _x38) {
    return _ref17.apply(this, arguments);
  };
}();
MongoConnection.prototype._dropIndex = function () {
  var _ref18 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee18(collectionName, index) {
    var self, collection;
    return _regenerator2.default.wrap(function _callee18$(_context18) {
      while (1) {
        switch (_context18.prev = _context18.next) {
          case 0:
            self = this;

            // This function is only used by test code, not within a method, so we don't
            // interact with the write fence.

            _context18.next = 3;
            return self.rawCollection(collectionName);

          case 3:
            collection = _context18.sent;
            return _context18.abrupt('return', new _promise2.default(function (resolve, reject) {
              collection.dropIndex(index, function (err, results) {
                if (err) reject(err);
                resolve(results);
              });
            }));

          case 5:
          case 'end':
            return _context18.stop();
        }
      }
    }, _callee18, this);
  }));

  return function (_x39, _x40) {
    return _ref18.apply(this, arguments);
  };
}();

// CURSORS

// There are several classes which relate to cursors:
//
// CursorDescription represents the arguments used to construct a cursor:
// collectionName, selector, and (find) options.  Because it is used as a key
// for cursor de-dup, everything in it should either be JSON-stringifiable or
// not affect observeChanges output (eg, options.transform functions are not
// stringifiable but do not affect observeChanges).
//
// SynchronousCursor is a wrapper around a MongoDB cursor
// which includes fully-synchronous versions of forEach, etc.
//
// Cursor is the cursor object returned from find(), which implements the
// documented Mongo.Collection cursor API.  It wraps a CursorDescription and a
// SynchronousCursor (lazily: it doesn't contact Mongo until you call a method
// like fetch or forEach on it).
//
// ObserveHandle is the "observe handle" returned from observeChanges. It has a
// reference to an ObserveMultiplexer.
//
// ObserveMultiplexer allows multiple identical ObserveHandles to be driven by a
// single observe driver.
//
// There are two "observe drivers" which drive ObserveMultiplexers:
//   - PollingObserveDriver caches the results of a query and reruns it when
//     necessary.
//   - OplogObserveDriver follows the Mongo operation log to directly observe
//     database changes.
// Both implementations follow the same simple interface: when you create them,
// they start sending observeChanges callbacks (and a ready() invocation) to
// their ObserveMultiplexer, and you stop them by calling their stop() method.

var CursorDescription = function CursorDescription(collectionName, selector, options) {
  var self = this;
  self.collectionName = collectionName;
  self.selector = Mongo.Collection._rewriteSelector(selector);
  self.options = options || {};
};

var Cursor = function Cursor(mongo, cursorDescription) {
  var self = this;

  self._mongo = mongo;
  self._cursorDescription = cursorDescription;
  self._synchronousCursor = null;
};

each(['forEach', 'map', 'fetch', 'count'], function (method) {
  Cursor.prototype[method] = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee19() {
    var self,
        _args19 = arguments;
    return _regenerator2.default.wrap(function _callee19$(_context19) {
      while (1) {
        switch (_context19.prev = _context19.next) {
          case 0:
            self = this;

            // You can only observe a tailable cursor.

            if (!self._cursorDescription.options.tailable) {
              _context19.next = 3;
              break;
            }

            throw new Error('Cannot call ' + method + ' on a tailable cursor');

          case 3:
            if (self._synchronousCursor) {
              _context19.next = 7;
              break;
            }

            _context19.next = 6;
            return self._mongo._createSynchronousCursor(self._cursorDescription, {
              // Make sure that the "self" argument to forEach/map callbacks is the
              // Cursor, not the SynchronousCursor.
              selfForIteration: self,
              useTransform: true
            });

          case 6:
            self._synchronousCursor = _context19.sent;

          case 7:
            return _context19.abrupt('return', self._synchronousCursor[method].apply(self._synchronousCursor, _args19));

          case 8:
          case 'end':
            return _context19.stop();
        }
      }
    }, _callee19, this);
  }));
});

// Since we don't actually have a "nextObject" interface, there's really no
// reason to have a "rewind" interface.  All it did was make multiple calls
// to fetch/map/forEach return nothing the second time.
// XXX COMPAT WITH 0.8.1
Cursor.prototype.rewind = function () {};

Cursor.prototype.getTransform = function () {
  return this._cursorDescription.options.transform;
};

// Used to guarantee that publish functions return at most one cursor per
// collection. Private, because we might later have cursors that include
// documents from multiple collections somehow.
Cursor.prototype._getCollectionName = function () {
  var self = this;
  return self._cursorDescription.collectionName;
};

MongoConnection.prototype._createSynchronousCursor = function () {
  var _ref20 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee20(cursorDescription, options) {
    var self, collection, cursorOptions, mongoOptions, dbCursor;
    return _regenerator2.default.wrap(function _callee20$(_context20) {
      while (1) {
        switch (_context20.prev = _context20.next) {
          case 0:
            self = this;

            options = pick(options || {}, 'selfForIteration', 'useTransform');

            _context20.next = 4;
            return self.rawCollection(cursorDescription.collectionName);

          case 4:
            collection = _context20.sent;
            cursorOptions = cursorDescription.options;
            mongoOptions = {
              sort: cursorOptions.sort,
              limit: cursorOptions.limit,
              skip: cursorOptions.skip

              // Do we want a tailable cursor (which only works on capped collections)?
            };
            if (cursorOptions.tailable) {
              // We want a tailable cursor...
              mongoOptions.tailable = true;
              // ... and for the server to wait a bit if any getMore has no data (rather
              // than making us put the relevant sleeps in the client)...
              mongoOptions.awaitdata = true;
              // ... and to keep querying the server indefinitely rather than just 5 times
              // if there's no more data.
              mongoOptions.numberOfRetries = -1;
            }

            dbCursor = collection.find(replaceTypes(cursorDescription.selector, replaceMeteorAtomWithMongo), cursorOptions.fields, mongoOptions);


            if (typeof cursorOptions.maxTimeMs !== 'undefined') {
              dbCursor = dbCursor.maxTimeMS(cursorOptions.maxTimeMs);
            }
            if (typeof cursorOptions.hint !== 'undefined') {
              dbCursor = dbCursor.hint(cursorOptions.hint);
            }

            return _context20.abrupt('return', new SynchronousCursor(dbCursor, cursorDescription, options));

          case 12:
          case 'end':
            return _context20.stop();
        }
      }
    }, _callee20, this);
  }));

  return function (_x41, _x42) {
    return _ref20.apply(this, arguments);
  };
}();

var SynchronousCursor = function SynchronousCursor(dbCursor, cursorDescription, options) {
  var self = this;
  options = pick(options || {}, 'selfForIteration', 'useTransform');

  self._dbCursor = dbCursor;
  self._cursorDescription = cursorDescription;
  // The "self" argument passed to forEach/map callbacks. If we're wrapped
  // inside a user-visible Cursor, we want to provide the outer cursor!
  self._selfForIteration = options.selfForIteration || self;
  if (options.useTransform && cursorDescription.options.transform) {
    self._transform = wrapTransform(cursorDescription.options.transform);
  } else {
    self._transform = null;
  }

  // Need to specify that the callback is the first argument to nextObject,
  // since otherwise when we try to call it with no args the driver will
  // interpret "undefined" first arg as an options hash and crash.
  self._synchronousNextObject = wrapAsync(dbCursor.nextObject, dbCursor);
  self._synchronousCount = wrapAsync(dbCursor.count, dbCursor);
  self._visitedIds = new _IdMap();
};

extend(SynchronousCursor.prototype, {
  _nextObject: function () {
    var _ref21 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee21() {
      var self, doc;
      return _regenerator2.default.wrap(function _callee21$(_context21) {
        while (1) {
          switch (_context21.prev = _context21.next) {
            case 0:
              self = this;

            case 1:
              if (!true) {
                _context21.next = 16;
                break;
              }

              _context21.next = 4;
              return self._synchronousNextObject();

            case 4:
              doc = _context21.sent;

              if (doc) {
                _context21.next = 7;
                break;
              }

              return _context21.abrupt('return', null);

            case 7:
              doc = replaceTypes(doc, replaceMongoAtomWithMeteor);

              if (!(!self._cursorDescription.options.tailable && has(doc, '_id'))) {
                _context21.next = 12;
                break;
              }

              if (!self._visitedIds.has(doc._id)) {
                _context21.next = 11;
                break;
              }

              return _context21.abrupt('continue', 1);

            case 11:
              self._visitedIds.set(doc._id, true);

            case 12:

              if (self._transform) {
                doc = self._transform(doc);
              }

              return _context21.abrupt('return', doc);

            case 16:
            case 'end':
              return _context21.stop();
          }
        }
      }, _callee21, this);
    }));

    function _nextObject() {
      return _ref21.apply(this, arguments);
    }

    return _nextObject;
  }(),

  forEach: function () {
    var _ref22 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee22(callback, thisArg) {
      var self, index, doc;
      return _regenerator2.default.wrap(function _callee22$(_context22) {
        while (1) {
          switch (_context22.prev = _context22.next) {
            case 0:
              self = this;

              // Get back to the beginning.

              self._rewind();

              // We implement the loop ourself instead of using self._dbCursor.each,
              // because "each" will call its callback outside of a fiber which makes it
              // much more complex to make this function synchronous.
              index = 0;

            case 3:
              if (!true) {
                _context22.next = 12;
                break;
              }

              _context22.next = 6;
              return self._nextObject();

            case 6:
              doc = _context22.sent;

              if (doc) {
                _context22.next = 9;
                break;
              }

              return _context22.abrupt('return');

            case 9:
              callback.call(thisArg, doc, index++, self._selfForIteration);
              _context22.next = 3;
              break;

            case 12:
            case 'end':
              return _context22.stop();
          }
        }
      }, _callee22, this);
    }));

    function forEach(_x43, _x44) {
      return _ref22.apply(this, arguments);
    }

    return forEach;
  }(),

  // XXX Allow overlapping callback executions if callback yields.
  map: function () {
    var _ref23 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee23(callback, thisArg) {
      var self, res;
      return _regenerator2.default.wrap(function _callee23$(_context23) {
        while (1) {
          switch (_context23.prev = _context23.next) {
            case 0:
              self = this;
              res = [];
              _context23.next = 4;
              return self.forEach(function (doc, index) {
                res.push(callback.call(thisArg, doc, index, self._selfForIteration));
              });

            case 4:
              return _context23.abrupt('return', res);

            case 5:
            case 'end':
              return _context23.stop();
          }
        }
      }, _callee23, this);
    }));

    function map(_x45, _x46) {
      return _ref23.apply(this, arguments);
    }

    return map;
  }(),

  _rewind: function _rewind() {
    var self = this;

    // known to be synchronous
    self._dbCursor.rewind();

    self._visitedIds = new _IdMap();
  },

  // Mostly usable for tailable cursors.
  close: function close() {
    var self = this;

    self._dbCursor.close();
  },

  fetch: function () {
    var _ref24 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee24() {
      var self;
      return _regenerator2.default.wrap(function _callee24$(_context24) {
        while (1) {
          switch (_context24.prev = _context24.next) {
            case 0:
              self = this;
              return _context24.abrupt('return', self.map(identity));

            case 2:
            case 'end':
              return _context24.stop();
          }
        }
      }, _callee24, this);
    }));

    function fetch() {
      return _ref24.apply(this, arguments);
    }

    return fetch;
  }(),

  count: function () {
    var _ref25 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee25() {
      var applySkipLimit = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
      var self;
      return _regenerator2.default.wrap(function _callee25$(_context25) {
        while (1) {
          switch (_context25.prev = _context25.next) {
            case 0:
              self = this;
              return _context25.abrupt('return', self._synchronousCount(applySkipLimit));

            case 2:
            case 'end':
              return _context25.stop();
          }
        }
      }, _callee25, this);
    }));

    function count() {
      return _ref25.apply(this, arguments);
    }

    return count;
  }(),

  // This method is NOT wrapped in Cursor.
  getRawObjects: function () {
    var _ref26 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee26(ordered) {
      var self, results;
      return _regenerator2.default.wrap(function _callee26$(_context26) {
        while (1) {
          switch (_context26.prev = _context26.next) {
            case 0:
              self = this;

              if (!ordered) {
                _context26.next = 5;
                break;
              }

              return _context26.abrupt('return', self.fetch());

            case 5:
              results = new _IdMap();
              _context26.next = 8;
              return self.forEach(function (doc) {
                results.set(doc._id, doc);
              });

            case 8:
              return _context26.abrupt('return', results);

            case 9:
            case 'end':
              return _context26.stop();
          }
        }
      }, _callee26, this);
    }));

    function getRawObjects(_x48) {
      return _ref26.apply(this, arguments);
    }

    return getRawObjects;
  }()
});

// XXX We probably need to find a better way to expose this. Right now
// it's only used by tests, but in fact you need it in normal
// operation to interact with capped collections.
MongoInternals.MongoTimestamp = MongoDB.Timestamp;

MongoInternals.Connection = MongoConnection;