'use strict';

// meteor/packages/mongo/remote_collection_driver.js
// 63a0076305b0f204d8acbd322bebd4ded9f2596d

/* eslint-disable camelcase */

var extend = require('lodash/extend');
var each = require('lodash/each');
var bind = require('lodash/bind');
var once = require('lodash/once');

var _require = require('./namespace'),
    MongoInternals = _require.MongoInternals;

var _require2 = require('./mongo_driver'),
    MongoConnection = _require2.MongoConnection;

MongoInternals.RemoteCollectionDriver = function (mongo_url, options) {
  var self = this;
  self.mongo = new MongoConnection(mongo_url, options);
};

extend(MongoInternals.RemoteCollectionDriver.prototype, {
  open: function open(name) {
    var self = this;
    var ret = {};
    each(['find', 'findOne', 'insert', 'update', 'upsert', 'remove', '_ensureIndex', '_dropIndex', '_createCappedCollection', 'dropCollection', 'rawCollection'], function (m) {
      ret[m] = bind(self.mongo[m], self.mongo, name);
    });
    return ret;
  }
});

// Create the singleton RemoteCollectionDriver only on demand, so we
// only require Mongo configuration if it's actually used (eg, not if
// you're only trying to receive data from a remote DDP server.)
MongoInternals.defaultRemoteCollectionDriver = once(function () {
  var connectionOptions = {};

  var mongoUrl = process.env.MONGO_URL;

  if (process.env.MONGO_OPLOG_URL) {
    connectionOptions.oplogUrl = process.env.MONGO_OPLOG_URL;
  }

  if (!mongoUrl) {
    throw new Error('MONGO_URL must be set in environment');
  }

  return new MongoInternals.RemoteCollectionDriver(mongoUrl, connectionOptions);
});