'use strict';

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var MongoBins = require('./mongo-bins');

var COMMAND = 'mongod';

module.exports = function () {
  function MongodHelper() {
    var commandArguments = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
    (0, _classCallCheck3.default)(this, MongodHelper);

    this.mongoBin = new MongoBins(COMMAND, commandArguments);
    this.resolveLink = function () {};
    this.rejectLink = function () {};
  }

  (0, _createClass3.default)(MongodHelper, [{
    key: 'run',
    value: function run() {
      var _this = this;

      return new _promise2.default(function (resolve, reject) {
        _this.resolveLink = resolve;
        _this.rejectLink = reject;

        _this.mongoBin.run().then(function () {
          _this.mongoBin.childProcess.stderr.on('data', function (data) {
            return _this.stderrHandler(data);
          });
          _this.mongoBin.childProcess.stdout.on('data', function (data) {
            return _this.stdoutHandler(data);
          });
          _this.mongoBin.childProcess.on('close', function (code) {
            return _this.closeHandler(code);
          });
        });
      });
    }
  }, {
    key: 'closeHandler',
    value: function closeHandler(code) {
      console.log('mongod close: ' + code);
    }
  }, {
    key: 'stderrHandler',
    value: function stderrHandler(message) {
      console.log('mongod stderr: ' + message);
    }
  }, {
    key: 'stdoutHandler',
    value: function stdoutHandler(message) {
      var log = message.toString();

      var mongodStartExpression = this.getMongodStartedExpression();
      var mongodAlreadyRunningExpression = this.getMongodAlreadyRunningExpression();
      var mongodAddressAlreadyInUseExpression = this.getMongodAddressAlreadyInUse();
      var mongodPermissionDeniedExpression = this.getMongodPermissionDeniedExpression();
      var mongodDataDirNotFounddExpression = this.getMongodDataDirNotFounddExpression();
      var mongodShutdownMessageExpression = this.getMongodShutdownMessageExpression();

      if (mongodStartExpression.test(log)) {
        this.resolveLink(true);
      }

      if (mongodAlreadyRunningExpression.test(log)) {
        console.log('already running');
        return this.resolveLink('already running');
      }

      if (mongodAddressAlreadyInUseExpression.test(log)) {
        console.log('address already in use');
        return this.resolveLink('address already in use');
      }

      if (mongodPermissionDeniedExpression.test(log)) {
        return this.rejectLink('permission denied');
      }

      if (mongodDataDirNotFounddExpression.test(log)) {
        return this.rejectLink('Data directory not found');
      }

      if (mongodShutdownMessageExpression.test(log)) {
        return this.rejectLink('Mongod shutting down');
      }
    }
  }, {
    key: 'getMongodStartedExpression',
    value: function getMongodStartedExpression() {
      return (/waiting for connections on port/i
      );
    }
  }, {
    key: 'getMongodAlreadyRunningExpression',
    value: function getMongodAlreadyRunningExpression() {
      return (/mongod instance already running/i
      );
    }
  }, {
    key: 'getMongodAddressAlreadyInUse',
    value: function getMongodAddressAlreadyInUse() {
      return (/Address already in use/i
      );
    }
  }, {
    key: 'getMongodPermissionDeniedExpression',
    value: function getMongodPermissionDeniedExpression() {
      return (/permission denied/i
      );
    }
  }, {
    key: 'getMongodDataDirNotFounddExpression',
    value: function getMongodDataDirNotFounddExpression() {
      return (/Data directory .*? not found/i
      );
    }
  }, {
    key: 'getMongodShutdownMessageExpression',
    value: function getMongodShutdownMessageExpression() {
      return (/shutting down with code/i
      );
    }
  }]);
  return MongodHelper;
}();