'use strict';

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint-disable prefer-promise-reject-errors */

var os = require('os');
var http = require('https');
var fs = require('fs-extra');
var path = require('path');
var getos = require('getos');
var url = require('url');
var decompress = require('decompress');
var request = require('request');
var md5File = require('md5-file');

var DOWNLOAD_URI = 'https://downloads.mongodb.org';
var MONGODB_VERSION = '3.2.15';

module.exports = function () {
  function MongoDBDownload(_ref) {
    var _ref$platform = _ref.platform,
        platform = _ref$platform === undefined ? os.platform() : _ref$platform,
        _ref$arch = _ref.arch,
        arch = _ref$arch === undefined ? os.arch() : _ref$arch,
        _ref$downloadDir = _ref.downloadDir,
        downloadDir = _ref$downloadDir === undefined ? os.tmpdir() : _ref$downloadDir,
        _ref$version = _ref.version,
        version = _ref$version === undefined ? MONGODB_VERSION : _ref$version,
        _ref$http = _ref.http,
        http = _ref$http === undefined ? {} : _ref$http;
    (0, _classCallCheck3.default)(this, MongoDBDownload);

    this.options = {
      platform: platform,
      arch: arch,
      downloadDir: downloadDir,
      version: version,
      http: http
    };

    this.mongoDBPlatform = new MongoDBPlatform(this.getPlatform(), this.getArch());
    this.options.downloadDir = path.resolve(this.options.downloadDir, 'mongodb-download');
    this.downloadProgress = {
      current: 0,
      length: 0,
      total: 0,
      lastStdout: ''
    };
  }

  (0, _createClass3.default)(MongoDBDownload, [{
    key: 'getPlatform',
    value: function getPlatform() {
      return this.options.platform;
    }
  }, {
    key: 'getArch',
    value: function getArch() {
      return this.options.arch;
    }
  }, {
    key: 'getVersion',
    value: function getVersion() {
      return this.options.version;
    }
  }, {
    key: 'getDownloadDir',
    value: function getDownloadDir() {
      return this.options.downloadDir;
    }
  }, {
    key: 'getDownloadLocation',
    value: function getDownloadLocation() {
      var _this = this;

      return new _promise2.default(function (resolve, reject) {
        _this.getArchiveName().then(function (archiveName) {
          var downloadDir = _this.getDownloadDir();
          var fullPath = path.resolve(downloadDir, archiveName);
          resolve(fullPath);
        });
      });
    }
  }, {
    key: 'getExtractLocation',
    value: function getExtractLocation() {
      var _this2 = this;

      return new _promise2.default(function (resolve, reject) {
        _this2.getMD5Hash().then(function (hash) {
          if (!hash) {
            console.error('hash is not returned @ getExtractLocation()');
            return reject();
          }
          var downloadDir = _this2.getDownloadDir();
          var extractLocation = path.resolve(downloadDir, hash);
          resolve(extractLocation);
        }, function (e) {
          console.error('hash is not returned @ getExtractLocation()', e);
          reject();
        });
      });
    }
  }, {
    key: 'getTempDownloadLocation',
    value: function getTempDownloadLocation() {
      var _this3 = this;

      return new _promise2.default(function (resolve, reject) {
        _this3.getArchiveName().then(function (archiveName) {
          var downloadDir = _this3.getDownloadDir();
          var fullPath = path.resolve(downloadDir, archiveName + '.downloading');
          resolve(fullPath);
        });
      });
    }
  }, {
    key: 'downloadAndExtract',
    value: function downloadAndExtract() {
      var _this4 = this;

      return new _promise2.default(function (resolve, reject) {
        _this4.download().then(function (archive) {
          _this4.extract().then(function (extractLocation) {
            resolve(extractLocation);
          });
        });
      });
    }
  }, {
    key: 'extract',
    value: function extract() {
      var _this5 = this;

      return new _promise2.default(function (resolve, reject) {
        _this5.getExtractLocation().then(function (extractLocation) {
          _this5.isExtractPresent().then(function (extractPresent) {
            if (extractPresent === true) {
              resolve(extractLocation);
            } else {
              _this5.getDownloadLocation().then(function (mongoDBArchive) {
                decompress(mongoDBArchive, extractLocation).then(function (files) {
                  resolve(extractLocation);
                }, function (e) {
                  console.log('extract() failed', extractLocation, e);
                });
              });
            }
          });
        });
      });
    }
  }, {
    key: 'download',
    value: function download() {
      var _this6 = this;

      return new _promise2.default(function (resolve, reject) {
        var httpOptionsPromise = _this6.getHttpOptions();
        var downloadLocationPromise = _this6.getDownloadLocation();
        var tempDownloadLocationPromise = _this6.getTempDownloadLocation();
        var createDownloadDirPromise = _this6.createDownloadDir();

        _promise2.default.all([httpOptionsPromise, downloadLocationPromise, tempDownloadLocationPromise, createDownloadDirPromise]).then(function (values) {
          var httpOptions = values[0];
          var downloadLocation = values[1];
          var tempDownloadLocation = values[2];

          _this6.isDownloadPresent().then(function (isDownloadPresent) {
            if (isDownloadPresent === true) {
              resolve(downloadLocation);
            } else {
              _this6.httpDownload(httpOptions, downloadLocation, tempDownloadLocation).then(function (location) {
                resolve(location);
              }, function (e) {
                reject(e);
              });
            }
          });
        });
      });
    }

    // TODO: needs refactoring

  }, {
    key: 'isDownloadPresent',
    value: function isDownloadPresent() {
      var _this7 = this;

      return new _promise2.default(function (resolve, reject) {
        _this7.getDownloadLocation().then(function (downloadLocation) {
          if (_this7.locationExists(downloadLocation) === true) {
            _this7.getMD5Hash().then(function (downloadHash) {
              md5File(downloadLocation, function (err, hash) {
                if (err) {
                  throw err;
                }
                if (hash === downloadHash) {
                  resolve(true);
                } else {
                  resolve(false);
                }
              });
            });
          } else {
            resolve(false);
          }
        });
      });
    }
  }, {
    key: 'isExtractPresent',
    value: function isExtractPresent() {
      var _this8 = this;

      return new _promise2.default(function (resolve, reject) {
        _this8.getMD5Hash().then(function (downloadHash) {
          _this8.getExtractLocation().then(function (extractLocation) {
            var present = _this8.locationExists(extractLocation);
            resolve(present);
          });
        });
      });
    }
  }, {
    key: 'getMD5HashFileLocation',
    value: function getMD5HashFileLocation() {
      var _this9 = this;

      return new _promise2.default(function (resolve, reject) {
        _this9.getDownloadLocation().then(function (downloadLocation) {
          var md5HashLocation = downloadLocation + '.md5';
          resolve(md5HashLocation);
        }, function (e) {
          console.error('error @ getMD5HashFileLocation', e);
          reject(e);
        });
      });
    }
  }, {
    key: 'cacheMD5Hash',
    value: function cacheMD5Hash(signature) {
      var _this10 = this;

      return new _promise2.default(function (resolve, reject) {
        _this10.getMD5HashFileLocation().then(function (hashFile) {
          fs.outputFile(hashFile, signature, function (err) {
            if (err) {
              console.log('@cacheMD5Hash unable to save signature', signature);
              reject();
            } else {
              resolve();
            }
          });
        });
      });
    }
  }, {
    key: 'getMD5Hash',
    value: function getMD5Hash() {
      var _this11 = this;

      return new _promise2.default(function (resolve, reject) {
        _this11.getMD5HashOffline().then(function (offlineSignature) {
          resolve(offlineSignature);
        }, function (e) {
          _this11.getMD5HashOnline().then(function (onlineSignature) {
            resolve(onlineSignature);
          }, function (e) {
            console.error('unable to get signature content', e);
            reject(e);
          });
        });
      });
    }
  }, {
    key: 'getMD5HashOnline',
    value: function getMD5HashOnline() {
      var _this12 = this;

      return new _promise2.default(function (resolve, reject) {
        _this12.getDownloadURIMD5().then(function (md5URL) {
          request(md5URL, function (e, res, signatureContent) {
            if (e) {
              console.error('unable to get signature content', e);
              reject(e);
              return;
            }

            var signatureMatch = signatureContent.match(/(.*?)\s/);
            var signature = signatureMatch[1];
            _this12.cacheMD5Hash(signature).then(function () {
              resolve(signature);
            }, function (e) {
              console.log('@getMD5HashOnline erorr', e);
              reject();
            });
          });
        });
      });
    }
  }, {
    key: 'getMD5HashOffline',
    value: function getMD5HashOffline() {
      var _this13 = this;

      return new _promise2.default(function (resolve, reject) {
        _this13.getMD5HashFileLocation().then(function (hashFile) {
          fs.readFile(hashFile, 'utf8', function (err, signature) {
            if (err) {
              reject();
            } else {
              resolve(signature);
            }
          });
        });
      });
    }
  }, {
    key: 'httpDownload',
    value: function httpDownload(httpOptions, downloadLocation, tempDownloadLocation) {
      var _this14 = this;

      return new _promise2.default(function (resolve, reject) {
        var fileStream = fs.createWriteStream(tempDownloadLocation);

        var request = http.get(httpOptions, function (response) {
          _this14.downloadProgress.current = 0;
          _this14.downloadProgress.length = parseInt(response.headers['content-length'], 10);
          _this14.downloadProgress.total = Math.round(_this14.downloadProgress.length / 1048576 * 10) / 10;

          response.pipe(fileStream);

          fileStream.on('finish', function () {
            process.stdout.clearLine();
            process.stdout.cursorTo(0);
            fileStream.close(function () {
              fs.renameSync(tempDownloadLocation, downloadLocation);
              resolve(downloadLocation);
            });
          });

          response.on('data', function (chunk) {
            _this14.printDownloadProgress(chunk);
          });

          request.on('error', function (e) {
            console.log('request error:', e);
            reject(e);
          });
        });
      });
    }
  }, {
    key: 'getCrReturn',
    value: function getCrReturn() {
      if (this.mongoDBPlatform.getPlatform() === 'win32') {
        return '\x1b[0G';
      } else {
        return '\r';
      }
    }
  }, {
    key: 'locationExists',
    value: function locationExists(location) {
      var exists = void 0;
      try {
        fs.lstatSync(location);
        exists = true;
      } catch (e) {
        if (e.code !== 'ENOENT') throw e;
        exists = false;
      }
      return exists;
    }
  }, {
    key: 'printDownloadProgress',
    value: function printDownloadProgress(chunk) {
      var crReturn = this.getCrReturn();
      this.downloadProgress.current += chunk.length;
      var percentComplete = Math.round(100.0 * this.downloadProgress.current / this.downloadProgress.length * 10) / 10;
      var mbComplete = Math.round(this.downloadProgress.current / 1048576 * 10) / 10;
      var textToPrint = 'Completed: ' + percentComplete + ' % (' + mbComplete + 'mb / ' + this.downloadProgress.total + 'mb)' + crReturn;
      if (this.downloadProgress.lastStdout !== textToPrint) {
        this.downloadProgress.lastStdout = textToPrint;
        process.stdout.clearLine();
        process.stdout.cursorTo(0);
        process.stdout.write(textToPrint);
      }
    }
  }, {
    key: 'getHttpOptions',
    value: function getHttpOptions() {
      var _this15 = this;

      return new _promise2.default(function (resolve, reject) {
        _this15.getDownloadURI().then(function (downloadURI) {
          _this15.options.http.protocol = downloadURI.protocol;
          _this15.options.http.hostname = downloadURI.hostname;
          _this15.options.http.path = downloadURI.path;
          resolve(_this15.options.http);
        });
      });
    }
  }, {
    key: 'getDownloadURI',
    value: function getDownloadURI() {
      var _this16 = this;

      return new _promise2.default(function (resolve, reject) {
        var downloadURL = DOWNLOAD_URI + '/' + _this16.mongoDBPlatform.getPlatform();
        _this16.getArchiveName().then(function (archiveName) {
          downloadURL += '/' + archiveName;
          var downloadURLObject = url.parse(downloadURL);
          resolve(downloadURLObject);
        });
      });
    }
  }, {
    key: 'getDownloadURIMD5',
    value: function getDownloadURIMD5() {
      var _this17 = this;

      return new _promise2.default(function (resolve, reject) {
        _this17.getDownloadURI().then(function (downloadURI) {
          var downloadURIMD5 = downloadURI.href + '.md5';
          resolve(downloadURIMD5);
        });
      });
    }
  }, {
    key: 'createDownloadDir',
    value: function createDownloadDir() {
      var _this18 = this;

      return new _promise2.default(function (resolve, reject) {
        var dirToCreate = _this18.getDownloadDir();
        fs.ensureDir(dirToCreate, function (err) {
          if (err) {
            console.log('createDownloadDir() error: ' + err);
            throw err;
          } else {
            resolve(true);
          }
        });
      });
    }
  }, {
    key: 'getArchiveName',
    value: function getArchiveName() {
      var _this19 = this;

      return new _promise2.default(function (resolve, reject) {
        // var name = "mongodb-" + mongo_platform + "-" + mongo_arch;
        var name = 'mongodb-' + _this19.mongoDBPlatform.getPlatform() + '-' + _this19.mongoDBPlatform.getArch();

        _this19.mongoDBPlatform.getOSVersionString().then(function (osString) {
          osString && (name += '-' + osString);
        }, function (error) {
          // nothing to add to name ... yet
          if (error) {
            console.log(error);
          }
        }).then(function () {
          name += '-' + _this19.getVersion() + '.' + _this19.mongoDBPlatform.getArchiveType();
          resolve(name);
        });
      });
    }
  }]);
  return MongoDBDownload;
}();

var MongoDBPlatform = function () {
  function MongoDBPlatform(platform, arch) {
    (0, _classCallCheck3.default)(this, MongoDBPlatform);

    this.platform = this.translatePlatform(platform);
    this.arch = this.translateArch(arch, this.getPlatform());
  }

  (0, _createClass3.default)(MongoDBPlatform, [{
    key: 'getPlatform',
    value: function getPlatform() {
      return this.platform;
    }
  }, {
    key: 'getArch',
    value: function getArch() {
      return this.arch;
    }
  }, {
    key: 'getArchiveType',
    value: function getArchiveType() {
      if (this.getPlatform() === 'win32') {
        return 'zip';
      } else {
        return 'tgz';
      }
    }
  }, {
    key: 'getCommonReleaseString',
    value: function getCommonReleaseString() {
      var name = 'mongodb-' + this.getPlatform() + '-' + this.getArch();
      return name;
    }
  }, {
    key: 'getOSVersionString',
    value: function getOSVersionString() {
      if (this.getPlatform() === 'linux' && this.getArch() !== 'i686') {
        return this.getLinuxOSVersionString();
      } else {
        return this.getOtherOSVersionString();
      }
    }
  }, {
    key: 'getOtherOSVersionString',
    value: function getOtherOSVersionString() {
      return new _promise2.default(function (resolve, reject) {
        reject('');
      });
    }
  }, {
    key: 'getLinuxOSVersionString',
    value: function getLinuxOSVersionString() {
      var _this20 = this;

      return new _promise2.default(function (resolve, reject) {
        getos(function (e, os) {
          if (/ubuntu/i.test(os.dist)) {
            resolve(_this20.getUbuntuVersionString(os));
          } else if (/elementary OS/i.test(os.dist)) {
            resolve(_this20.getElementaryOSVersionString(os));
          } else if (/suse/i.test(os.dist)) {
            resolve(_this20.getSuseVersionString(os));
          } else if (/rhel/i.test(os.dist) || /centos/i.test(os.dist) || /scientific/i.test(os.dist)) {
            resolve(_this20.getRhelVersionString(os));
          } else if (/fedora/i.test(os.dist)) {
            resolve(_this20.getFedoraVersionString(os));
          } else if (/debian/i.test(os.dist)) {
            resolve(_this20.getDebianVersionString(os));
          } else {
            reject('');
          }
        });
      });
    }
  }, {
    key: 'getDebianVersionString',
    value: function getDebianVersionString(os) {
      var name = 'debian';
      var release = parseFloat(os.release);
      if (release >= 8.1) {
        name += '81';
      } else if (release >= 7.1) {
        name += '71';
      }
      return name;
    }
  }, {
    key: 'getFedoraVersionString',
    value: function getFedoraVersionString(os) {
      var name = 'rhel';
      var fedoraVersion = parseInt(os.release);
      if (fedoraVersion > 18) {
        name += '70';
      } else if (fedoraVersion < 19 && fedoraVersion >= 12) {
        name += '62';
      } else if (fedoraVersion < 12 && fedoraVersion >= 6) {
        name += '55';
      }
      return name;
    }
  }, {
    key: 'getRhelVersionString',
    value: function getRhelVersionString(os) {
      var name = 'rhel';
      if (/^7/.test(os.release)) {
        name += '70';
      } else if (/^6/.test(os.release)) {
        name += '62';
      } else if (/^5/.test(os.release)) {
        name += '55';
      }
      return name;
    }
  }, {
    key: 'getElementaryOSVersionString',
    value: function getElementaryOSVersionString(os) {
      var name = 'ubuntu1404';
      return name;
    }
  }, {
    key: 'getSuseVersionString',
    value: function getSuseVersionString(os) {
      var _ref2 = os.release.match(/(^11|^12)/) || [null],
          _ref3 = (0, _slicedToArray3.default)(_ref2, 1),
          release = _ref3[0];

      if (release) {
        return 'suse' + release;
      } else {
        return '';
      }
    }
  }, {
    key: 'getUbuntuVersionString',
    value: function getUbuntuVersionString(os) {
      var name = 'ubuntu';
      var ubuntuVersion = os.release ? os.release.split('.') : '';
      var majorVersion = parseInt(ubuntuVersion[0]);

      if (os.release === '12.04') {
        name += '1204';
      } else if (os.release === '14.04') {
        name += '1404';
      } else if (os.release === '14.10') {
        name += '1410-clang';
      } else if (majorVersion === 14) {
        // default for major 14 to 1404
        name += '1404';
      } else if (os.release === '16.04') {
        name += '1604';
      } else if (majorVersion === 16) {
        // default for major 16 to 1604
        name += '1604';
      } else {
        // this needs to default to legacy release, this is a BUG
        name += '1404';
      }
      return name;
    }
  }, {
    key: 'translatePlatform',
    value: function translatePlatform(platform) {
      switch (platform) {
        case 'darwin':
          return 'osx';
        case 'win32':
          return 'win32';
        case 'linux':
          return 'linux';
        case 'elementary OS':
          // os.platform() doesn't return linux for elementary OS.
          return 'linux';
        case 'sunos':
          return 'sunos5';
        default:
          console.log('unsupported platform %s by MongoDB', platform);
          throw new Error('unsupported OS ' + platform);
      }
    }
  }, {
    key: 'translateArch',
    value: function translateArch(arch, mongoPlatform) {
      if (arch === 'ia32') {
        if (mongoPlatform === 'linux') {
          return 'i686';
        } else if (mongoPlatform === 'win32') {
          return 'i386';
        } else {
          console.log('unsupported mongo platform and os combination');
          throw new Error('unsupported architecture');
        }
      } else if (arch === 'x64') {
        return 'x86_64';
      } else {
        console.log('unsupported architecture');
        throw new Error('unsupported architecture, ia32 and x64 are the only valid options');
      }
    }
  }]);
  return MongoDBPlatform;
}();